#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import os
import glob
import shutil

sys.path.append(os.path.join("..", "..", "build"))
import build_lib
from build_lib import *

INSTALL_FILES = " ".join([
    "xnc_serialization.mli",
    "xnc_serialization.cmi",
    "xnc_property.mli",
    "xnc_property.cmi",
    "xnc_string_transform.mli",
    "xnc_string_transform.cmi",
    "xnc_formatting.mli",
    "xnc_formatting.cmi",
    "xnc_printing.mli",
    "xnc_printing.cmi",
    "xnc_writer.mli",
    "xnc_writer.cmi",
    "xnc_scanning.mli",
    "xnc_scanning.cmi",
    "xnc_token_eval.mli",
    "xnc_token_eval.cmi",
    "xnc_lexing.mli",
    "xnc_lexing.cmi",
    "xnc_reader.mli",
    "xnc_reader.cmi",
    "xnc_serializable.mli",
    "xnc_serializable.cmi",
    "xnc_pipe_conv.mli",
    "xnc_pipe_conv.cmi",
    "xnc_pipe_chain.mli",
    "xnc_pipe_chain.cmi",
    "xnc_serializing.mli",
    "xnc_serializing.cmi",
    "xnc_field_collection.mli",
    "xnc_field_collection.cmi",
    "serialization.cma",
    "serialization.cmxa",
    "serialization.a"
    ])

def print_help():
    print """This is the Ex-nunc build system, you are welcome.
Usage: build.py target [target2 target3...]

Available targets:

Cleaning targets:
  clean     - remove generated files

Compilation  targets:
  lib (default) - compiles serialization (bytecode, NO INSTALL)
  lib.opt       - compiles serialization (native and bytecode, NO INSTALL)
  install       - INSTALL serialization (native and bytecode)
  test          - builds and executes serialization test (native and bytecode)
  help          - show this help message"""

    sys.exit(0)

##########################
# THE SCRIPT BEGINS HERE #
##########################
if (find_list2(["--help", "-help", "-h", "help"], sys.argv)):
    print_help()

if (len(sys.argv) == 1):
    sys.argv.append("lib")

if not os.path.exists("myocamlbuild.ml"):
    shutil.copy("../../myocamlbuild.ml", ".")

for arg in sys.argv[1:]:
    if arg == "clean":
        run("ocamlbuild -clean")
        enter_dir_and_do("test", lambda : run("ocamlbuild -clean"))
        if os.path.exists("test/myocamlbuild.ml"):
            os.unlink("test/myocamlbuild.ml")
        if os.path.exists("myocamlbuild.ml"):
            os.unlink("myocamlbuild.ml")
    elif arg == "lib":
        run("ocamlbuild -I xnc_core serialization.cma")
    elif arg == "lib.opt":
        ret = run("ocamlbuild -I xnc_core " +
                expand_exts("serialization", ["cma", "cmxa", "a"]))
        if ret != 0:
            sys.exit(1)
    elif arg == "install":
        shutil.copy("META", "_build")
        enter_dir_and_do("_build",
                lambda : run("ocamlfind install xnc-serialization META " +
                    INSTALL_FILES))
    elif arg == "test":
        if not os.path.exists("test/myocamlbuild.ml"):
            shutil.copy("../../myocamlbuild.ml", "test/")
        def job():
            run("ocamlbuild test_suite.byte test_suite.native")
            run("./test_suite.byte")
            run("./test_suite.native")
        enter_dir_and_do("test", job)
    else:
        sys.stderr.write(
            "%s: unknown action \"%s\": try with --help\n" % (sys.argv[0], arg)
        )
        sys.exit(1)


# vim:set encoding=utf8:

