open OUnit

(* fixture setup and teardown *)
let set_up _ =
  new Xnc_formatting.standard_formatter ()
;;

let tear_down _ =
  ()
;;

(* tests *)
let test_format_null (formatter : _ Xnc_formatting.formatter) =
  OUnit.assert_equal
    ~printer:Std.identity
    ~msg:"Unexpected null serialization"
    "None"
    (formatter # format_null ());
;;
 
let test_format_int (formatter : _ Xnc_formatting.formatter) =
  OUnit.assert_equal
    ~printer:Std.identity
    ~msg:"Unexpected int serialization"
    "1"
    (formatter # format_int 1);
;;
 
let test_format_float (formatter : _ Xnc_formatting.formatter) =
  OUnit.assert_equal
    ~printer:Std.identity
    ~msg:"Unexpected float serialization"
    "3.141592"
    (formatter # format_float 3.141592);
  OUnit.assert_equal
    ~printer:Std.identity
    ~msg:"Unexpected float serialization"
    "3."
    (formatter # format_float 3.0);
  OUnit.assert_equal
    ~printer:Std.identity
    ~msg:"Unexpected float serialization"
    "0.035"
    (formatter # format_float 0.035);
;;

let test_format_char (formatter : _ Xnc_formatting.formatter) =
  OUnit.assert_equal
    ~printer:Std.identity
    ~msg:"Unexpected char serialization"
    "c"
    (formatter # format_char 'c');
;;

let test_format_bool (formatter : _ Xnc_formatting.formatter) =
  OUnit.assert_equal
    ~printer:Std.identity
    ~msg:"Unexpected bool serialization"
    "true"
    (formatter # format_bool true);
;;

let test_format_raw_string (formatter : _ Xnc_formatting.formatter) =
  OUnit.assert_equal
    ~printer:Std.identity
    ~msg:"Unexpected raw string serialization"
    "test"
    (formatter # format_raw_string "test");
;;

let test_format_string (formatter : _ Xnc_formatting.formatter) =
  let config = { Xnc_serialization.default_config
                   with Xnc_serialization.delimiter_char = '#' } in
  let formatter' = new Xnc_formatting.standard_formatter
                     ~config:config () in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected string serialization"
      "\"test\""
      (formatter # format_string "test");
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected string serialization"
      "#test#"
      (formatter' # format_string "test");
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected string serialization"
      "#test\\#escape\\##"
      (formatter' # format_string "test#escape#");
;;

let test_format_date (formatter : _ Xnc_formatting.formatter) =
  let date =
    { Netdate.year = 2006;
      Netdate.month = 10;
      Netdate.day = 22;
      Netdate.hour = 15;
      Netdate.minute = 47;
      Netdate.second = 5;
      Netdate.zone = 120;
      Netdate.week_day = 0 
    }
  in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected date serialization"
      "10/22/2006 15:47:05"
      (formatter # format_date date);
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected date serialization"
      "10/22/2006 01:02:05"
      (formatter # format_date { date
                                   with Netdate.hour = 1;
                                        Netdate.minute = 2 });
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected date serialization"
      "02/01/2006 15:47:05"
      (formatter # format_date { date
                                   with Netdate.day = 1;
                                        Netdate.month = 2 });
;;

(* test suite *)
let suite = "Formatting test" >:::
  ["test_format_null" >:: (bracket set_up test_format_null tear_down);
   "test_format_int" >:: (bracket set_up test_format_int tear_down);
   "test_format_float" >:: (bracket set_up test_format_float tear_down);
   "test_format_char" >:: (bracket set_up test_format_char tear_down);
   "test_format_bool" >:: (bracket set_up test_format_bool tear_down);
   "test_format_raw_string"
     >:: (bracket set_up test_format_raw_string tear_down);
   "test_format_string" >:: (bracket set_up test_format_string tear_down);
   "test_format_date" >:: (bracket set_up test_format_date tear_down)]
;;

