open OUnit

(* tests *)
let test_build_out_pipe _ =
  let id_pipe = new Netchannels.pipe () in
  let chain = Xnc_pipe_chain.create () in
  let _ = Xnc_pipe_chain.add chain id_pipe in
  let buf = Buffer.create 256 in
  let endpoint_ch = new Netchannels.output_buffer buf in
  let out_ch = Xnc_pipe_chain.build_output_filter chain endpoint_ch in
    out_ch # output_string "Pipe ";
    out_ch # output_string "chain ";
    out_ch # output_string "test!";
    Xnc_pipe_chain.close chain;
    endpoint_ch # close_out ();
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected endpoint contents"
      "Pipe chain test!"
      (Buffer.contents buf)
;;

let test_build_in_pipe _ =
  let id_pipe = new Netchannels.pipe () in
  let chain = Xnc_pipe_chain.create () in
  let _ = Xnc_pipe_chain.add chain id_pipe in
  let endpoint_ch = new Netchannels.input_string "Pipe chain test!" in
  let in_ch = Xnc_pipe_chain.build_input_filter chain endpoint_ch in
  let s = in_ch # input_line () in
    Xnc_pipe_chain.close chain;
    endpoint_ch # close_in ();
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected endpoint contents"
      "Pipe chain test!"
      s
;;

let test_base64_out_pipe _ =
  let id_pipe = new Netchannels.pipe () in
  let id_pipe' = new Netchannels.pipe () in
  let base64_pipe = new Netencoding.Base64.encoding_pipe () in
  let chain = Xnc_pipe_chain.create () in
  let _ = Xnc_pipe_chain.add chain id_pipe in
  let _ = Xnc_pipe_chain.add chain base64_pipe in
  let _ = Xnc_pipe_chain.add chain id_pipe' in
  let buf = Buffer.create 256 in
  let endpoint_ch = new Netchannels.output_buffer buf in
  let out_ch = Xnc_pipe_chain.build_output_filter chain endpoint_ch in
    out_ch # output_string "Pipe ";
    out_ch # output_string "chain ";
    out_ch # output_string "test!";
    Xnc_pipe_chain.close chain;
    endpoint_ch # close_out ();
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected endpoint contents"
      "UGlwZSBjaGFpbiB0ZXN0IQ=="
      (Buffer.contents buf)
;;

let test_base64_in_pipe _ =
  let id_pipe = new Netchannels.pipe () in
  let id_pipe' = new Netchannels.pipe () in
  let base64_pipe = new Netencoding.Base64.decoding_pipe () in
  let chain = Xnc_pipe_chain.create () in
  let _ = Xnc_pipe_chain.add chain id_pipe in
  let _ = Xnc_pipe_chain.add chain base64_pipe in
  let _ = Xnc_pipe_chain.add chain id_pipe' in
  let endpoint_ch = new Netchannels.input_string "UGlwZSBjaGFpbiB0ZXN0IQ==" in
  let in_ch = Xnc_pipe_chain.build_input_filter chain endpoint_ch in
  let s = in_ch # input_line () in
    Xnc_pipe_chain.close chain;
    endpoint_ch # close_in ();
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected endpoint contents"
      "Pipe chain test!"
      s
;;

let test_xor_out_pipe _ =
  let xor_pipe = new Netchannels.pipe
                   ~conv:(Xnc_pipe_conv.xor_conv "A") () in
  let xor_pipe' = new Netchannels.pipe
                    ~conv:(Xnc_pipe_conv.xor_conv "A") () in
  let chain = Xnc_pipe_chain.create () in
  let _ = Xnc_pipe_chain.add chain xor_pipe in
  let _ = Xnc_pipe_chain.add chain xor_pipe' in
  let buf = Buffer.create 256 in
  let endpoint_ch = new Netchannels.output_buffer buf in
  let out_ch = Xnc_pipe_chain.build_output_filter chain endpoint_ch in
    out_ch # output_string "pipe ";
    out_ch # output_string "chain ";
    out_ch # output_string "test!";
    Xnc_pipe_chain.close chain;
    endpoint_ch # close_out ();
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected endpoint contents"
      "pipe chain test!"
      (Buffer.contents buf)
;;

let test_xor_in_pipe _ =
  let xor_pipe = new Netchannels.pipe
                   ~conv:(Xnc_pipe_conv.xor_conv "A") () in
  let xor_pipe' = new Netchannels.pipe
                    ~conv:(Xnc_pipe_conv.xor_conv "A") () in
  let chain = Xnc_pipe_chain.create () in
  let _ = Xnc_pipe_chain.add chain xor_pipe in
  let _ = Xnc_pipe_chain.add chain xor_pipe' in
  let endpoint_ch = new Netchannels.input_string "pipe chain test!" in
  let in_ch = Xnc_pipe_chain.build_input_filter chain endpoint_ch in
  let s = in_ch # input_line () in
    Xnc_pipe_chain.close chain;
    endpoint_ch # close_in ();
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected endpoint contents"
      "pipe chain test!"
      s
;;

let test_xor_base64_xor_out_pipe _ =
  let xor_pipe = new Netchannels.pipe
                   ~conv:(Xnc_pipe_conv.xor_conv "\001") () in
  let xor_pipe' = new Netchannels.pipe
                    ~conv:(Xnc_pipe_conv.xor_conv "\001") () in
  let base64_pipe = new Netencoding.Base64.encoding_pipe () in
  let chain = Xnc_pipe_chain.create () in
  let _ = Xnc_pipe_chain.add chain xor_pipe in
  let _ = Xnc_pipe_chain.add chain base64_pipe in
  let _ = Xnc_pipe_chain.add chain xor_pipe' in
  let buf = Buffer.create 256 in
  let endpoint_ch = new Netchannels.output_buffer buf in
  let out_ch = Xnc_pipe_chain.build_output_filter chain endpoint_ch in
    out_ch # output_string "pipe ";
    out_ch # output_string "chain ";
    out_ch # output_string "test!";
    Xnc_pipe_chain.close chain;
    endpoint_ch # close_out ();
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected endpoint contents"
      "bViy[BGh`VCncxG0[IK0H@<<"
      (Buffer.contents buf)
;;

let test_xor_base64_xor_in_pipe _ =
  let xor_pipe = new Netchannels.pipe
                   ~conv:(Xnc_pipe_conv.xor_conv "\001") () in
  let xor_pipe' = new Netchannels.pipe
                    ~conv:(Xnc_pipe_conv.xor_conv "\001") () in
  let base64_pipe = new Netencoding.Base64.decoding_pipe () in
  let chain = Xnc_pipe_chain.create () in
  let _ = Xnc_pipe_chain.add chain xor_pipe in
  let _ = Xnc_pipe_chain.add chain base64_pipe in
  let _ = Xnc_pipe_chain.add chain xor_pipe' in
  let endpoint_ch = new Netchannels.input_string "bViy[BGh`VCncxG0[IK0H@<<" in
  let in_ch = Xnc_pipe_chain.build_input_filter chain endpoint_ch in
  let s = in_ch # input_line () in
    Xnc_pipe_chain.close chain;
    endpoint_ch # close_in ();
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected endpoint contents"
      "pipe chain test!"
      s
;;

let test_rebuild _ =
  let id_pipe = new Netchannels.pipe () in
  let chain = Xnc_pipe_chain.create () in
  let _ = Xnc_pipe_chain.add chain id_pipe in
  let in_endpoint = new Netchannels.input_string "Pipe chain test!" in
  let _ = Xnc_pipe_chain.build_input_filter chain in_endpoint in
  let buf = Buffer.create 256 in
  let out_endpoint = new Netchannels.output_buffer buf in
    OUnit.assert_raises
      (Failure "Cannot build more than a filter for a pipe chain")
      (fun () -> Xnc_pipe_chain.build_output_filter chain out_endpoint);
;;

let test_close_uninitialized _ =
  let id_pipe = new Netchannels.pipe () in
  let chain = Xnc_pipe_chain.create () in
  let _ = Xnc_pipe_chain.add chain id_pipe in
    OUnit.assert_raises
      (Failure "Cannot close a pipe chain before building a filter")
      (fun () -> Xnc_pipe_chain.close chain);
;;

let test_with_output_filter _ =
  let buf = Buffer.create 256 in
  let endpoint_ch = new Netchannels.output_buffer buf in
  let xor_pipe = new Netchannels.pipe
                   ~conv:(Xnc_pipe_conv.xor_conv "C") () in
  let xor_pipe' = new Netchannels.pipe
                    ~conv:(Xnc_pipe_conv.xor_conv "C") () in
  let _ =
    Xnc_pipe_chain.with_output_filter
      [ xor_pipe; xor_pipe' ]
      endpoint_ch
      (fun out_ch ->
         out_ch # output_string "Pipe ";
         out_ch # output_string "chain ";
         out_ch # output_string "test!")
  in
    endpoint_ch # close_out ();
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected endpoint contents"
      "Pipe chain test!"
      (Buffer.contents buf)
;;

let test_with_input_filter _ =
  let endpoint_ch = new Netchannels.input_string "Pipe chain test!" in
  let id_pipe = new Netchannels.pipe () in
  let s =
    Xnc_pipe_chain.with_input_filter
      [ id_pipe ]
      endpoint_ch
      (fun in_ch -> in_ch # input_line ())
  in
    endpoint_ch # close_in ();
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected endpoint contents"
      "Pipe chain test!"
      s
;;

(* test suite *)
let suite = "Pipe chain test" >:::
  ["test_build_out_pipe" >:: test_build_out_pipe;
   "test_build_in_pipe" >:: test_build_in_pipe;
   "test_base64_out_pipe" >:: test_base64_out_pipe;
   "test_base64_in_pipe" >:: test_base64_in_pipe;
   "test_xor_out_pipe" >:: test_xor_out_pipe;
   "test_xor_in_pipe" >:: test_xor_in_pipe;
   "test_xor_base64_xor_out_pipe" >:: test_xor_base64_xor_out_pipe;
   "test_xor_base64_xor_in_pipe" >:: test_xor_base64_xor_in_pipe;
   "test_rebuild" >:: test_rebuild;
   "test_close_uninitialized" >:: test_close_uninitialized;
   "test_with_output_filter" >:: test_with_output_filter;
   "test_with_input_filter" >:: test_with_input_filter]
;;

