open OUnit

(* tests *)
let test_get_node_type _ =
  let element = Xnc_property.from_list
                  [Xnc_property.element; Xnc_property.Length 10] in
  let attribute = Xnc_property.singleton Xnc_property.attribute in
  let text = Xnc_property.from_list
               [Xnc_property.Precision 2; Xnc_property.text] in
  let cdata = Xnc_property.singleton Xnc_property.cdata in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected node type (should be Element)"
      (Xnc_property.Element)
      (Xnc_property.get_node_type element);
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected node type (should be Attribute)"
      (Xnc_property.Attribute)
      (Xnc_property.get_node_type attribute);
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected node type (should be Text)"
      (Xnc_property.Text)
      (Xnc_property.get_node_type text);
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected node type (should be Cdata)"
      (Xnc_property.Cdata)
      (Xnc_property.get_node_type cdata);
;;

let test_default_node_type _ =
  OUnit.assert_equal
    ~printer:Std.dump
    ~msg:"Unexpected node type (should be Element)"
    (Xnc_property.Element)
    (Xnc_property.get_node_type Xnc_property.empty);
;;

let test_is_element _ =
  OUnit.assert_equal
    ~printer:Std.dump
    ~msg:"Element property unexpectedly not found"
    true
    (Xnc_property.is_element
       (Xnc_property.from_list
          [Xnc_property.Length 10; Xnc_property.element]));
  OUnit.assert_equal
    ~printer:Std.dump
    ~msg:"Element property unexpectedly found"
    false
    (Xnc_property.is_element Xnc_property.empty);
  OUnit.assert_equal
    ~printer:Std.dump
    ~msg:"Element property unexpectedly found"
    false
    (Xnc_property.is_element
       (Xnc_property.from_list
          [Xnc_property.Length 10; Xnc_property.attribute]));
;;

let test_is_attribute _ =
  OUnit.assert_equal
    ~printer:Std.dump
    ~msg:"Attribute property unexpectedly not found"
    true
    (Xnc_property.is_attribute
       (Xnc_property.from_list
          [Xnc_property.Length 10; Xnc_property.attribute]));
  OUnit.assert_equal
    ~printer:Std.dump
    ~msg:"Attribute property unexpectedly found"
    false
    (Xnc_property.is_attribute Xnc_property.empty);
  OUnit.assert_equal
    ~printer:Std.dump
    ~msg:"Attribute property unexpectedly found"
    false
    (Xnc_property.is_attribute
       (Xnc_property.from_list
          [Xnc_property.Length 10; Xnc_property.element]));
;;

let test_is_text _ =
  OUnit.assert_equal
    ~printer:Std.dump
    ~msg:"Text property unexpectedly not found"
    true
    (Xnc_property.is_text
       (Xnc_property.from_list
          [Xnc_property.Length 10; Xnc_property.text]));
  OUnit.assert_equal
    ~printer:Std.dump
    ~msg:"Text property unexpectedly found"
    false
    (Xnc_property.is_text Xnc_property.empty);
  OUnit.assert_equal
    ~printer:Std.dump
    ~msg:"Text property unexpectedly found"
    false
    (Xnc_property.is_text
       (Xnc_property.from_list
          [Xnc_property.Length 10; Xnc_property.element]));
;;

let test_is_cdata _ =
  OUnit.assert_equal
    ~printer:Std.dump
    ~msg:"Cdata property unexpectedly not found"
    true
    (Xnc_property.is_cdata
       (Xnc_property.from_list
          [Xnc_property.Length 10; Xnc_property.cdata]));
  OUnit.assert_equal
    ~printer:Std.dump
    ~msg:"Cdata property unexpectedly found"
    false
    (Xnc_property.is_cdata Xnc_property.empty);
  OUnit.assert_equal
    ~printer:Std.dump
    ~msg:"Cdata property unexpectedly found"
    false
    (Xnc_property.is_cdata
       (Xnc_property.from_list
          [Xnc_property.Length 10; Xnc_property.element]));
;;

let test_get_length _ =
  OUnit.assert_equal
    ~printer:string_of_int
    ~msg:"Unexpected length"
    10
    (Xnc_property.get_length
       (Xnc_property.from_list
          [Xnc_property.cdata; Xnc_property.Length 10])
       0);
  OUnit.assert_equal
    ~printer:string_of_int
    ~msg:"Unexpected length"
    0
    (Xnc_property.get_length Xnc_property.empty 0);
  OUnit.assert_equal
    ~printer:string_of_int
    ~msg:"Unexpected length"
    100
    (Xnc_property.get_length
       (Xnc_property.from_list
          [Xnc_property.Precision 2; Xnc_property.element])
       100);
;;

let test_get_precision _ =
  OUnit.assert_equal
    ~printer:string_of_int
    ~msg:"Unexpected precision"
    2
    (Xnc_property.get_precision
       (Xnc_property.from_list
          [Xnc_property.Precision 2; Xnc_property.element])
       0);
  OUnit.assert_equal
    ~printer:string_of_int
    ~msg:"Unexpected precision"
    0
    (Xnc_property.get_precision Xnc_property.empty 0);
  OUnit.assert_equal
    ~printer:string_of_int
    ~msg:"Unexpected precision"
    1
    (Xnc_property.get_precision
       (Xnc_property.from_list
          [Xnc_property.cdata; Xnc_property.Length 10])
       1);
;;

let test_equal _ =
  let p1 = Xnc_property.from_list [Xnc_property.element;
                                   Xnc_property.Length 10;
                                   Xnc_property.Precision 2] in
  let p2 = Xnc_property.from_list [Xnc_property.Precision 2;
                                   Xnc_property.element;
                                   Xnc_property.Length 10] in
  let p3 = Xnc_property.from_list [Xnc_property.element;
                                   Xnc_property.Length 10] in
  let p4 = Xnc_property.from_list [Xnc_property.Length 11;
                                   Xnc_property.element] in
  let p5 = Xnc_property.from_list [Xnc_property.Length 10;
                                   Xnc_property.element;
                                   Xnc_property.Precision 2] in
  OUnit.assert_bool
    "p1 = p1 failed"
    (Xnc_property.equal p1 p1);
  OUnit.assert_bool
    "p1 = p2 failed"
    (Xnc_property.equal p1 p2);
  OUnit.assert_bool
    "p2 = p1 failed"
    (Xnc_property.equal p2 p1);
  OUnit.assert_bool
    "p1 != p2 failed"
    (not (Xnc_property.equal p1 p3));
  OUnit.assert_bool
    "p3 != p4 failed"
    (not (Xnc_property.equal p3 p4));
  OUnit.assert_bool
    "p2 = p5 failed"
    (Xnc_property.equal p2 p5);
  OUnit.assert_bool
    "p1 = p5 failed"
    (Xnc_property.equal p1 p5);
;;

let test_subset _ =
  let p1 = Xnc_property.empty in
  let p2 = Xnc_property.from_list [Xnc_property.Precision 2;
                                   Xnc_property.element;
                                   Xnc_property.Length 10] in
  let p3 = Xnc_property.from_list [Xnc_property.Length 10;
                                   Xnc_property.element] in
  let p4 = Xnc_property.from_list [Xnc_property.Length 11;
                                   Xnc_property.element] in
  let p5 = Xnc_property.from_list [Xnc_property.Length 10;
                                   Xnc_property.element;
                                   Xnc_property.Precision 2] in
  OUnit.assert_bool
    "p1 should be a subset of p1"
    (Xnc_property.subset p1 p1);
  OUnit.assert_bool
    "p1 should be a subset of p2"
    (Xnc_property.subset p1 p2);
  OUnit.assert_bool
    "p3 should be a subset of p2"
    (Xnc_property.subset p3 p2);
  OUnit.assert_bool
    "p2 should not be a subset of p1"
    (not (Xnc_property.subset p2 p1));
  OUnit.assert_bool
    "p3 should not be a subset p4"
    (not (Xnc_property.subset p3 p4));
  OUnit.assert_bool
    "p3 should be a subset of p5"
    (Xnc_property.subset p3 p5);
  OUnit.assert_bool
    "p1 should be a subset of p5"
    (Xnc_property.subset p1 p5);
;;

(* test suite *)
let suite = "Property test" >:::
  ["test_get_node_type" >:: test_get_node_type;
   "test_default_node_type" >:: test_default_node_type;
   "test_is_element" >:: test_is_element;
   "test_is_attribute" >:: test_is_attribute;
   "test_is_text" >:: test_is_text;
   "test_is_cdata" >:: test_is_cdata;
   "test_get_length" >:: test_get_length;
   "test_get_precision" >:: test_get_precision;
   "test_equal" >:: test_equal;
   "test_subset" >:: test_subset]
;;

