open OUnit

(* tests *)
let test_peek_char _ =
  let scanbuf = Xnc_scanning.Scanning_buffer.create_from_string "1" in
  let empty_scanbuf = Xnc_scanning.Scanning_buffer.create_from_string "" in
    OUnit.assert_equal
      ~printer:Std.string_of_char
      ~msg:"Unexpected first char peeked (1st time)"
      '1'
      (Xnc_scanning.Scanning_buffer.peek_char scanbuf);
    OUnit.assert_equal
      ~printer:Std.string_of_char
      ~msg:"Unexpected first char peeked (2nd time)"
      '1'
      (Xnc_scanning.Scanning_buffer.peek_char scanbuf);
    OUnit.assert_equal
      ~printer:Std.string_of_char
      ~msg:"Unexpected char peeked from an empty buffer"
      '\000'
      (Xnc_scanning.Scanning_buffer.peek_char empty_scanbuf);
;;
 
let test_next_char _ =
  let scanbuf = Xnc_scanning.Scanning_buffer.create_from_string "12" in
    OUnit.assert_equal
      ~printer:Std.string_of_char
      ~msg:"Unexpected first char"
      '1'
      (Xnc_scanning.Scanning_buffer.next_char scanbuf);
    OUnit.assert_equal
      ~printer:Std.string_of_char
      ~msg:"Unexpected second char"
      '2'
      (Xnc_scanning.Scanning_buffer.next_char scanbuf);
    OUnit.assert_equal
      ~printer:Std.string_of_char
      ~msg:"Unexpected third char"
      '\000'
      (Xnc_scanning.Scanning_buffer.next_char scanbuf);
;;
 
let test_peek_string _ =
  let scanbuf = Xnc_scanning.Scanning_buffer.create_from_string "123ab" in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected string"
      "123"
      (Xnc_scanning.Scanning_buffer.peek_string scanbuf 3);
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected string"
      ""
      (Xnc_scanning.Scanning_buffer.peek_string scanbuf 6);
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected string"
      ""
      (Xnc_scanning.Scanning_buffer.peek_string scanbuf 0);
;;
 
let test_next_string _ =
  let scanbuf = Xnc_scanning.Scanning_buffer.create_from_string "123ab" in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected string"
      "123"
      (Xnc_scanning.Scanning_buffer.next_string scanbuf 3);
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected string"
      "ab"
      (Xnc_scanning.Scanning_buffer.next_string scanbuf 2);
    OUnit.assert_raises
      (Xnc_scanning.Scan_failure
         "bad input at line 1, char 6: \
          '' (unscanned buffer: \"\", EOF: true), \
          expecting 2-character long string")
      (fun () -> Xnc_scanning.Scanning_buffer.next_string scanbuf 2);
;;
 
let test_peek_literal_string _ =
  let scanbuf = Xnc_scanning.Scanning_buffer.create_from_string "123ab" in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected literal string \"123\""
      "123"
      (Xnc_scanning.Scanning_buffer.peek_literal_string scanbuf "123");
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected literal string \"ab\""
      ""
      (Xnc_scanning.Scanning_buffer.peek_literal_string scanbuf "ab");
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected literal string \"123abc\""
      ""
      (Xnc_scanning.Scanning_buffer.peek_literal_string scanbuf "123abc");
;;
 
let test_next_literal_string _ =
  let scanbuf = Xnc_scanning.Scanning_buffer.create_from_string "123ab" in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected literal string \"123\""
      "123"
      (Xnc_scanning.Scanning_buffer.next_literal_string scanbuf "123");
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected literal string \"ab\""
      "ab"
      (Xnc_scanning.Scanning_buffer.next_literal_string scanbuf "ab");
    OUnit.assert_raises
      (Xnc_scanning.Scan_failure
         "bad input at line 1, char 6: \
          '' (unscanned buffer: \"\", EOF: true), \
          expecting \"abc\"")
      (fun () -> Xnc_scanning.Scanning_buffer.next_literal_string scanbuf "abc");
;;
 
let test_peek_matching_string _ =
  let scanbuf = Xnc_scanning.Scanning_buffer.create_from_string "123ab" in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected string matching pattern [0-9]+"
      "123"
      (Xnc_scanning.Scanning_buffer.peek_matching_string scanbuf
         (Str.regexp "[0-9]+"));
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected string matching pattern [0-9]+"
      "2"
      (Xnc_scanning.Scanning_buffer.peek_matching_string
         ~get_match:(Str.matched_group 1)
         scanbuf
         (Str.regexp "[0-9]\\([0-9]\\)[0-9]"));
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected string matching pattern [a-z]+"
      ""
      (Xnc_scanning.Scanning_buffer.peek_matching_string scanbuf
         (Str.regexp "[a-z]+"));
;;
 
let test_next_matching_string _ =
  let scanbuf = Xnc_scanning.Scanning_buffer.create_from_string "123ab" in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected string matching pattern [0-9]+"
      "123"
      (Xnc_scanning.Scanning_buffer.next_matching_string scanbuf
         (Str.regexp "[0-9]+"));
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected string matching pattern [a-z]+"
      "ab"
      (Xnc_scanning.Scanning_buffer.next_matching_string scanbuf
         (Str.regexp "[a-z]+"));
    OUnit.assert_raises
      (Xnc_scanning.Scan_failure
         "bad input at line 1, char 6: \
          '' (unscanned buffer: \"\", EOF: true), \
          pattern not matched")
      (fun () -> Xnc_scanning.Scanning_buffer.next_matching_string scanbuf
                   (Str.regexp "a"));
;;
 
let test_skip _ =
  let scanbuf = Xnc_scanning.Scanning_buffer.create_from_string "123ab" in
    Xnc_scanning.Scanning_buffer.skip scanbuf 3;
    OUnit.assert_equal
      ~printer:Std.string_of_char
      ~msg:"Unexpected third char peeked"
      'a'
      (Xnc_scanning.Scanning_buffer.peek_char scanbuf);
    Xnc_scanning.Scanning_buffer.skip scanbuf 1;
    OUnit.assert_equal
      ~printer:Std.string_of_char
      ~msg:"Unexpected fifth char peeked"
      'b'
      (Xnc_scanning.Scanning_buffer.peek_char scanbuf);
    OUnit.assert_raises
      (Xnc_scanning.Scan_failure
         "bad input at line 1, char 5: \
          'b' (unscanned buffer: \"b\", EOF: false), \
          expecting 2-character long string")
      (fun () -> Xnc_scanning.Scanning_buffer.skip scanbuf 2);
;;
 
(* test suite *)
let suite = "Scanning buffer test" >:::
  ["test_peek_char" >:: test_peek_char;
   "test_next_char" >:: test_next_char;
   "test_peek_string" >:: test_peek_string;
   "test_next_string" >:: test_next_string;
   "test_peek_literal_string" >:: test_peek_literal_string;
   "test_next_literal_string" >:: test_next_literal_string;
   "test_peek_matching_string" >:: test_peek_matching_string;
   "test_next_matching_string" >:: test_next_matching_string;
   "test_skip" >:: test_skip]
;;

