open OUnit

(* fixture setup and teardown *)
let set_up _ =
  new Xnc_token_eval.standard_evaluator ()
;;

let tear_down _ =
  ()
;;

(* tests *)
let test_get_int (evaluator : _ Xnc_token_eval.token_evaluator) =
  OUnit.assert_equal
    ~printer:string_of_int
    ~msg:"Unexpected int value"
    1
    (evaluator # get_int "1");
  OUnit.assert_raises
    (Xnc_token_eval.Token_evaluation_failed
       "error trying to convert \"abc\" to int")
    (fun () -> evaluator # get_int "abc");
;;
 
let test_get_float (evaluator : _ Xnc_token_eval.token_evaluator) =
  OUnit.assert_equal
    ~printer:string_of_float
    ~msg:"Unexpected float value"
    3.1416
    (evaluator # get_float "3.1416");
  OUnit.assert_equal
    ~printer:string_of_float
    ~msg:"Unexpected float value"
    3.
    (evaluator # get_float "3.0");
  OUnit.assert_equal
    ~printer:string_of_float
    ~msg:"Unexpected float value"
    0.035
    (evaluator # get_float "3.5e-2");
  OUnit.assert_raises
    (Xnc_token_eval.Token_evaluation_failed
       "error trying to convert \"xyz\" to float")
    (fun () -> evaluator # get_float "xyz");
;;

let test_get_char (evaluator : _ Xnc_token_eval.token_evaluator) =
  OUnit.assert_equal
    ~printer:Std.string_of_char
    ~msg:"Unexpected char value"
    's'
    (evaluator # get_char "s");
  OUnit.assert_raises
    (Xnc_token_eval.Token_evaluation_failed
       "error trying to convert \"\" to char")
    (fun () -> evaluator # get_char "")
;;

let test_get_bool (evaluator : _ Xnc_token_eval.token_evaluator) =
  OUnit.assert_equal
    ~printer:string_of_bool
    ~msg:"Unexpected bool value"
    true
    (evaluator # get_bool "true");
  OUnit.assert_equal
    ~printer:string_of_bool
    ~msg:"Unexpected bool value"
    false
    (evaluator # get_bool "false");
  OUnit.assert_raises
    (Xnc_token_eval.Token_evaluation_failed
       "error trying to convert \"\" to bool")
    (fun () -> evaluator # get_bool "")
;;

let test_get_string (evaluator : _ Xnc_token_eval.token_evaluator) =
  OUnit.assert_equal
    ~printer:Std.identity
    ~msg:"Exact string not matched"
    "test"
    (evaluator # get_string "test")
;;

let test_get_date (evaluator : _ Xnc_token_eval.token_evaluator) =
  OUnit.assert_equal
    ~printer:(Netdate.format ~fmt:"%D %H")
    ~msg:"Unexpected date value"
    { Netdate.year = 2006;
      Netdate.month = 10;
      Netdate.day = 22;
      Netdate.hour = 16;
      Netdate.minute = 46;
      Netdate.second = 30;
      Netdate.zone = 0;
      Netdate.week_day = -1
    }
    (evaluator # get_date "10/22/2006 16:46:30")
;;

(* test suite *)
let suite = "Token evaluator test" >:::
  ["test_get_int" >:: (bracket set_up test_get_int tear_down);
   "test_get_float" >:: (bracket set_up test_get_float tear_down);
   "test_get_char" >:: (bracket set_up test_get_char tear_down);
   "test_get_bool" >:: (bracket set_up test_get_bool tear_down);
   "test_get_string" >:: (bracket set_up test_get_string tear_down);
   "test_get_date" >:: (bracket set_up test_get_date tear_down)]
;;

