class type ['a] formatter =
object
  method format_null :
    ?properties:Xnc_property.properties ->
    unit -> 'a
  method format_int :
    ?properties:Xnc_property.properties ->
    int -> 'a
  method format_float :
    ?properties:Xnc_property.properties ->
    float -> 'a
  method format_char :
    ?properties:Xnc_property.properties ->
    char -> 'a
  method format_bool :
    ?properties:Xnc_property.properties ->
    bool -> 'a
  method format_raw_string :
    ?properties:Xnc_property.properties ->
    string -> 'a
  method format_string :
    ?properties:Xnc_property.properties ->
    string -> 'a
  method format_date :
    ?properties:Xnc_property.properties ->
    Netdate.t -> 'a
end

(* Constants *)
let date_format = "%m/%d/%Y %T";;

class virtual basic_formatter () : [string] formatter =
object(self)
  method format_null ?properties () =
    ""

  method format_int ?properties i =
    string_of_int i

  method format_float ?properties f =
    string_of_float f

  method format_char ?properties c =
    Std.string_of_char c

  method format_bool ?properties b =
    string_of_bool b

  method format_raw_string ?properties s =
    s

  method format_string ?properties s =
    s

  method format_date ?properties d =
    Netdate.format ~fmt:date_format d

end;;

class standard_formatter
  ?(config = Xnc_serialization.default_config) () : [string] formatter =

(* Class variables *)
let delimiter_string =
  Std.string_of_char config.Xnc_serialization.delimiter_char in
let escape_string =
  Std.string_of_char config.Xnc_serialization.escape_char in
let escape s =
  let delimiter_regexp = Str.regexp (Str.quote delimiter_string) in
  let escaped_delimiter = escape_string ^ delimiter_string in
    Str.global_replace delimiter_regexp escaped_delimiter s in

(* Class definition *)
object(self)
  inherit basic_formatter ()

  val config = config

  method format_null ?properties () =
    config.Xnc_serialization.none_string

  method format_string ?properties s =
    let escaped = escape (self # format_raw_string s) in
      delimiter_string ^ escaped ^ delimiter_string

end;;

class xml_formatter () : [string] formatter =
object(self)
  inherit basic_formatter ()

  method format_string ?properties s =
    Xnc_string_transform.escape_xml s

end;;

