type filter_list_type =
    Uninitialized
  | Input of Netchannels.input_filter list
  | Output of Netchannels.output_filter list

type t =
    { mutable pipe_list : Netchannels.pipe list;
      mutable filter_list : filter_list_type;
    }

let create () =
  { pipe_list = [];
    filter_list = Uninitialized;
  }
;;

let add c pipe =
  c.pipe_list <- pipe :: c.pipe_list;
;;

let build_output_filter c out_endpoint =
  match c.filter_list with
      Uninitialized ->
        List.fold_left
          (fun out_ch pipe ->
             let out_filter_list =
               match c.filter_list with
                   Uninitialized -> []
                 | Output l -> l
                 | _ -> assert false in
             let out_filter = new Netchannels.output_filter pipe out_ch in
               c.filter_list <- Output (out_filter :: out_filter_list);
               out_filter
          )
          out_endpoint
          c.pipe_list
    | _ ->
        failwith "Cannot build more than a filter for a pipe chain"
;;

let build_input_filter c in_endpoint =
  match c.filter_list with
      Uninitialized ->
        List.fold_left
          (fun in_ch pipe ->
             let in_filter_list =
               match c.filter_list with
                   Uninitialized -> []
                 | Input l -> l
                 | _ -> assert false in
             let in_filter = new Netchannels.input_filter in_ch pipe in
               c.filter_list <- Input (in_filter :: in_filter_list);
               in_filter
          )
          in_endpoint
          c.pipe_list
    | _ ->
        failwith "Cannot rebuild a pipe chain"
;;

let close c =
  let close_in_filter filter =
    filter # close_in () in
  let close_out_filter filter =
    filter # close_out () in
  let close_filter_list l close =
    List.iter
      (fun filter ->
         try
           close filter
         with Netchannels.Closed_channel ->
           ()
      )
      l
  in
    match c.filter_list with
        Uninitialized ->
          failwith "Cannot close a pipe chain before building a filter"
      | Input l -> close_filter_list l close_in_filter
      | Output l -> close_filter_list l close_out_filter
;;

let with_output_filter pipe_list endpoint_ch write =
  let chain = create () in
  let _ = chain.pipe_list <- (List.rev pipe_list) in
  let out_ch = build_output_filter chain endpoint_ch in
    try
      let result = write out_ch in
        close chain;
        result
    with e ->
      close chain;
      raise e
;;

let with_input_filter pipe_list endpoint_ch read =
  let chain = create () in
  let _ = chain.pipe_list <- (List.rev pipe_list) in
  let in_ch = build_input_filter chain endpoint_ch in
    try
      let result = read in_ch in
        close chain;
        result
    with e ->
      close chain;
      raise e
;;

