exception Invalid_digest

let convert transform incoming incoming_eof outgoing =
  let in_buf = Netbuffer.unsafe_buffer incoming in
  let in_pos = 0 in
  let in_len = Netbuffer.length incoming in
  let in_string = String.sub in_buf in_pos in_len in
  let out_string = transform in_string in
  let out_n = String.length out_string in
    ignore(
      Netbuffer.add_inplace
        ~len:out_n
        outgoing
        (fun out_buf out_pos out_len ->
           String.blit out_string 0 out_buf out_pos out_n;
           Netbuffer.clear incoming;
           out_n
        )
    )
;;

let xor_conv k incoming incoming_eof outgoing =
  convert
    (Xnc_string_transform.xor_transform k)
    incoming incoming_eof outgoing
;;

let escape_xml_conv incoming incoming_eof outgoing =
  convert
    Xnc_string_transform.escape_xml
    incoming incoming_eof outgoing
;;

let unescape_xml_conv incoming incoming_eof outgoing =
  convert
    Xnc_string_transform.unescape_xml
    incoming incoming_eof outgoing
;;

let cryptokit_conv transform incoming incoming_eof outgoing =
  let in_buf = Netbuffer.unsafe_buffer incoming in
  let in_len = Netbuffer.length incoming in
    transform # put_substring in_buf 0 in_len;
    if incoming_eof then
      begin
        let _ = transform # finish in
        let (t_buf, t_pos, t_len) = transform # get_substring in
        let _ =
          Netbuffer.add_inplace
            ~len:t_len
            outgoing
            (fun out_buf out_pos out_len ->
               String.blit t_buf t_pos out_buf out_pos t_len;
               t_len
            )
        in
          transform # wipe
      end
    else
      ();
    Netbuffer.clear incoming;
;;

let digest_conv hash incoming incoming_eof outgoing =
  let in_buf = Netbuffer.unsafe_buffer incoming in
  let in_len = Netbuffer.length incoming in
    if incoming_eof then
      begin
        let _ = hash # add_substring in_buf 0 in_len in
        let digest = hash # result in
        let digest_len = hash # hash_size in
        let len = in_len + digest_len in
          ignore
            (Netbuffer.add_inplace
               ~len
               outgoing
               (fun out_buf out_pos out_len ->
                  String.blit digest 0
                    out_buf out_pos
                    digest_len;
                  Netbuffer.blit incoming 0
                    out_buf (out_pos + digest_len)
                    in_len;
                  Netbuffer.clear incoming;
                  len
               ))
      end
    else
      ()
;;

let verify_digest_conv hash incoming incoming_eof outgoing =
  let in_buf = Netbuffer.unsafe_buffer incoming in
  let in_len = Netbuffer.length incoming in
    if incoming_eof then
      begin
        let digest_len = hash # hash_size in
        let digest = String.sub in_buf 0 digest_len in
        let len = in_len - digest_len in
        let _ = hash # add_substring in_buf digest_len len in
        let digest' = hash # result in
          if digest = digest' then
            ignore
              (Netbuffer.add_inplace
                 ~len
                 outgoing
                 (fun out_buf out_pos out_len ->
                    Netbuffer.blit incoming digest_len out_buf out_pos len;
                    Netbuffer.clear incoming;
                    len
               ))
          else
            raise Invalid_digest
      end
    else
      ()
;;

