(** Pipe conversions
  *
  * This module provides functions to build a conversion pipe (an instance of
  * [Netchannels.pipe]).
  *)

exception Invalid_digest
(** Raised if the digest is not valid. *)

val convert : (string -> string) -> Netbuffer.t -> bool -> Netbuffer.t -> unit
(** [convert f i eof o] is a generic conversion function that applies [f] to the
  * pipe input. *)

val xor_conv : string -> Netbuffer.t -> 'a -> Netbuffer.t -> unit
(** [xor_conv k i eof o] applies {!val: Xnc_string_transform.xor_transform} to
  * the pipe input. *)

val escape_xml_conv : Netbuffer.t -> 'a -> Netbuffer.t -> unit
(** [escape_xml_conv k i eof o] applies {!val: Xnc_string_transform.escape_xml}
  * to the pipe input. *)

val unescape_xml_conv : Netbuffer.t -> 'a -> Netbuffer.t -> unit
(** [unescape_xml_conv k i eof o] applies
  * {!val: Xnc_string_transform.unescape_xml} to the pipe input. *)

val cryptokit_conv :
  Cryptokit.transform -> Netbuffer.t -> bool -> Netbuffer.t -> unit
(** [cryptokit_conv t i eof o] applies the Cryptokit transform [t] to the pipe
  * input.
  *)

val digest_conv :
  Cryptokit.hash -> Netbuffer.t -> bool -> Netbuffer.t -> unit
(** [digest_conv h i eof o] applies the Cryptokit hash [h] to the pipe input.
  *)

val verify_digest_conv :
  Cryptokit.hash -> Netbuffer.t -> bool -> Netbuffer.t -> unit
(** [verify_digest_conv h i eof o] verifies that the pipe input starts with a
  * valid digest computed by [h]. Raises [Invalid_digest] if the digest found in
  * pipe input is different from the recomputed one. *)

