type node_type =
    Element
  | Attribute
  | Text
  | Cdata
;;

type property =
    Node_type of node_type
  | Length of int
  | Precision of int
;;

module OrderedProperty =
struct
  type t = property
  let compare = compare
end;;

module PropertySet = Set.Make(OrderedProperty);;

type properties = PropertySet.t;;

let empty =
  PropertySet.empty
;;

let singleton =
  PropertySet.singleton
;;

let from_list pl =
  List.fold_left
    (fun s e ->
       PropertySet.add e s)
    empty
    pl
;;

let element = Node_type Element;;

let attribute = Node_type Attribute;;

let text = Node_type Text;;

let cdata = Node_type Cdata;;

let get_node_type p =
  PropertySet.fold
    (fun e acc ->
       match e with
           Node_type n -> n
         | _ -> acc)
    p
    Element
;;

let is_element p =
  PropertySet.mem (Node_type Element) p
;;

let is_attribute p =
  PropertySet.mem (Node_type Attribute) p
;;

let is_text p =
  PropertySet.mem (Node_type Text) p
;;

let is_cdata p =
  PropertySet.mem (Node_type Cdata) p
;;

let get_length p default =
  PropertySet.fold
    (fun e acc ->
       match e with
           Length l -> l
         | _ -> acc)
    p
    default
;;

let get_precision p default =
  PropertySet.fold
    (fun e acc ->
       match e with
           Precision p -> p
         | _ -> acc)
    p
    default
;;

let equal =
  PropertySet.equal
;;

let subset =
  PropertySet.subset
;;

