class type ['a] serializer =
object
  method serialize :
    'a Xnc_serializable.field -> 'a
  method deserialize :
    'a Xnc_serializable.field -> 'a -> unit
  method to_netchannel :
    'a Xnc_serializable.field -> Netchannels.out_obj_channel -> unit
  method from_netchannel :
    'a Xnc_serializable.field -> Netchannels.in_obj_channel -> unit

end;;

class virtual netchannel_serializer ?(buf_size = 256) () =
object(self)
  method serialize serializable_field =
    let buf = Buffer.create buf_size in
    let endpoint_ch = new Netchannels.output_buffer buf in
      self # to_netchannel serializable_field endpoint_ch;
      endpoint_ch # close_out ();
      Buffer.contents buf

  method deserialize prototype serialization =
    let endpoint_ch = new Netchannels.input_string serialization in
      self # from_netchannel prototype endpoint_ch;
      try
        endpoint_ch # close_in ()
      with Netchannels.Closed_channel -> ()

  method virtual to_netchannel :
    string Xnc_serializable.field -> Netchannels.out_obj_channel -> unit

  method virtual from_netchannel :
    string Xnc_serializable.field -> Netchannels.in_obj_channel -> unit

end;;

class null_serializer () : [string] serializer =
object
  method serialize serializable_field =
    ""

  method deserialize prototype serialization =
    ()

  method to_netchannel serializable_field out_ch =
    ()

  method from_netchannel prototype in_ch =
    ()

end;;

class simple_serializer
  ?(config = Xnc_serialization.default_config) () : [string] serializer =
object(self)
  inherit netchannel_serializer ()

  method to_netchannel serializable_field out_ch =
    let escape_xml_pipe =
      new Netchannels.pipe ~conv:(Xnc_pipe_conv.escape_xml_conv) ()
    in
      Xnc_pipe_chain.with_output_filter
        [ escape_xml_pipe ]
        out_ch
        (fun out_ch ->
           let writer =
             Xnc_writer.create_from_netchannel ~config out_ch
           in
             Xnc_serializable.write writer serializable_field;
             Xnc_writer.flush writer)

  method from_netchannel prototype in_ch =
    let unescape_xml_pipe =
      new Netchannels.pipe ~conv:(Xnc_pipe_conv.unescape_xml_conv) ()
    in
      Xnc_pipe_chain.with_input_filter
        [ unescape_xml_pipe ]
        in_ch
        (fun in_ch ->
           let reader = Xnc_reader.create_from_netchannel ~config in_ch in
             Xnc_serializable.read reader prototype)

end;;

class xml_serializer
  ?(config = Pxp_types.default_config) () : [string] serializer =
object(self)
  inherit netchannel_serializer ()

  method to_netchannel serializable_field out_ch =
    let formatter = new Xnc_formatting.xml_formatter () in
    let printer =
      new Xnc_printing.Netchannel_pretty_printer.xml_printer out_ch in
    let writer = Xnc_writer.create formatter printer in
      Xnc_serializable.write writer serializable_field;
      Xnc_writer.flush writer

  method from_netchannel prototype in_ch =
    let mgr = Pxp_ev_parser.create_entity_manager
                config
                (Pxp_types.from_obj_channel in_ch) in
    let stream = Xnc_scanning.pull_parser_to_string_stream config mgr in
    let reader = Xnc_reader.create_from_stream stream in
      Xnc_serializable.read reader prototype

end;;

class validating_xml_serializer
  ?(config = Pxp_types.default_config)
  ?alt
  ?system_id
  ?fixenc
  ?(spec = Pxp_tree_parser.default_spec)
  () : [string] serializer =
object(self)
  inherit xml_serializer ()

  method from_netchannel prototype in_ch =
    let doc = Pxp_tree_parser.parse_document_entity
                config
                (Pxp_types.from_obj_channel ?alt ?system_id ?fixenc in_ch)
                spec in
    let node = doc # root in
    let reader = Xnc_reader.create_from_pxp_node node in
      Xnc_serializable.read reader prototype

end;;

