(** Token evaluation classes
  *
  * This module provides the low level interface and classes used to evaluate
  * tokens.
  *)

exception Token_evaluation_failed of string
(** Raised when a token conversion fails. *)

class type ['a] token_evaluator =
object
  method get_int :
    ?properties:Xnc_property.properties ->
    'a -> int
  (** [get_int t] converts token [t] to an [int] value. *)

  method get_float :
    ?properties:Xnc_property.properties ->
    'a -> float
  (** [get_float t] converts token [t] to a [float] value. *)

  method get_char :
    ?properties:Xnc_property.properties ->
    'a -> char
  (** [get_char t] converts token [t] to a [char] value. *)

  method get_bool :
    ?properties:Xnc_property.properties ->
    'a -> bool
  (** [get_bool t] converts token [t] to a [bool] value. *)

  method get_string :
    ?properties:Xnc_property.properties ->
    'a -> string
  (** [get_string t] converts token [t] to a [string] value. *)

  method get_date :
    ?properties:Xnc_property.properties ->
    'a -> Netdate.t
  (** [get_date t] converts token [t] to a date value. *)

end
(** This interface defines an evaluator that converts scanned tokens of type
  * ['a] to typed values.
  *)

class standard_evaluator :
  ?config:Xnc_serialization.common_config ->
  unit ->
  [string] token_evaluator
(** Standard string token evaluator. Its implementation makes use of the
  * standard conversion functions: [int_of_string], [float_of_string],
  * [bool_of_string], and [Netdate.parse]. *)

