type 'a t =
    { formatter : 'a Xnc_formatting.formatter;
      printer : 'a Xnc_printing.printer;
    }

let create f p =
  { formatter = f;
    printer = p
  }
;;

let create_from_netchannel
      ?(config = Xnc_serialization.default_config)
      out_ch =
  let f = new Xnc_formatting.standard_formatter ~config () in
  let p = new Xnc_printing.Netchannel_printer.default_printer ~config out_ch in
    create f p
;;

let create_from_buffer
      ?(config = Xnc_serialization.default_config)
      buf =
  let f = new Xnc_formatting.standard_formatter ~config () in
  let p = new Xnc_printing.Buffer_printer.default_printer ~config buf in
    create f p
;;

let write_null ?properties writer () =
  writer.printer # print_value ()
    (writer.formatter # format_null ?properties)
;;

let write_int ?properties writer i =
  writer.printer # print_value i
    (writer.formatter # format_int ?properties)
;;

let write_float ?properties writer f =
  writer.printer # print_value f
    (writer.formatter # format_float ?properties)
;;

let write_char ?properties writer c =
  writer.printer # print_value c
    (writer.formatter # format_char ?properties)
;;

let write_bool ?properties writer b =
  writer.printer # print_value b
    (writer.formatter # format_bool ?properties)
;;

let write_raw_string ?properties writer s =
  writer.printer # print_value s
    (writer.formatter # format_raw_string ?properties)
;;

let write_string ?properties writer s =
  writer.printer # print_value s
    (writer.formatter # format_string ?properties)
;;

let write_date ?properties writer s =
  writer.printer # print_value s
    (writer.formatter # format_date ?properties)
;;

let write_value writer v format_value =
  writer.printer # print_value v format_value
;;

let write_option writer o write_null write_value =
  writer.printer # print_option o write_null write_value
;;

let write_object ?properties writer o =
  writer.printer # print_object o
    (write_string ?properties writer)
;;

let write_enum writer e write_element =
  writer.printer # print_enum e write_element
;;

let write_collection writer coll to_enum write_element =
  let enum = to_enum coll in
    write_enum writer enum write_element
;;

let write_field writer prefix name value properties write_value =
  writer.printer # print_field prefix name value properties write_value
;;

let flush writer =
  writer.printer # flush ()
;;

let close writer =
  writer.printer # close ()
;;

