#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import os
import glob
import shutil

sys.path.append(os.path.join("..", "..", "build"))
import build_lib
from build_lib import *


INSTALL_FILES = " ".join([
    "xnc_stdlib.mli",
    "xnc_stdlib.cmi",
    "xnc_functional.mli",
    "xnc_functional.cmi",
    "xnc_core.mli",
    "xnc_core.cmi",
    "xnc_cache.mli",
    "xnc_cache.cmi",
    "xnc_singleton.mli",
    "xnc_singleton.cmi",
    "xnc_scheduler.mli",
    "xnc_scheduler.cmi",
    "xnc_cache_manager.mli",
    "xnc_cache_manager.cmi",
    "xnc_logger.mli",
    "xnc_logger.cmi",
    "xnc_socket.mli",
    "xnc_socket.cmi",
    "xnc_callback_repository.mli",
    "xnc_callback_repository.cmi",
    "xnc_stream.mli",
    "xnc_stream.cmi",
    "xnc_dbm.mli",
    "xnc_dbm.cmi",
    "xnc_core.cma",
    "xnc_core.cmxa",
    "xnc_core.a"
    ])

def print_help():
    print """This is the Ex-nunc build system, you are welcome.
Usage: build.py target [target2 target3...]

Available targets:

Cleaning targets:
  clean     - remove generated files

Compilation  targets:
  lib (default) - compiles xnc_core (bytecode, NO INSTALL)
  lib.opt       - compiles xnc_core (native and bytecode, NO INSTALL)
  install       - INSTALL xnc_core (native and bytecode)
  test          - build and executes xnc_core test suite (native and bytecode)
  help          - show this help message"""

    sys.exit(0)

##########################
# THE SCRIPT BEGINS HERE #
##########################
if (find_list2(["--help", "-help", "-h", "help"], sys.argv)):
    print_help()

if (len(sys.argv) == 1):
    sys.argv.append("lib")

if not os.path.exists("myocamlbuild.ml"):
    shutil.copy("../../myocamlbuild.ml", ".")

for arg in sys.argv[1:]:
    if arg == "clean":
        run("ocamlbuild -clean")
        enter_dir_and_do("test", lambda : run("ocamlbuild -clean"))
        if os.path.exists("test/myocamlbuild.ml"):
            os.unlink("test/myocamlbuild.ml")
        if os.path.exists("myocamlbuild.ml"):
            os.unlink("myocamlbuild.ml")
    elif arg == "lib":
        run("ocamlbuild xnc_core.cma")
    elif arg == "lib.opt":
        run("ocamlbuild xnc_core.cma xnc_core.cmxa xnc_core.a")
    elif arg == "install":
        shutil.copy("META", "_build")
        enter_dir_and_do("_build",
                lambda : run("ocamlfind install xnc-core META " + INSTALL_FILES))
    elif arg == "test":
        if not os.path.exists("test/myocamlbuild.ml"):
            shutil.copy("../../myocamlbuild.ml", "test/")
        def job():
            run("ocamlbuild test_suite.byte test_suite.native")
            run("./test_suite.byte")
            run("./test_suite.native")
        enter_dir_and_do("test", job)
    else:
        sys.stderr.write(
            "%s: unknown action \"%s\": try with --help\n" % (sys.argv[0], arg)
        )
        sys.exit(1)


# vim:set encoding=utf8:

