open OUnit

type events = Event1 | Event2 | Event3;;

(* tests *)
let test_count _ =
  let cbr = Xnc_callback_repository.create () in
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected number of callbacks"
      0
      (Xnc_callback_repository.count cbr)
;;

let test_add_handler _ =
  let cbr = Xnc_callback_repository.create () in
    Xnc_callback_repository.add_handler cbr
      Event1
      (fun () -> ());
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected number of callbacks"
      1
      (Xnc_callback_repository.count cbr)
;;

let test_extract_callback_list _ =
  let cbr = Xnc_callback_repository.create () in
  let () = Xnc_callback_repository.add_handler cbr
             Event1
             (fun () -> ()) in
  let () = Xnc_callback_repository.add_handler cbr
             Event3
             (fun () -> ()) in
  let () = Xnc_callback_repository.add_handler cbr
             Event3
             (fun () -> ()) in
  let post_creation_list =
    Xnc_callback_repository.extract_callback_list cbr Event1 in
  let post_event_handling_list =
    Xnc_callback_repository.extract_callback_list cbr Event2 in
  let post_render_list =
    Xnc_callback_repository.extract_callback_list cbr Event3 in
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected number of callbacks"
      1
      (List.length post_creation_list);
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected number of callbacks"
      0
      (List.length post_event_handling_list);
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected number of callbacks"
      2
      (List.length post_render_list)
;;

let test_invoke _ =
  let i = ref 5 in
  let cbr = Xnc_callback_repository.create () in
    Xnc_callback_repository.add_handler
      cbr
      Event1
      (fun () -> ignore(i := !i + 10));
    Xnc_callback_repository.add_handler
      cbr
      Event1
      (fun () -> ignore(i := !i * 2));
    Xnc_callback_repository.invoke
      cbr
      Event1;
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Wrong callback invocation order"
      30
      !i
;;

(* test suite *)
let suite = "Callback repository test" >:::
  ["test_count" >:: test_count;
   "test_add_handler" >:: test_add_handler;
   "test_extract_callback_list" >:: test_extract_callback_list;
   "test_invoke" >:: test_invoke]
;;

