open OUnit

module Test_value =
struct
  type t = int

  let counter = ref 0

  let create () =
    incr counter;
    !counter

end;;

module Test_singleton =
  Xnc_singleton.Make(Test_value)(Xnc_core.Null_mutex)
;;

module Test_safe_singleton =
  Xnc_singleton.Make(Test_value)(Mutex)
;;

(* fixture setup and teardown *)
let set_up _ =
  Test_value.counter := 0
;;

let tear_down _ =
  ()
;;

(* tests *)
let test_get_instance _ =
  ignore(Test_singleton.get_instance ());
  OUnit.assert_equal
    ~printer:string_of_int
    ~msg:"Unexpected singleton content"
    1
    (Test_singleton.get_instance ());
;;

let test_update_instance _ =
  ignore(Test_singleton.get_instance ());
  Test_singleton.update_instance 2;
  OUnit.assert_equal
    ~printer:string_of_int
    ~msg:"Unexpected singleton content"
    2
    (Test_singleton.get_instance ());
;;

let test_clear_instance _ =
  ignore(Test_singleton.get_instance ());
  Test_singleton.clear_instance ();
  OUnit.assert_equal
    ~printer:string_of_int
    ~msg:"Unexpected singleton content"
    2
    (Test_singleton.get_instance ());
;;

let test_safe_get_instance _ =
  let get_instance _ =
    ignore(Test_safe_singleton.get_instance ()) in
  let t1 = Thread.create get_instance () in
  let t2 = Thread.create get_instance () in
  let _ = Thread.join t1 in
  let _ = Thread.join t2 in
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected singleton content"
      1
      (Test_safe_singleton.get_instance ());
;;

(* test suite *)
let suite = "Singleton test" >:::
  ["test_get_instance" >:: test_get_instance;
   "test_update_instance" >:: test_update_instance;
   "test_clear_instance" >:: test_clear_instance;
   "test_safe_get_instance"
     >:: (bracket set_up test_safe_get_instance tear_down)]
;;

