open OUnit

(* tests *)
let test_for_loop _ =
  let acc = ref 0 in
  let stream = Stream.from (fun i -> Some i) in
    Xnc_stream.for_loop
      ~test:(fun i -> i < 10)
      ~f:(fun i -> acc := !acc + i; false)
      stream;
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected acc value"
      45
      !acc;
;;

let test_fold_loop _ =
  let stream = Stream.from (fun i -> Some i) in
  let acc =
    Xnc_stream.fold
      ~test:(fun i -> i < 10)
      ~f:(fun i acc -> (acc + i, false))
      0
      stream
  in
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected acc value"
      45
      acc;
;;

let test_for2 _ =
  let acc = ref 0 in
  let s1 = Stream.from (fun i -> Some i) in
  let s2 = Stream.from (fun i -> Some (-i)) in
    Xnc_stream.for2
      ~test:(fun i1 i2 -> i1 - i2 < 10)
      ~f:(fun i1 i2 -> acc := !acc - i1 + i2; false)
      s1
      s2;
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected acc value"
      (-20)
      !acc;
;;

let test_fold2 _ =
  let s1 = Stream.from (fun i -> Some i) in
  let s2 = Stream.from (fun i -> Some (-i)) in
  let acc =
    Xnc_stream.fold2
      ~test:(fun i1 i2 -> i1 - i2 < 10)
      ~f:(fun i1 i2 acc -> (acc - i1 + i2, false))
      0
      s1
      s2
  in
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected acc value"
      (-20)
      acc;
;;

(* test suite *)
let suite = "Stream test" >:::
  ["test_for_loop" >:: test_for_loop;
   "test_fold_loop" >:: test_fold_loop;
   "test_for2" >:: test_for2;
   "test_fold2" >:: test_fold2;
   ]
;;

