module type CACHE_CONF =
sig
  type key
  type item

  val interval : float
end;;

module type CACHE_MANAGER =
sig
  type key
  type item

  module Cache : Xnc_cache.CACHE
  val get_cache : unit -> (key, item) Cache.t
end;;

module Make
  (Conf : CACHE_CONF)
  (Lockable : Xnc_core.LOCKABLE)
  (Scheduler : Xnc_scheduler.SCHEDULER) =
struct
  type key = Conf.key
  type item = Conf.item

  module Cache = Xnc_cache.Make(Lockable)

  module Initializable =
  struct
    type t = (key, item) Cache.t

    let create () =
      let cache = Cache.create () in
      let scheduler = Scheduler.create
                        (fun () ->
                           Cache.expire cache)
                        Conf.interval in
        Scheduler.start scheduler;
        cache

  end

  module Singleton = Xnc_singleton.Make(Initializable)(Lockable)

  let get_cache = Singleton.get_instance

end;;

