(** Cache manager
  *
  * This module implements a cache manager, that provides a global cache with
  * scheduled expiration and locking policy.
  *)

module type CACHE_CONF =
sig
  type key
  (** Type of keys. *)
  type item
  (** Type of items. *)

  val interval : float
  (** The period of the timer that triggers cache expiration. *)

end
(** Signature of cache configuration. *)

module Make :
  functor (Conf : CACHE_CONF) ->
  functor (Lockable : Xnc_core.LOCKABLE) ->
  functor (Scheduler : Xnc_scheduler.SCHEDULER) ->
sig
  module Cache : Xnc_cache.CACHE
  (** Implements the global cache. *)

  val get_cache : unit -> (Conf.key, Conf.item) Cache.t
  (** [get_cache ()] returns the global cache. *)

end
(** Functor that builds the cache manager. *)

