(** Callback repository.
  *
  * This module implements a repository to store a sequence of callbacks
  * associated to an event. *)

type handler = unit -> unit
(** The type of the callback functions. *)

type 'a t
(** The type of the repostitory of callbacks associated to events of type ['a].
  *)

val create : ?size:int -> unit -> 'a t
(** [create ()] creates an empty repository.
  *
  * @params size The initial size of the repository. *)

val count : 'a t -> int
(** [count r] returns the total number of callbacks stored in [r]. *)

val add_handler : 'a t -> 'a -> handler -> unit
(** [add_handler r e c] adds callback [c] to repository [r] associated to event
  * [e]. *)

val extract_callback_list : 'a t -> 'a -> handler list
(** [extract_callback_list r e] extracts the list of callbacks associated to
  * event [e], in the same order of introduction in repository [r]. *)

val invoke : 'a t -> 'a -> unit
(** [invoke r e] invokes the callbacks associated to event [e], in the same
  * order of introduction in repository [r]. *)

