(** Core module.
  *
  * This module implements some common types shared by other modules.
  *)

module type INITIALIZABLE =
sig
  type t
  (** The type of the value to store. *)
  
  val create : unit -> t
  (** [create ()] creates an instance. *)

end
(** Module interface that collects a type and its constructor. *)

module type LOCKABLE =
sig
  type t
  (** The type of locks. *)

  val create : unit -> t
  (** [create ()] creates a lock. *)

  val lock : t -> unit
  (** [lock l] acquires lock [l]. *)

  val unlock : t -> unit
  (** [unlock l] releases lock [l]. *)

end
(** Module interface that describes a lockable resource. *)

module Null_mutex : LOCKABLE
(** Implements a no-op lock, that can be used in single-threaded applications,
  * to avoid the mutex overhead. *)

