module type SCHEDULER =
sig
  type t

  val create : (unit -> unit) -> float -> t
  val start : t -> unit
  val stop : t -> unit
end;;

module Null : SCHEDULER =
struct
  type t = unit

  let create _ _ = ()
  let start _ = ()
  let stop _ = ()
end;;

module Itimer : SCHEDULER =
struct
  type t =
      { callback : unit -> unit;
        interval : float;
        mutable original_behavior : Sys.signal_behavior;
        mutable original_status : Unix.interval_timer_status
      }

  let create c i =
    { callback = c;
      interval = i;
      original_behavior = Sys.Signal_default;
      original_status =
        { Unix.it_interval = 0.;
          Unix.it_value = 0.
        }
    }

  let start s =
    let behavior =
      Sys.signal Sys.sigalrm
        (Sys.Signal_handle
           (fun _ ->
              s.callback ())) in
    let status = Unix.setitimer Unix.ITIMER_REAL
                   { Unix.it_interval = s.interval;
                     Unix.it_value = s.interval
                   }
    in
      s.original_behavior <- behavior;
      s.original_status <- status

  let stop s =
    Sys.set_signal Sys.sigalrm s.original_behavior;
    ignore (Unix.setitimer Unix.ITIMER_REAL s.original_status)

end;;

