(** Scheduler
  *
  * This module implements a scheduler that executes a function at specified
  * intervals.
  *)

module type SCHEDULER =
sig
  type t
  (** The type of schedulers. *)

  val create : (unit -> unit) -> float -> t
  (** [create f s] creates a scheduler that executes function [f] every [s]
    * seconds. *)

  val start : t -> unit
  (** [start s] starts scheduler [s]. *)

  val stop : t -> unit
  (** [stop s] stops scheduler [s]. *)

end
(** Signature of scheduler modules. *)

module Null : SCHEDULER
(** This module implements a scheduler that does nothing. *)

module Itimer : SCHEDULER
(** This module implements a scheduler based on UNIX interval timer. *)

