module Make
  (Initializable : Xnc_core.INITIALIZABLE)
  (Lockable : Xnc_core.LOCKABLE) =
struct
  type t = Initializable.t

  let instance = Global.empty "instance"
  let l = Lockable.create ()

  (* private *)
  let with_lock l f =
    try
      let () = Lockable.lock l in
      let result = f () in
        Lockable.unlock l;
        result
    with e ->
      Lockable.unlock l;
      raise e
      
  (* private *)
  let initizialize_instance () =
    if not (Global.isdef instance) then
      begin
        with_lock l
          (fun () ->
            if not (Global.isdef instance) then
              begin
                let value = Initializable.create () in
                  Global.set instance value
              end
            else
              ())
      end
    else
      ()

  let get_instance () =
    initizialize_instance ();
    Global.get instance

  let update_instance value =
    initizialize_instance ();
    with_lock l
      (fun () -> Global.set instance value)

  let clear_instance () =
    with_lock l
      (fun () -> Global.undef instance)

end;;

