let (|>) x f = f x
let rec loop p f x = if (p x) then x else loop p f (f x)

let char_of_string s =
  if String.length s <> 1 then invalid_arg "Xnc_stdlib.char_of_string"
  else String.get s 0

module Map =
struct
  module type MAP =
  sig
    include Map.S

    val find_opt : key -> 'a t -> 'a option
    val keys : 'a t -> key list
    val values : 'a t -> 'a list
    val filter_key : (key -> bool) -> 'a t -> 'a t
    val filter_value : ('a -> bool) -> 'a t -> 'a t
    val map_of_assoc : (key * 'a) list -> 'a t
  end

  module Make (Ord : Map.OrderedType) : MAP with type key = Ord.t =
  struct
    include Map.Make(Ord)

    let find_opt key map =
      try Some (find key map)
      with | Not_found -> None

    let keys m = fold (fun k v acc -> k::acc) m [] |> List.rev
    let values m = fold (fun k v acc -> v::acc) m [] |> List.rev
    let filter_key p m = fold (fun k v new_map ->
                                 if p k then add k v new_map else new_map)
                           m
                           empty
    let filter_value p m = fold (fun k v new_map ->
                                 if p v then add k v new_map else new_map)
                           m
                           empty
    let map_of_assoc l =
      List.fold_left (fun m (key, value) -> add key value m) empty l
  end
end;;


module Set =
struct
  module type SET =
  sig
    include Set.S
    
    val set_of_list : elt list -> t
  end
    
  module Make (Ord : Set.OrderedType) : SET with type elt = Ord.t =
  struct
    include Set.Make(Ord)
    
    let set_of_list l =
      List.fold_left (fun set list_el -> add list_el set) empty l
  end
end


module List =
struct
  include List

  let str_sort = List.sort String.compare
  (*
   * Sort a list of strings in increasing order according to the comparison
   * function [String.compare].
   *)

  let hd_opt l = try Some (List.hd l) with | Failure "hd" -> None
  (*
   * Return [Some f], the first element of the given list.
   * Return [None] if the list is empty.
   *)

  let find_opt p l = try Some (List.find p l) with Not_found -> None
  (*
   * [find_opt p l] returns the first element of the list [l] that satisfies the
   * predicate [p]. Returns None if there is no value that satisfies [p] in the
   * list [l].
   *)

  (*
   * Returns a list of integers from 0 to stop-1 inclusives.
   *
   * If start and/or step are specified the sequence starts from "start"
   * with step "step"
   *)
  let range ?(start = 0) ?(step = 1) stop =
    loop (fun (n,acc) -> n >= stop) (fun (n,acc) -> (n+step,n::acc)) (start,[])
      |> snd |> List.rev;;

end
(** List operations. *)


module Array =
struct
  include Array

  (*
   * Returns an array of integers from 0 to stop-1 inclusives.
   *
   * If start and/or step are specified the sequence starts from "start"
   * with step "step"
   *)
  let range ?(start = 0) ?(step = 1) stop =
    let len =
      if (stop - start) mod step = 0
      then (stop - start) / step
      else (stop - start) / step + 1 in
    let result = Array.make len 0 in
    let count = ref start in
    let index = ref 0 in
      while !count < stop
      do
        result.(!index) <- !count;
        count := !count + step;
        incr index
      done;
      result
end

