(** Stream iterators.
  *
  * This module implements iterative functions over streams.
  *)

val fold :
  ?test:('a -> bool) ->
  f:('a -> 'b -> 'b * bool) ->
  'b -> 'a Stream.t -> 'b
(** [fold t f a s] folds stream [s]. For each element of [s], it applies test
  * function [t]. If [t] returns [true], if applies function [f] to the current
  * element. [f] returns the new accumulator value and an early termination flag
  * that if set to [true] stops the folding. Raises [Stream.Failure] if stream
  * end is reached. *)

val fold2 :
  ?test:('a -> 'b -> bool) ->
  f:('a -> 'b -> 'c -> 'c * bool) ->
  'c -> 'a Stream.t -> 'b Stream.t -> 'c
(** [fold2 t f a s1 s2] folds streams [s1] and [s2] in parallel. For each pair
  * of elements from [s1] and [s2], it applies test function [t]. If [t] returns
  * [true], if applies function [f] to the current pair. [f] returns the new
  * accumulator value and an early termination flag that if set to [true] stops
  * the folding. Raises [Stream.Failure] if stream end is reached.  *)

val for_loop :
  ?test:('a -> bool) ->
  f:('a -> bool) ->
  'a Stream.t -> unit
(** [for_loop t f s] simulates an imperative [for] statement over stream [s],
  * looping until test [t] returns [false] (or early termination flag returned
  * by [f] is set to [true]) and applying [f] to each element of [s]. Raises
  * [Stream.Failure] if stream end is reached. *)

val for2 :
  ?test:('a -> 'b -> bool) ->
  f:('a -> 'b -> bool) ->
  'a Stream.t -> 'b Stream.t -> unit
(** [for2 t f s1 s2] simulates an imperative [for] statement over streams [s1]
  * and [s2], looping until test [t] returns [false] (or early termination flag
  * returned by [f] is set to [true]) and applying [f] to each pair of elements
  * from [s1] and [s2]. Raises Stream.Failure if stream end is reached. *)

