#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import os
import glob
import shutil

sys.path.append(os.path.join("..", "build"))
import build_lib
from build_lib import *

INSTALL_FILES = " ".join([
    "srv_types.mli",
    "srv_types.cmi",
    "srv_values.mli",
    "srv_values.cmi",
    "srv_command_line.mli",
    "srv_command_line.cmi",
    "srv_configuration_loader.mli",
    "srv_configuration_loader.cmi",
    "srv_configuration.mli",
    "srv_configuration.cmi",
    "srv_statistics.mli",
    "srv_statistics.cmi",
    "srv_runtime_context.mli",
    "srv_runtime_context.cmi",
    "srv_resource_list.mli",
    "srv_resource_list.cmi",
    "srv_state_persisting.mli",
    "srv_state_persisting.cmi",
    "srv_state.mli",
    "srv_state.cmi",
    "srv_input_validator.mli",
    "srv_input_validator.cmi",
    "srv_validator_collection.mli",
    "srv_validator_collection.cmi",
    "srv_page_element.mli",
    "srv_page_element.cmi",
    "srv_page_structure.mli",
    "srv_page_structure.cmi",
    "srv_page_helper.mli",
    "srv_page_helper.cmi",
    "srv_page_state.mli",
    "srv_page_state.cmi",
    "srv_page_behavior.mli",
    "srv_page_behavior.cmi",
    "srv_standard_error_page.mli",
    "srv_standard_error_page.cmi",
    "srv_standard_not_found_page.mli",
    "srv_standard_not_found_page.cmi",
    "srv_page_state_machine.mli",
    "srv_page_state_machine.cmi",
    "srv_page_dispatcher.mli",
    "srv_page_dispatcher.cmi",
    "srv_request_processor.mli",
    "srv_request_processor.cmi",
    "srv_main_loop.mli",
    "srv_main_loop.cmi",
    "srv_sqlite3_state.mli",
    "srv_sqlite3_state.cmi",
    "application_server.cma",
    "application_server.cmxa",
    "application_server.a"
    ])

def print_help():
    print """This is the Ex-nunc build system, you are welcome.
Usage: build.py target [target2 target3...]

Available targets:

Cleaning targets:
  clean     - remove generated files

Compilation  targets:
  lib (default) - compiles application_server (bytecode, NO INSTALL)
  lib.opt       - compiles application_server (native and bytecode, NO INSTALL)
  install       - INSTALL application_server (native and bytecode)
  test          - build and exec. application_server test (native and bytecode)
  help          - show this help message"""

    sys.exit(0)

##########################
# THE SCRIPT BEGINS HERE #
##########################
if (find_list2(["--help", "-help", "-h", "help"], sys.argv)):
    print_help()

if (len(sys.argv) == 1):
    sys.argv.append("lib")

if not os.path.exists("myocamlbuild.ml"):
    shutil.copy("../myocamlbuild.ml", ".")

for arg in sys.argv[1:]:
    if arg == "clean":
        run("ocamlbuild -clean")
        enter_dir_and_do("test", lambda : run("ocamlbuild -clean"))
        if os.path.exists("test/myocamlbuild.ml"):
            os.unlink("test/myocamlbuild.ml")
        if os.path.exists("myocamlbuild.ml"):
            os.unlink("myocamlbuild.ml")
    elif arg == "lib":
        run("ocamlbuild " +
                "-I xnc_core " +
                "-I serialization " +
                "application_server.cma")
    elif arg == "lib.opt":
        run("ocamlbuild " +
                "-I xnc_core " +
                "-I serialization " +
                expand_exts("application_server", ["cma", "cmxa", "a"]))
    elif arg == "install":
        shutil.copy("META", "_build")
        enter_dir_and_do("_build",
                lambda : run("ocamlfind install xnc-server META " +
                    INSTALL_FILES))
    elif arg == "test":
        if not os.path.exists("test/myocamlbuild.ml"):
            shutil.copy("../myocamlbuild.ml", "test/")
        def job():
            run("ocamlbuild test_suite.byte test_suite.native")
            run("./test_suite.byte")
            run("./test_suite.native")
        enter_dir_and_do("test", job)
    else:
        sys.stderr.write(
            "%s: unknown action \"%s\": try with --help\n" % (sys.argv[0], arg)
        )
        sys.exit(1)


# vim:set encoding=utf8:

