type args =
    { application_name : string;
      configuration_path : string;
      configuration_dtd : string
    }
;;

let default_args =
  { application_name = "demo.fcgi";
    configuration_path = "ex-nunc-config.xml";
    configuration_dtd = "ex-nunc-config.dtd";
  }
;;

let program_version_long application_name server_version =
  Printf.printf
    "This application, \"%s\", has been generated using the\n\
     Ex-nunc Application System version %s\n\
     \n\
     Use --help for a quick help.\n\
     Use --copyright for copyright informations. Please, take a look.\n"
    application_name
    server_version;
  exit 0
;;

let display_application_version application_name application_version =
  Printf.printf
    "%s (version %s)\n"
    application_name
    application_version;
  exit 0
;;

let ex_nunc_copyright_info application_name server_version =
  let alessandro_email = "astrada" ^
                         "@" ^
                         "users.sourceforge.net" in
  let paolo_email = "pdonadeo" ^
                    "@" ^
                    "users.sourceforge.net"
  in
    Printf.printf
      "This application, \"%s\", has been generated using the

              E X   N U N C   A P P L I C A T I O N   S Y S T E M
                                version %s


Ex-nunc Application System copyright informations:
                  
Copyright (c) 2005-2008 by Paolo Donadeo <%s>
                           and
                           Alessandro Strada <%s>

Permission is hereby granted, free of charge, to any person obtaining
a copy of this software and associated documentation files (the
\"Software\"), to deal in the Software without restriction, including
without limitation the rights to use, copy, modify, merge, publish,
distribute, sublicense, and/or sell copies of the Software, and to
permit persons to whom the Software is furnished to do so, subject to
the following conditions:

The above copyright notice and this permission notice shall be included
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND,
EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Use --help for a quick help."
      application_name
      server_version
      paolo_email
      alessandro_email;
  exit 0
;;

let parse application_values =
  let application_name = Filename.basename Sys.executable_name in
  let configuration_path = ref default_args.configuration_path in
  let configuration_dtd = ref default_args.configuration_dtd in
  let arguments_spec =
    Arg.align
      [
        "-v",
          Arg.Unit (fun () -> display_application_version
                                application_name
                                application_values.Srv_values.version),
          " Print application version";
        "--version",
          Arg.Unit (fun () -> program_version_long
                                application_name
                                Srv_values.server_version),
          " Print more verbose application server informations";
        "--copyright",
          Arg.Unit (fun () -> ex_nunc_copyright_info
                                application_name
                                Srv_values.server_version),
          " Print informations on Ex-nunc copyright. Please take a look";
        "-f",
          Arg.Set_string configuration_path,
          " Set the configuration file path (default: \"" ^
            !configuration_path ^ "\")";
        "-d",
          Arg.Set_string configuration_dtd,
          " Set the configuration DTD path (default: \"" ^
            !configuration_dtd ^ "\")"
      ] in
  let anonymous_arguments_handler param =
    Printf.printf
      "Ex-nunc: WARNING: ignoring unrecognized parameter \"%s\"\n"
      param
  in
    Arg.parse
      arguments_spec
      anonymous_arguments_handler
      ("Usage: " ^ application_name ^ " [options]\nOptions are:");
    { application_name = application_name;
      configuration_path = !configuration_path;
      configuration_dtd = !configuration_dtd
    }
;;

