exception Invalid_type_name

class type input_validator =
object
  method get_error_message : string

  method get_value : string

  method is_valid : unit -> bool

end

class virtual input_validator_base
  (value : string)
  (error_message : string) =
object
  method get_value =
    value

  method get_error_message =
    error_message

  method virtual is_valid : unit -> bool

end;;

class custom_validator
  (value : string)
  (validator : string -> bool)
  (err_message : string) =
object(self)
  inherit input_validator_base value err_message

  method is_valid () =
    validator (self # get_value)

end;;

class required_field_validator
  (value : string)
  (err_message : string) =
object(self)
  inherit input_validator_base value err_message

  method is_valid () =
    (self # get_value) <> ""

end;;

class type_validator
  (value : string)
  (type_name : string)
  (err_message : string) =
object(self)
  inherit input_validator_base value err_message

  method is_valid () =
    let type_converter = match type_name with
        "int" ->
          (fun s -> ignore(int_of_string s))
      | "float" ->
          (fun s -> ignore(float_of_string s))
      | "char" ->
          (fun s ->
             if (String.length s) <> 1 then
               raise (Failure "char_of_string"))
      | "string" ->
          (fun s -> ())
      | "bool" ->
          (fun s -> ignore(bool_of_string s))
      | _ -> raise Invalid_type_name
    in
      try
        type_converter (self # get_value);
        true
      with Failure _ -> false
        | Invalid_argument _ -> false

end;;

