(** Server page dispatcher.
  *
  * This module implements the page dispatcher. *)
  
module type PAGE_DISPATCHER =
sig
  type page_factory_type
  (** The type of page factories. *)

  type page_parameter_type
  (** The type of page parameters. *)

  val serve_page :
    page_factory_type ->
    Netcgi.cgi_activation ->
    Srv_configuration.t ->
    Xnc_logger.t ->
    Srv_state_persisting.Serialized.t ->
    unit
  (** [serve_page f cgi conf l s] invokes the page recovering name, and
    * parameters [p] from state [s], using page factory [f], logger [l],
    * configuration [conf]. *)

  val serve_error_page :
    page_factory_type ->
    Netcgi.cgi_activation ->
    Srv_configuration.t ->
    Xnc_logger.t ->
    Srv_state_persisting.Serialized.t ->
    exn ->
    unit
  (** [serve_error_page f cgi conf l s e] invokes the error page to display
    * exception [e], using page factory [f], logger [l], configuration [conf],
    * state [s]. *)

end
(** Signature of page dispatcher module. *)

module Make:
  functor (Page_parameters : Srv_types.PAGE_PARAMETERS) ->
  functor (Page_state_machine : Srv_page_state_machine.PAGE_STATE_MACHINE
             with type page_parameter_type = Page_parameters.t) ->
sig
  include PAGE_DISPATCHER
    with type page_factory_type = Page_state_machine.page_factory_type
      and type page_parameter_type = Page_parameters.t
end
(** This functor builds the module, that contains the page dispatcher
  * implementation. *)

