type ui_event = Click

module Attributes =
struct
  let render_attribute
        name value (out : Netchannels.trans_out_obj_channel) =
    out # output_string " ";
    out # output_string name;
    out # output_string "=\"";
    out # output_string (Xnc_string_transform.escape_xml value);
    out # output_string "\""

  let render_implied_attribute
        name value (out : Netchannels.trans_out_obj_channel) =
    Option.may
      (fun v -> render_attribute name v out)
      value

  let render_attribute_with_default
        name value default (out : Netchannels.trans_out_obj_channel) =
    render_attribute name (Option.default default value) out

  class virtual i18n =
  object
    val mutable lang = None
    val mutable dir = None

    method get_lang = lang
    method get_dir = dir

    method set_lang value = lang <- value
    method set_dir value = dir <- value

    method render_attributes (out : Netchannels.trans_out_obj_channel) =
      render_implied_attribute "xml:lang" lang out;
      render_implied_attribute "dir" dir out;

  end

  class virtual coreattrs =
  object
    val mutable id = None
    val mutable css_class = None
    val mutable style = None
    val mutable title = None

    method get_id = id
    method get_class = css_class
    method get_style = style
    method get_title = title

    method set_id value = id <- value
    method set_class value = css_class <- value
    method set_style value = style <- value
    method set_title value = title <- value

    method render_attributes (out : Netchannels.trans_out_obj_channel) =
      render_implied_attribute "id" id out;
      render_implied_attribute "class" css_class out;
      render_implied_attribute "style" style out;
      render_implied_attribute "title" title out;

  end

  class virtual events =
  object
    val mutable onclick = None
    val mutable ondblclick = None
    val mutable onmousedown = None
    val mutable onmouseup = None
    val mutable onmouseover = None
    val mutable onmousemove = None
    val mutable onmouseout = None
    val mutable onkeypress = None
    val mutable onkeydown = None
    val mutable onkeyup = None

    method get_onclick = onclick
    method get_ondblclick = ondblclick
    method get_onmousedown = onmousedown
    method get_onmouseup = onmouseup
    method get_onmouseover = onmouseover
    method get_onmousemove = onmousemove
    method get_onmouseout = onmouseout
    method get_onkeypress = onkeypress
    method get_onkeydown = onkeydown
    method get_onkeyup = onkeyup

    method set_onclick value = onclick <- value
    method set_ondblclick value = ondblclick <- value
    method set_onmousedown value = onmousedown <- value
    method set_onmouseup value = onmouseup <- value
    method set_onmouseover value = onmouseover <- value
    method set_onmousemove value = onmousemove <- value
    method set_onmouseout value = onmouseout <- value
    method set_onkeypress value = onkeypress <- value
    method set_onkeydown value = onkeydown <- value
    method set_onkeyup value = onkeyup <- value

    method render_attributes (out : Netchannels.trans_out_obj_channel) =
      render_implied_attribute "onclick" onclick out;
      render_implied_attribute "ondblclick" ondblclick out;
      render_implied_attribute "onmousedown" onmousedown out;
      render_implied_attribute "onmouseup" onmouseup out;
      render_implied_attribute "onmouseover" onmouseover out;
      render_implied_attribute "onmousemove" onmousemove out;
      render_implied_attribute "onmouseout" onmouseout out;
      render_implied_attribute "onkeypress" onkeypress out;
      render_implied_attribute "onkeydown" onkeydown out;
      render_implied_attribute "onkeyup" onkeyup out;

  end

  class virtual attrs =
  object
    inherit coreattrs as coreattrs
    inherit i18n as i18n
    inherit events as events

    method render_attributes (out : Netchannels.trans_out_obj_channel) =
      coreattrs # render_attributes out;
      i18n # render_attributes out;
      events # render_attributes out;

  end

  class virtual focus =
  object
    val mutable accesskey = None
    val mutable tabindex = None
    val mutable onfocus = None
    val mutable onblur = None

    method get_accesskey = accesskey
    method get_tabindex = tabindex
    method get_onfocus = onfocus
    method get_onblur = onblur

    method set_accesskey value = accesskey <- value
    method set_tabindex value = tabindex <- value
    method set_onfocus value = onfocus <- value
    method set_onblur value = onblur <- value

    method render_attributes (out : Netchannels.trans_out_obj_channel) =
      render_implied_attribute "accesskey" accesskey out;
      render_implied_attribute "tabindex" tabindex out;
      render_implied_attribute "onfocus" onfocus out;
      render_implied_attribute "onblur" onblur out;

  end

end;;

class type page_element =
object
  method render : Netchannels.trans_out_obj_channel -> unit

  method handle_ui_events : Netcgi.cgi_activation -> unit

  method add_element : page_element -> unit

end;;

class virtual page_element_base =
object
  method virtual render : Netchannels.trans_out_obj_channel -> unit

  method handle_ui_events (cgi : Netcgi.cgi_activation) =
    ()

  method add_element (element : page_element) =
    ()

end;;

class literal_element content =
object
  inherit page_element_base

  method render (out : Netchannels.trans_out_obj_channel) =
    out # output_string content

end;;

class container_element =
object
  inherit page_element_base

  val elements : page_element Queue.t = Queue.create ()

  method render (out : Netchannels.trans_out_obj_channel) =
    Queue.iter
      (fun element ->
         element # render out)
      elements

  method handle_ui_events (cgi : Netcgi.cgi_activation) =
    Queue.iter
      (fun element ->
         element # handle_ui_events cgi)
      elements

  method add_element element =
    Queue.add element elements

end;;

class virtual hidden_field name =
object(self)
  inherit page_element_base

  method render (out : Netchannels.trans_out_obj_channel) =
    out # output_string "<input type=\"hidden\" name=\"";
    out # output_string name;
    out # output_string "\" value=\"";
    self # render_value out;
    out # output_string "\" />";

  method private virtual render_value :
    Netchannels.trans_out_obj_channel -> unit

end;;

class simple_hidden_field name value =
object(self)
  inherit hidden_field name

  method private render_value (out : Netchannels.trans_out_obj_channel) =
    out # output_string value;

end;;

class complex_hidden_field ?serializer name field_collection =
object(self)
  inherit hidden_field name

  method private render_value (out : Netchannels.trans_out_obj_channel) =
    Xnc_field_collection.to_netchannel
      ?serializer
      field_collection
      (out :> Netchannels.out_obj_channel);

end;;

class form_element action =
object(self)
  inherit container_element as super
  inherit Attributes.attrs as attrs
  
  val mutable enctype = None
  val mutable onsubmit = None
  val mutable onreset = None
  val mutable accept = None
  val mutable accept_charset = None

  method get_enctype = enctype
  method get_onsubmit = onsubmit
  method get_onreset = onreset
  method get_accept = accept
  method get_accept_charset = accept_charset

  method set_enctype value = enctype <- value
  method set_onsubmit value = onsubmit <- value
  method set_onreset value = onreset <- value
  method set_accept value = accept <- value
  method set_accept_charset value = accept_charset <- value

  val hidden_fields : hidden_field Queue.t =
    Queue.create ()

  method render (out : Netchannels.trans_out_obj_channel) =
    out # output_string ("<form method=\"post\" \
                                action=\"" ^ action ^ "\"");
    attrs # render_attributes out;
    Attributes.render_implied_attribute "enctype" enctype out;
    Attributes.render_implied_attribute "onsubmit" onsubmit out;
    Attributes.render_implied_attribute "onreset" onreset out;
    Attributes.render_implied_attribute "accept" accept out;
    Attributes.render_implied_attribute "accept-charset" accept_charset out;
    out # output_string ">";
    super # render out;
    self # render_hidden_fields out;
    out # output_string "</form>";

  method add_hidden_field hf =
    Queue.add hf hidden_fields

  method private render_hidden_fields (out: Netchannels.trans_out_obj_channel) =
    if not (Queue.is_empty hidden_fields) then
      begin
        out # output_string "<div>";
        Queue.iter
          (fun hidden_field ->
             hidden_field # render out)
          hidden_fields;
        out # output_string "</div>";
      end
    else
      ()

end;;

class validator_element (validator : Srv_input_validator.input_validator) =
object(self)
  inherit page_element_base

  method render (out : Netchannels.trans_out_obj_channel) =
    if not (validator # is_valid ()) then
      out # output_string
        ("<span>" ^ validator # get_error_message ^ "</span>")

end;;

class validation_summary validators summary_header =
object(self)
  inherit page_element_base

  method render (out : Netchannels.trans_out_obj_channel) =
    if not (Srv_validator_collection.is_valid validators) then
      begin
        out # output_string ("<span>" ^ summary_header ^ "<br />");
        List.iter
          (fun elem ->
             out # output_string ("- " ^ elem ^ "<br />"))
          (Srv_validator_collection.error_message_list validators);
        out # output_string "</span>"
      end

end;;

class head_element =
object(self)
  inherit container_element as super
  inherit Attributes.i18n as i18n

  val mutable id = None
  val mutable profile = None

  method get_id = id
  method get_profile = profile

  method set_id value = id <- value
  method set_profile value = profile <- value

  method render (out : Netchannels.trans_out_obj_channel) =
    out # output_string "<head";
    i18n # render_attributes out;
    Attributes.render_implied_attribute "id" id out;
    Attributes.render_implied_attribute "profile" profile out;
    out # output_string ">";
    super # render out;
    out # output_string "</head>";

end;;

class body_element =
object(self)
  inherit container_element as super
  inherit Attributes.attrs as attrs

  val mutable onload = None
  val mutable onunload = None

  method get_onload = onload
  method get_onunload = onunload

  method set_onload value = onload <- value
  method set_onunload value = onunload <- value

  method render (out : Netchannels.trans_out_obj_channel) =
    out # output_string "<body";
    attrs # render_attributes out;
    Attributes.render_implied_attribute "onload" onload out;
    Attributes.render_implied_attribute "onunload" onunload out;
    out # output_string ">";
    super # render out;
    out # output_string "</body>";

end;;

class div_element =
object(self)
  inherit container_element as super
  inherit Attributes.attrs as attrs

  method render (out : Netchannels.trans_out_obj_channel) =
    out # output_string "<div";
    attrs # render_attributes out;
    out # output_string ">";
    super # render out;
    out # output_string "</div>";

end;;

class input_element =
object(self)
  inherit page_element_base
  inherit Attributes.attrs as attrs
  inherit Attributes.focus as focus

  val mutable input_type = None
  val mutable name = None
  val mutable input_value = None
  val mutable checked = None
  val mutable disabled = None
  val mutable readonly = None
  val mutable size = None
  val mutable maxlength = None
  val mutable src = None
  val mutable alt = None
  val mutable usemap = None
  val mutable onselect = None
  val mutable onchange = None
  val mutable accept = None

  method get_type = input_type
  method get_name = name
  method get_value = input_value
  method get_checked = checked
  method get_disabled = disabled
  method get_readonly = readonly
  method get_size = size
  method get_maxlength = maxlength
  method get_src = src
  method get_alt = alt
  method get_usemap = usemap
  method get_onselect = onselect
  method get_onchange = onchange
  method get_accept = accept

  method set_type value = input_type <- value
  method set_name value = name <- value
  method set_value value = input_value <- value
  method set_checked value = checked <- value
  method set_disabled value = disabled <- value
  method set_readonly value = readonly <- value
  method set_size value = size <- value
  method set_maxlength value = maxlength <- value
  method set_src value = src <- value
  method set_alt value = alt <- value
  method set_usemap value = usemap <- value
  method set_onselect value = onselect <- value
  method set_onchange value = onchange <- value
  method set_accept value = accept <- value

  method render (out : Netchannels.trans_out_obj_channel) =
    out # output_string "<input";
    attrs # render_attributes out;
    focus # render_attributes out;
    Attributes.render_attribute_with_default "type" input_type "text" out;
    Attributes.render_implied_attribute "name" name out;
    Attributes.render_implied_attribute "value" input_value out;
    Attributes.render_implied_attribute "checked" checked out;
    Attributes.render_implied_attribute "disabled" disabled out;
    Attributes.render_implied_attribute "readonly" readonly out;
    Attributes.render_implied_attribute "size" size out;
    Attributes.render_implied_attribute "maxlength" maxlength out;
    Attributes.render_implied_attribute "src" src out;
    Attributes.render_implied_attribute "alt" alt out;
    Attributes.render_implied_attribute "usemap" usemap out;
    Attributes.render_implied_attribute "onselect" onselect out;
    Attributes.render_implied_attribute "onchange" onchange out;
    Attributes.render_implied_attribute "accept" accept out;
    out # output_string " />";

end;;

class button_element button_name =
object(self)
  inherit input_element as super

  val callbacks : (ui_event Xnc_callback_repository.t) =
    Xnc_callback_repository.create ()

  method render (out : Netchannels.trans_out_obj_channel) =
    super # set_type (Some "submit");
    super # set_name (Some button_name);
    super # set_value (Some button_name);
    super # render out;

  method handle_ui_events (cgi : Netcgi.cgi_activation) =
    if (cgi # argument_value button_name) = button_name then
      self # on_click ()
    else
      ()

  method add_click_handler handler =
    Xnc_callback_repository.add_handler callbacks Click handler

  method private on_click () =
    Xnc_callback_repository.invoke callbacks Click

end;;

