(** Page elements.
  *
  * This module provides interface and implementations for page elements. *)

type ui_event =
    Click
(** The type of UI events that can be handled by page elements. *)

module Attributes :
sig
  class virtual i18n :
  object
    method get_lang : string option
    (** [get_lang] returns the value of the optional {e lang} attribute. *)

    method get_dir : string option
    (** [get_dir] returns the value of the optional {e dir} attribute. *)

    method set_lang : string option -> unit
    (** [set_lang v] sets the value of the optional {e lang} attribute to [v].
      *)

    method set_dir : string option -> unit
    (** [set_dir v] sets the value of the optional {e dir} attribute to [v]. *)

    method render_attributes : Netchannels.trans_out_obj_channel -> unit
    (** [render_attributes c] renders collected attributes to output object
      * channel [c]. *)

  end
  (** This class groups XHTML I18n attributes. *)

  class virtual coreattrs :
  object
    method get_id : string option
    (** [get_id] returns the value of the optional {e id} attribute. *)

    method get_class : string option
    (** [get_class] returns the value of the optional {e class} attribute. *)

    method get_style : string option
    (** [get_style] returns the value of the optional {e style} attribute. *)

    method get_title : string option
    (** [get_title] returns the value of the optional {e title} attribute. *)

    method set_id : string option -> unit
    (** [set_id v] sets the value of the optional {e id} attribute to [v]. *)

    method set_class : string option -> unit
    (** [set_class v] sets the value of the optional {e class} attribute to [v].
      *)

    method set_style : string option -> unit
    (** [set_style v] sets the value of the optional {e style} attribute to [v].
      *)

    method set_title : string option -> unit
    (** [set_title v] sets the value of the optional {e title} attribute to [v].
      *)

    method render_attributes : Netchannels.trans_out_obj_channel -> unit
    (** [render_attributes c] renders collected attributes to output object
      * channel [c]. *)

  end
  (** This class groups XHTML core attributes. *)

  class virtual events :
  object
    method get_onclick : string option
    (** [get_onclick] returns the value of the optional {e onclick} attribute.
      *)

    method get_ondblclick : string option
    (** [get_ondblclick] returns the value of the optional {e ondblclick}
      * attribute. *)

    method get_onmousedown : string option
    (** [get_onmousedown] returns the value of the optional {e onmousedown}
      * attribute. *)

    method get_onmouseup : string option
    (** [get_onmouseup] returns the value of the optional {e onmouseup}
      * attribute. *)

    method get_onmouseover : string option
    (** [get_onmouseover] returns the value of the optional {e onmouseover}
      * attribute. *)

    method get_onmousemove : string option
    (** [get_onmousemove] returns the value of the optional {e onmousemove}
      * attribute. *)

    method get_onmouseout : string option
    (** [get_onmouseout] returns the value of the optional {e onmouseout}
      * attribute. *)

    method get_onkeypress : string option
    (** [get_onkeypress] returns the value of the optional {e onkeypress}
      * attribute. *)

    method get_onkeydown : string option
    (** [get_onkeydown] returns the value of the optional {e onkeydown}
      * attribute. *)

    method get_onkeyup : string option
    (** [get_onkeyup] returns the value of the optional {e onkeyup} attribute.
      *)

    method set_onclick : string option -> unit
    (** [set_onclick v] sets the value of the optional {e onclick} attribute to
      * [v]. *)

    method set_ondblclick : string option -> unit
    (** [set_ondblclick v] sets the value of the optional {e ondblclick}
      * attribute to [v]. *)

    method set_onmousedown : string option -> unit
    (** [set_onmousedown v] sets the value of the optional {e onmousedown}
      * attribute to [v]. *)

    method set_onmouseup : string option -> unit
    (** [set_onmouseup v] sets the value of the optional {e onmouseup} attribute
      * to [v]. *)

    method set_onmouseover : string option -> unit
    (** [set_onmouseover v] sets the value of the optional {e onmouseover}
      * attribute to [v]. *)

    method set_onmousemove : string option -> unit
    (** [set_onmousemove v] sets the value of the optional {e onmousemove}
      * attribute to [v]. *)

    method set_onmouseout : string option -> unit
    (** [set_onmouseout v] sets the value of the optional {e onmouseout}
      * attribute to [v]. *)

    method set_onkeypress : string option -> unit
    (** [set_onkeypress v] sets the value of the optional {e onkeypress}
      * attribute to [v]. *)

    method set_onkeydown : string option -> unit
    (** [set_onkeydown v] sets the value of the optional {e onkeydown} attribute
      * to [v]. *)

    method set_onkeyup : string option -> unit
    (** [set_onkeyup v] sets the value of the optional {e onkeyup} attribute to
      * [v]. *)

    method render_attributes : Netchannels.trans_out_obj_channel -> unit
    (** [render_attributes c] renders collected attributes to output object
      * channel [c]. *)

  end
  (** This class groups XHTML event attributes. *)

  class virtual attrs :
  object
    inherit coreattrs
    inherit i18n
    inherit events

    method render_attributes : Netchannels.trans_out_obj_channel -> unit
    (** [render_attributes c] renders collected attributes to output object
      * channel [c]. *)

  end
  (** This class groups XHTML core, I18n, and event attributes. *)

  class virtual focus :
  object
    method get_accesskey : string option
    (** [get_accesskey] returns the value of the optional {e accesskey}
      * attribute. *)

    method get_tabindex : string option
    (** [get_tabindex] returns the value of the optional {e tabindex}
      * attribute. *)

    method get_onfocus : string option
    (** [get_onfocus] returns the value of the optional {e onfocus} attribute.
      *)

    method get_onblur : string option
    (** [get_onblur] returns the value of the optional {e onblur} attribute.
      *)

    method set_accesskey : string option -> unit
    (** [set_accesskey v] sets the value of the optional {e accesskey} attribute
      * to [v]. *)

    method set_tabindex : string option -> unit
    (** [set_tabindex v] sets the value of the optional {e tabindex} attribute
      * to [v]. *)

    method set_onfocus : string option -> unit
    (** [set_onfocus v] sets the value of the optional {e onfocus} attribute to
      * [v]. *)

    method set_onblur : string option -> unit
    (** [set_onblur v] sets the value of the optional {e onblur} attribute to
      * [v]. *)

    method render_attributes : Netchannels.trans_out_obj_channel -> unit
    (** [render_attributes c] renders collected attributes to output object
      * channel [c]. *)

  end
  (** This class groups XHTML focus attributes. *)

end
(** This module collects classes that groups XHTML attributes. Page elements
  * compose one or more of this groups of attributes using multiple inheritance.
  * The type [string option] is used for optional attributes, the type [string]
  * is used for required attributes. *)

class type page_element =
object
  method add_element : page_element -> unit
  (** [add_element e] adds a child element [e]. *)

  method handle_ui_events : Netcgi.cgi_activation -> unit
  (** [handle_ui_events c] handles UI events detected from user input available
    * through [c]. *)

  method render : Netchannels.trans_out_obj_channel -> unit
  (** [render c] renders the element to output object channel [c]. *)

end
(** This interface specifies the common behavior of a page element. *)

class virtual page_element_base :
object
  method add_element : page_element -> unit
  (** [add_element e] default implementation does nothing. *)

  method handle_ui_events : Netcgi.cgi_activation -> unit
  (** [handle_ui_events c] default implementation does nothing. *)

  method virtual render : Netchannels.trans_out_obj_channel -> unit
  (** [render c] renders the element to output object channel [c]. *)

end
(** Base class for page elements.. *)

class literal_element :
  string ->
  page_element
(** [new literal_element s] returns a literal page element that renders [s]
  * string. *)

class container_element : page_element
(** [new container_element] returns a container of page elements. *)

class virtual hidden_field :
  string ->
object
  inherit page_element_base

  method render : Netchannels.trans_out_obj_channel -> unit
  (** [render c] renders the hidden field to output object channel [c]. *)

  method private virtual render_value :
    Netchannels.trans_out_obj_channel -> unit
  (** [render_value c] renders the value of the field to output object channel
    * [c]. *)

end
(** [new hidden_field n] returns a page element named [n] that renders an XHTML
  * {e input} element of type {e hidden}. *)

class simple_hidden_field :
  string ->
  string ->
  page_element
(** [new simple_hidden_field n v] returns a hidden field with name [n] and value
  * [v]. *)

class complex_hidden_field :
  ?serializer:string Xnc_serializing.serializer ->
  string ->
  Xnc_field_collection.t ->
  page_element
(** [new complex_hidden_field n c] returns a hidden field with name [n] and
  * value obtained serializing [c].
  *
  * @param serializer Optional custom serializer. *)

class form_element :
  string ->
object
  inherit container_element
  inherit Attributes.attrs
  
  method get_enctype : string option
  (** [get_enctype] returns the value of the optional {e enctype} attribute. *)
  
  method get_onsubmit : string option
  (** [get_onsubmit] returns the value of the optional {e onsubmit}
    * attribute. *)
  
  method get_onreset : string option
  (** [get_onreset] returns the value of the optional {e onreset} attribute. *)
  
  method get_accept : string option
  (** [get_accept] returns the value of the optional {e accept} attribute. *)
  
  method get_accept_charset : string option
  (** [get_accept_charset] returns the value of the optional {e accept_charset}
    * attribute. *)

  method set_enctype : string option -> unit
  (** [set_enctype v] sets the value of the optional {e enctype} attribute
    * to [v]. *)

  method set_onsubmit : string option -> unit
  (** [set_onsubmit v] sets the value of the optional {e onsubmit} attribute
    * to [v]. *)

  method set_onreset : string option -> unit
  (** [set_onreset v] sets the value of the optional {e onreset} attribute
    * to [v]. *)

  method set_accept : string option -> unit
  (** [set_accept v] sets the value of the optional {e accept} attribute
    * to [v]. *)

  method set_accept_charset : string option -> unit
  (** [set_accept_charset v] sets the value of the optional {e accept_charset}
    * attribute to [v]. *)

  method render : Netchannels.trans_out_obj_channel -> unit
  (** [render c] renders the form element to output object channel [c]. *)

  method add_hidden_field : hidden_field -> unit
  (** [add_hidden_field h] adds hidden field element [h] to the hidden fields of
    * the form. *)

end
(** [new form_element a] returns a page element that renders an XHTML {e form}
  * element with action [a]. *)

class validator_element :
  Srv_input_validator.input_validator ->
  page_element
(** [new validator_element v] returns a page element associated to an input
  * validator [v] that renders the error message if the validation fails. *)

class validation_summary :
  Srv_validator_collection.t ->
  string ->
  page_element
(** [new validator_element c] returns a page element associated to an input
  * validator collection that renders a summary of all the error messages if the
  * validation fails. *)

class head_element :
object
  inherit container_element
  inherit Attributes.i18n

  method get_id : string option
  (** [get_id] returns the value of the optional {e id} attribute. *)

  method get_profile : string option
  (** [get_profile] returns the value of the optional {e profile} attribute. *)

  method set_id : string option -> unit
  (** [set_id v] sets the value of the optional {e id} attribute to [v]. *)

  method set_profile : string option -> unit
  (** [set_profile v] sets the value of the optional {e profile} attribute to
    * [v]. *)

  method render : Netchannels.trans_out_obj_channel -> unit
  (** [render c] renders the head element to output object channel [c]. *)

end
(** [new head_element] returns a page element that renders an XHTML {e head}
  * element. *)

class body_element :
object
  inherit container_element
  inherit Attributes.attrs

  method get_onload : string option
  (** [get_onload] returns the value of the optional {e onload} attribute. *)

  method get_onunload : string option
  (** [get_onunload] returns the value of the optional {e onunload} attribute.
    *)

  method set_onload : string option -> unit
  (** [set_onload v] sets the value of the optional {e onload} attribute to [v].
    *)

  method set_onunload : string option -> unit
  (** [set_onunload v] sets the value of the optional {e onunload} attribute to
    * [v]. *)

  method render : Netchannels.trans_out_obj_channel -> unit
  (** [render c] renders the body element to output object channel [c]. *)

end
(** [new body_element] returns a page element that renders an XHTML {e body}
  * element. *)

class div_element :
object
  inherit container_element
  inherit Attributes.attrs

  method render : Netchannels.trans_out_obj_channel -> unit
  (** [render c] renders the div element to output object channel [c]. *)

end
(** [new div_element] returns a page element that renders an XHTML {e div}
  * element. *)

class input_element :
object
  inherit page_element_base
  inherit Attributes.attrs
  inherit Attributes.focus

  method get_type : string option
  (** [get_type] returns the value of the optional {e type} attribute. *)

  method get_name : string option
  (** [get_name] returns the value of the optional {e name} attribute. *)

  method get_value : string option
  (** [get_value] returns the value of the optional {e value} attribute. *)

  method get_checked : string option
  (** [get_checked] returns the value of the optional {e checked} attribute. *)

  method get_disabled : string option
  (** [get_disabled] returns the value of the optional {e disabled} attribute.
    *)

  method get_readonly : string option
  (** [get_readonly] returns the value of the optional {e readonly} attribute.
    *)

  method get_size : string option
  (** [get_size] returns the value of the optional {e size} attribute. *)

  method get_maxlength : string option
  (** [get_maxlength] returns the value of the optional {e maxlength} attribute.
    *)

  method get_src : string option
  (** [get_src] returns the value of the optional {e src} attribute. *)

  method get_alt : string option
  (** [get_alt] returns the value of the optional {e alt} attribute. *)

  method get_usemap : string option
  (** [get_usemap] returns the value of the optional {e usemap} attribute. *)

  method get_onselect : string option
  (** [get_onselect] returns the value of the optional {e onselect} attribute.
    *)

  method get_onchange : string option
  (** [get_onchange] returns the value of the optional {e onchange} attribute.
    *)

  method get_accept : string option
  (** [get_accept] returns the value of the optional {e accept} attribute.
    *)

  method set_type : string option -> unit
  (** [set_type v] sets the value of the optional {e type} attribute to [v].
    *)

  method set_name : string option -> unit
  (** [set_name v] sets the value of the optional {e name} attribute to [v].
    *)

  method set_value : string option -> unit
  (** [set_value v] sets the value of the optional {e value} attribute to [v].
    *)

  method set_checked : string option -> unit
  (** [set_checked v] sets the value of the optional {e checked} attribute to
    * [v]. *)

  method set_disabled : string option -> unit
  (** [set_disabled v] sets the value of the optional {e disabled} attribute to
    * [v]. *)

  method set_readonly : string option -> unit
  (** [set_readonly v] sets the value of the optional {e readonly} attribute to
    * [v]. *)

  method set_maxlength : string option -> unit
  (** [set_maxlength v] sets the value of the optional {e maxlength} attribute
    * to [v]. *)

  method set_size : string option -> unit
  (** [set_size v] sets the value of the optional {e size} attribute to [v].
    *)

  method set_src : string option -> unit
  (** [set_src v] sets the value of the optional {e src} attribute to [v]. *)

  method set_alt : string option -> unit
  (** [set_alt v] sets the value of the optional {e alt} attribute to [v]. *)

  method set_usemap : string option -> unit
  (** [set_usemap v] sets the value of the optional {e usemap} attribute to [v].
    *)

  method set_onselect : string option -> unit
  (** [set_onselect v] sets the value of the optional {e onselect} attribute to
    * [v]. *)

  method set_onchange : string option -> unit
  (** [set_onchange v] sets the value of the optional {e onchange} attribute to
    * [v]. *)

  method set_accept : string option -> unit
  (** [set_accept v] sets the value of the optional {e accept} attribute to [v].
    *)

  method render : Netchannels.trans_out_obj_channel -> unit
  (** [render c] renders the input element to output object channel [c]. *)

end
(** [new div_element] returns a page element that renders an XHTML {e div}
  * element. *)

class button_element :
  string ->
object
  inherit input_element

  method add_click_handler : Xnc_callback_repository.handler -> unit
  (** [add_click_handler h] adds handler [h] to the callback chain, that is
    * invoked when handling UI event [Click]. *)

  method render : Netchannels.trans_out_obj_channel -> unit
  (** [render c] renders the button element to output object channel [c]. *)

end
(** [new button_element n] returns a page element named [n] that renders an
  * XHTML {e input} element of type {e submit}. *)

