let text = Xnc_string_transform.escape_xml;;

class name_value_table label
  (name_value_list_builder : unit -> (string * string) list) =
object
  inherit Srv_page_element.page_element_base

  method render (out : Netchannels.trans_out_obj_channel) =
    let name_value_list = name_value_list_builder () in
    let output = out # output_string in
      output "<table>";
      output ("<caption>" ^ label ^ "</caption>");
      output "<col class=\"name\" />\
              <col class=\"value\" />\
                <tr>\
                  <th>Name</th>\
                  <th>Value</th>\
                </tr>";
    ignore(List.fold_left
             (fun parity (name, value) ->
                let css_class = if parity then "even" else "odd" in
                  output ("<tr class=\"" ^ css_class ^ "\">\
                             <td>" ^ name ^ "</td>\
                             <td>" ^ value ^ "</td>\
                           </tr>");
                  not parity)
             true
             name_value_list);
    output "</table>"

end;;

class misc_info
  (context : Srv_runtime_context.context)
  (cgi : Netcgi.cgi_activation) =
object
  inherit Srv_page_element.page_element_base

  method render (out : Netchannels.trans_out_obj_channel) =
    let statistics = context.Srv_runtime_context.statistics in
    let total_requests = statistics.Srv_statistics.total_requests in
    let total_errors = statistics.Srv_statistics.total_errors in
    let successful_requests =
      total_requests - total_errors in
    let success_perc = (float_of_int successful_requests) /.
                       (float_of_int total_requests) *. 100.0 in
    let error_perc = (float_of_int total_errors) /.
                     (float_of_int total_requests) *. 100.0 in
    let output = out # output_string in
      output "<h2>Server info</h2>\
              <p>";
      output ("Total requests: " ^
              (string_of_int total_requests) ^ "<br />");
      output (Printf.sprintf
                "Successfully processed requests: %d (%.2f %%) <br />"
                successful_requests
                success_perc);
      output (Printf.sprintf
                "Erroneous requests: %d (%.2f %%) <br />"
                total_errors
                error_perc);
      output ("Server PID: " ^
              (string_of_int (Unix.getpid ())) ^ "<br />");
      output "</p>\
              <h2>Request info</h2>\
              <p>";
      output ("Remote host: " ^
              (cgi # environment # cgi_remote_addr) ^ "</p>")

end;;

let create_page_logger () =
  let format_record level message timestamp start_time =
    let level_string = Xnc_logger.string_of_level level in
      Printf.sprintf "<tr class=\"%s\">\
                        <td>%s</td>\
                        <td>%s</td>\
                        <td>%s</td>\
                        <td>%.9f</td>\
                      </tr>"
        level_string
        (Xnc_logger.string_of_time timestamp)
        level_string
        message
        (timestamp -. start_time)
  in
    Xnc_logger.create ~layout:(Xnc_logger.Custom format_record) ()
;;

class trace_table
  (cgi : Netcgi.cgi_activation)
  (logger : Xnc_logger.t) =
object
  inherit Srv_page_element.page_element_base

  method render (out : Netchannels.trans_out_obj_channel) =
    out # output_string "<table id=\"pagetrace\">\
                           <caption>Trace</caption>\
                           <col class=\"date\" />\
                           <col class=\"level\" />\
                           <col class=\"message\" />\
                           <col class=\"elapsed\" />\
                           <tr>\
                             <th>Date</th>\
                             <th>Level</th>\
                             <th>Message</th>\
                             <th>Elapsed (sec)</th>\
                           </tr>";
    Xnc_logger.write_to_netchannel logger out;
    out # output_string "</table>";

end;;

