(** Server request processing module.
  *
  * This module implements the request processor. *)
  
module type REQUEST_PROCESSOR =
sig
  val set_up : unit -> Srv_runtime_context.context
  (** [set_up ()] creates the runtime context. It is called at startup time. *)

  val register_signal_handlers : Srv_runtime_context.context -> unit
  (** [register_signal_handlers c] registers signal handlers. *)

  val select_request_processor :
    Srv_runtime_context.context ->
    Netcgi.cgi_activation ->
    (Netcgi.cgi_activation -> unit)
  (** [select_request_processor ctx cgi] returns the function, used to process
    * requests. *)

end
(** Signature of request processor module. *)

module Make:
  functor (Builder : Srv_runtime_context.CONTEXT_BUILDER) ->
  functor (Context : Srv_runtime_context.RUNTIME_CONTEXT) ->
  functor (Page_dispatcher : Srv_page_dispatcher.PAGE_DISPATCHER) ->
  functor (Page_factory_builder : Xnc_core.INITIALIZABLE
             with type t = Page_dispatcher.page_factory_type) ->
  functor (Resource_list_builder : Xnc_core.INITIALIZABLE
             with type t = Srv_resource_list.t) ->
  REQUEST_PROCESSOR
(** This functor builds modules implementing the request processor. *)

