module Make
  (Page_parameters : Srv_types.PAGE_PARAMETERS)
  (Page_behavior : Srv_page_behavior.PAGE_BEHAVIOR
     with type page_parameter_type = Page_parameters.t) =
struct

  class virtual error_page_base
    page_name
    page_context =
  object(self)
    inherit Page_behavior.page_behavior_base
          page_name
          Page_parameters.default_error_page_parameters
          page_context as super

    method private get_error_description =
      Page_parameters.get_error_description
        (self # get_page_parameters)

    method private get_backtrace =
      Page_parameters.get_backtrace
        (self # get_page_parameters)

    method init () =
      self # register_callback
        Srv_types.Pre_render
        (fun () -> self # get_cgi_context # out_channel # rollback_work ())

    method load_session () =
      try
        super # load_session ()
      with e ->
        (* TODO: trace to main logger from Context *)
        self # trace ~level:Xnc_logger.Error
          ("load_session: " ^ (Printexc.to_string e))

    method build_validator_collection () =
      ()

    method read_user_input () =
      ()

  end

  class standard_error_page
    page_context =
  object(self)
    inherit error_page_base
          "standard_error_page.xnunc"
          page_context

    method build_page_structure parameters =
      (* Set attributes *)
      self # get_page_structure # set_debug true;

      (* Head element *)
      let head = new Srv_page_element.head_element in
        Srv_page_structure.add_element (self # get_page_structure) head;
        Srv_page_structure.add_element head
          (new Srv_page_element.literal_element
             "<title>Ex-nunc error page</title>\
              <link rel=\"stylesheet\" \
                    type=\"text/css\" \
                    href=\"/ex-nunc/style.css\" />");

      (* Body element *)
      let body = Srv_page_structure.create_body_element
                   (self # get_page_structure) in

      let add e content = Srv_page_structure.add_element
                            e
                            (new Srv_page_element.literal_element content) in

        add body "<h1>Software error</h1>\
                    <p>Unhandled exception:<br />\
                      <tt>";

        add body (Srv_page_helper.text (self # get_error_description));

        add body "</tt><br /></p>";

        add body "<p>Backtrace:<br />\
                    <tt>";

        add body (Srv_page_helper.text (self # get_backtrace));

        add body "</tt><br /></p>";

  end

end;;

