(** Standard error page.
  *
  * This module implements the standard error pages, that reports the unhandled
  * exception and the debug information. *)

module Make :
  functor (Page_parameters : Srv_types.PAGE_PARAMETERS) ->
  functor (Page_behavior : Srv_page_behavior.PAGE_BEHAVIOR
             with type page_parameter_type = Page_parameters.t) ->
sig
  class virtual error_page_base :
    string ->
    Srv_page_state.page_context ->
  object
    inherit Page_behavior.page_behavior_base

    method private get_error_description : string
    (** [get_error_description] returns the error description. *)

    method private get_backtrace : string
    (** [get_backtrace] returns the error backtrace. *)

    method build_validator_collection : unit -> unit
    (** [build_validator_collection ()] does nothing. *)

    method read_user_input : unit -> unit
    (** [read_user_input ()] does nothing. *)

  end
  (** [error_page n c f]: Base class of error pages, with name [n], CGI
    * activation [c] and first access flag [f]. *)

  class standard_error_page :
    Srv_page_state.page_context ->
  object
    inherit error_page_base

    method build_page_structure : unit -> unit
    (** [build_page_structure ()] builds page structure of standard error page.
      *)

  end
  (** This class implements the standard error page. *)

end
(** This functor builds concrete standard error page modules. *)

