(** Standard error page.
  *
  * This module implements the standard not found (404) page, that reports the
  * missing resource. *)

module Make :
  functor (Page_parameters : Srv_types.PAGE_PARAMETERS) ->
  functor (Page_behavior : Srv_page_behavior.PAGE_BEHAVIOR
             with type page_parameter_type = Page_parameters.t) ->
sig
  class virtual not_found_page_base :
    string ->
    Srv_page_state.page_context ->
  object
    inherit Page_behavior.page_behavior_base

    method private get_missing_resource : string
    (** [get_missing_resource] returns the missing resource. *)

    method build_validator_collection : unit -> unit
    (** [build_validator_collection ()] does nothing. *)

    method read_user_input : unit -> unit
    (** [read_user_input ()] does nothing. *)

  end
  (** [not_found_page_base n c]: Base class of not found (404) pages, with name
    * [n], and context [c]. *)

  class standard_not_found_page :
    Srv_page_state.page_context ->
  object
    inherit not_found_page_base

    method build_page_structure : unit -> unit
    (** [build_page_structure ()] builds page structure of standard not found
      * (404) page. *)

  end
  (** This class implements the standard not found (404) page. *)

end
(** This functor builds concrete standard not found (404) page modules. *)

