exception Invalid_page_parameters

type lifecycle_hook =
    Pre_session_loading
  | Post_session_loading
  | Pre_parameter_init
  | Post_parameter_init
  | Pre_user_input_validation
  | Post_user_input_validation
  | Pre_user_input_reading
  | Post_user_input_reading
  | Pre_structure_building
  | Post_structure_building
  | Pre_ui_event_handling
  | Post_ui_event_handling
  | Pre_state_saving
  | Post_state_saving
  | Pre_render
  | Post_render
  | Pre_completion
  | Post_completion
;;

type manager_mode =
    Off
  | Client
  | In_memory
  | Sqlite3
;;

let manager_mode_of_string s =
  match String.lowercase s with
      "off" -> Off
    | "client" -> Client
    | "in_memory" -> In_memory
    | "sqlite3" -> Sqlite3
    | _ -> failwith "manager_mode_of_string"
;;

type state_persistence_mode =
    Hidden_field
  | Cookie
;;

let state_persistence_mode_of_string s =
  match String.lowercase s with
      "hidden_field" -> Hidden_field
    | "cookie" -> Cookie
    | _ -> failwith "state_persistence_mode_of_string"
;;

type url_rendering_mode =
    Cgi_style
  | Plain_url
;;

type redirect_mode =
    Server_side
  | Client_side of url_rendering_mode
;;

module type SERIALIZABLE_DATA =
sig
  type t

  val to_field_collection : t -> Xnc_field_collection.t

  val from_field_collection : t -> Xnc_field_collection.t -> t

end;;

module type PAGE_PARAMETERS =
sig
  type t

  (* Start page *)
  val default_start_page_parameters : t

  (* Error page *)
  val create_error_page_parameters : string -> string -> t
  val default_error_page_parameters : t
  val get_error_description : t -> string
  val get_backtrace : t -> string

  (* Not found (404) page *)
  val create_not_found_page_parameters : string -> t
  val default_not_found_page_parameters : t
  val get_missing_resource : t -> string

  (* Serializazion and deserialization *)
  val to_field_collection : t -> Xnc_field_collection.t
  val from_field_collection : t -> Xnc_field_collection.t -> t

end;;

module type SESSION_DATA =
sig
  type t

  val create : unit -> t
  val to_field_collection : t -> Xnc_field_collection.t
  val from_field_collection : Xnc_field_collection.t -> t

end;;

let nethttpd_regexp = Str.regexp_string_case_fold "nethttpd";;

let is_nethttpd (cgi : Netcgi.cgi_activation) =
  try
    ignore (Str.search_forward
              nethttpd_regexp
              cgi # environment # cgi_server_software
              0);
    true
  with Not_found ->
    false
;;

