(** Validator collection.
  *
  * This module implements a repository to store input validators. *)

type t
(** The type of the collection. *)

val create : ?size:int -> unit -> t
(** [create ()] creates an empty collection.
  *
  * @param size The initial size of the collection. *)

val count : t -> int
(** [count c] returns the total number of validators stored in collection [c].
  *)

val add_validator : t -> string -> Srv_input_validator.input_validator -> unit
(** [add_validator c l v] adds validator [v] labeled [l] to collection [c] *)

val is_valid : t -> bool
(** [is_valid c] returns [true] if all the validators stored in [c] are
  * associated with valid input, returns [false] otherwise. *)

val error_message_list : t -> string list
(** [error_message_list c] returns the list of all the error messages collected
  * from validators stored in [c] that are associated with invalid input. *)

val find_validator : t -> string -> Srv_input_validator.input_validator
(** [find_validator c l] returns the validator labeled [l]. *)

