open OUnit

(* test helpers *)
let load_conf application_name =
  try
    Srv_configuration.load
      Srv_values.default_values
      { Srv_command_line.application_name = application_name;
        Srv_command_line.configuration_path =
          "../ex-nunc-config.xml";
        Srv_command_line.configuration_dtd =
          "../ex-nunc-config.dtd"
      }
  with e ->
    print_endline (Pxp_types.string_of_exn e);
    raise e
;;

(* tests *)
let test_start_page _ =
  let conf = load_conf "demo1" in
  let start_page = Srv_configuration.start_page conf in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected start page"
      "start_page.xnunc"
      start_page
;;

let test_error_page_demo1 _ =
  let conf = load_conf "demo1" in
  let error_page = Srv_configuration.error_page conf in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected error page demo1"
      "error_page.xnunc"
      error_page
;;

let test_error_page_demo2 _ =
  let conf = load_conf "demo2" in
  let error_page = Srv_configuration.error_page conf in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected error page demo2"
      "standard_error_page.xnunc"
      error_page
;;

let test_not_found_page_demo1 _ =
  let conf = load_conf "demo1" in
  let not_found_page = Srv_configuration.not_found_page conf in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected not found page demo1"
      "standard_not_found_page.xnunc"
      not_found_page
;;

let test_not_found_page_demo2 _ =
  let conf = load_conf "demo2" in
  let not_found_page = Srv_configuration.not_found_page conf in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected not found page demo2"
      "not_found_page.xnunc"
      not_found_page
;;

let test_session_manager_mode _ =
  let conf = load_conf "demo1" in
  let session_manager_mode = Srv_configuration.session_manager_mode conf in
    OUnit.assert_equal
      ~msg:"Unexpected session manager mode"
      Srv_types.Client
      session_manager_mode
;;

let test_session_persistence_mode _ =
  let conf = load_conf "demo1" in
  let session_persistence_mode =
    Srv_configuration.session_persistence_mode conf
  in
    OUnit.assert_equal
      ~msg:"Unexpected session ID storage mode"
      Srv_types.Hidden_field
      session_persistence_mode
;;

let test_session_timeout _ =
  let conf = load_conf "demo1" in
  let session_timeout =
    Srv_configuration.session_timeout conf
  in
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected session timeout"
      30
      session_timeout
;;

let test_page_parameter_manager_mode _ =
  let conf = load_conf "demo1" in
  let page_parameter_manager_mode =
    Srv_configuration.page_parameter_manager_mode conf
  in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected page parameter manager mode"
      Srv_types.Client
      page_parameter_manager_mode
;;

let test_page_parameter_persistence_mode _ =
  let conf = load_conf "demo1" in
  let page_parameter_persistence_mode =
    Srv_configuration.page_parameter_persistence_mode conf
  in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected page parameter persistence mode"
      Srv_types.Hidden_field
      page_parameter_persistence_mode
;;

let test_page_parameter_timeout _ =
  let conf = load_conf "demo1" in
  let page_parameter_timeout =
    Srv_configuration.page_parameter_timeout conf
  in
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected page parameter timeout"
      30
      page_parameter_timeout
;;

let test_sqlite3_path _ =
  let conf = load_conf "demo1" in
  let sqlite3_path =
    Srv_configuration.sqlite3_path conf
  in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected Sqlite3 file path"
      "/tmp/state.db"
      sqlite3_path
;;

let test_current_page_field_name _ =
  let conf = load_conf "demo1" in
  let current_page_field_name =
    Srv_configuration.current_page_field_name conf
  in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected current page field name"
      "xnunc_page"
      current_page_field_name
;;

let test_page_parameters_field_name _ =
  let conf = load_conf "demo1" in
  let page_parameters_field_name =
    Srv_configuration.page_parameters_field_name conf
  in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected page parameter field name"
      "xnunc_page_parameters"
      page_parameters_field_name
;;

let test_session_field_name _ =
  let conf = load_conf "demo1" in
  let session_field_name =
    Srv_configuration.session_field_name conf
  in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected session field name"
      "xnunc_session"
      session_field_name
;;

let test_connector_type _ =
  let conf = load_conf "demo1" in
  let connector_type = Srv_configuration.connector_type conf in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected connector type"
      Srv_configuration.Netplex
      connector_type
;;

let test_socket_type _ =
  let conf = load_conf "demo2" in
  let socket_type = Srv_configuration.socket_type conf in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected socket type"
      Srv_configuration.Internet
      socket_type;
;;

let test_port _ =
  let conf = load_conf "demo2" in
  let port = Srv_configuration.port conf in
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected socket port"
      9876
      port;
;;

let test_bind_address _ =
  let conf = load_conf "demo2" in
  let addr = Srv_configuration.bind_address conf in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected socket address"
      "192.168.1.2"
      addr;
;;

let test_socket_path _ =
  let conf = load_conf "demo2" in
    OUnit.assert_raises
      ~msg:"Internet socket should not have a path"
      (Failure "No socket path available for non-Local sockets")
      (fun () -> Srv_configuration.socket_path conf);
;;

let test_socket_address _ =
  let conf = load_conf "demo2" in
  let sockaddr = Srv_configuration.socket_address conf in
  let (addr, port) =
    match sockaddr with
       | Unix.ADDR_INET(addr, port) -> (Unix.string_of_inet_addr addr, port)
       | _ -> failwith "Unsupported socket address" in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected socket address"
      "192.168.1.2"
      addr;
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected socket port"
      9876
      port;
;;

let test_netplex_configuration_path _ =
  let conf = load_conf "demo1" in
  let netplex_configuration_path =
    Srv_configuration.netplex_configuration_path conf
  in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected netplex configuration path"
      "/tmp/netplex.cfg"
      netplex_configuration_path
;;

let test_netplex_enable_netcgi _ =
  let conf = load_conf "demo1" in
  let netplex_enable_netcgi =
    Srv_configuration.netplex_enable_netcgi conf
  in
    OUnit.assert_equal
      ~printer:string_of_bool
      ~msg:"Unexpected netplex enable netcgi flag"
      true
      netplex_enable_netcgi
;;

let test_netplex_enable_nethttpd _ =
  let conf = load_conf "demo1" in
  let netplex_enable_nethttpd =
    Srv_configuration.netplex_enable_nethttpd conf
  in
    OUnit.assert_equal
      ~printer:string_of_bool
      ~msg:"Unexpected netplex enable nethttpd flag"
      false
      netplex_enable_nethttpd
;;

let test_netplex_nethttpd_handler _ =
  let conf = load_conf "demo1" in
  let netplex_nethttpd_handler =
    Srv_configuration.netplex_nethttpd_handler conf
  in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected netplex nethttpd handler"
      "ex-nunc"
      netplex_nethttpd_handler
;;

let test_netplex_nethttpd_path _ =
  let conf = load_conf "demo1" in
  let netplex_nethttpd_path =
    Srv_configuration.netplex_nethttpd_path conf
  in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected netplex nethttpd path"
      "/ex-nunc"
      netplex_nethttpd_path
;;

let test_prepare_netplex_configuration _ =
  let conf = load_conf "demo1" in
  let () = Srv_configuration.prepare_netplex_configuration conf in
  let () = OUnit.assert_bool
             "Configuration file not present"
             (Sys.file_exists "/tmp/netplex.cfg") in
  let () = OUnit.assert_raises
             ~msg:"Should raise an exception, \
                   if netplex conf file is already present"
             (Sys_error "/tmp/netplex.cfg: File exists")
             (fun () -> Srv_configuration.prepare_netplex_configuration conf) in
  let in_ch = open_in "/tmp/netplex.cfg" in
  let content = input_line in_ch in
  let () = close_in in_ch in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected netplex configuration file content"
      "netplex { }"
      content;
    Srv_configuration.clean_netplex_configuration conf;
;;

let test_clean_netplex_configuration _ =
  let conf = load_conf "demo1" in
    Srv_configuration.prepare_netplex_configuration conf;
    Srv_configuration.clean_netplex_configuration conf;
    OUnit.assert_bool
      "Configuration file not present"
      (not (Sys.file_exists "/tmp/netplex.cfg"));
    Srv_configuration.clean_netplex_configuration conf;
;;

(* test suite *)
let suite = "Server configuration test" >:::
  ["test_start_page" >:: test_start_page;
   "test_error_page_demo1" >:: test_error_page_demo1;
   "test_error_page_demo2" >:: test_error_page_demo2;
   "test_not_found_page_demo1" >:: test_not_found_page_demo1;
   "test_not_found_page_demo2" >:: test_not_found_page_demo2;
   "test_session_manager_mode" >:: test_session_manager_mode;
   "test_session_persistence_mode" >:: test_session_persistence_mode;
   "test_session_timeout" >:: test_session_timeout;
   "test_page_parameter_manager_mode" >:: test_page_parameter_manager_mode;
   "test_page_parameter_persistence_mode"
     >:: test_page_parameter_persistence_mode;
   "test_page_parameter_timeout" >:: test_page_parameter_timeout;
   "test_sqlite3_path" >:: test_sqlite3_path;
   "test_current_page_field_name" >:: test_current_page_field_name;
   "test_page_parameters_field_name" >:: test_page_parameters_field_name;
   "test_session_field_name" >:: test_session_field_name;
   "test_connector_type" >:: test_connector_type;
   "test_socket_type" >:: test_socket_type;
   "test_bind_address" >:: test_bind_address;
   "test_port" >:: test_port;
   "test_socket_path" >:: test_socket_path;
   "test_socket_address" >:: test_socket_address;
   "test_netplex_configuration_path" >:: test_netplex_configuration_path;
   "test_netplex_enable_netcgi" >:: test_netplex_enable_netcgi;
   "test_netplex_enable_nethttpd" >:: test_netplex_enable_nethttpd;
   "test_netplex_nethttpd_handler" >:: test_netplex_nethttpd_handler;
   "test_netplex_nethttpd_path" >:: test_netplex_nethttpd_path;
   "test_prepare_netplex_configuration" >:: test_prepare_netplex_configuration;
   "test_clean_netplex_configuration" >:: test_clean_netplex_configuration]
;;

