open OUnit

module S = Xnc_serializable;;
module P = Xnc_property;;

(* fixture setup and teardown *)
let set_up _ =
  new Netchannels.input_string
"<?xml version=\"1.0\" encoding=\"iso8859-1\"?>
<!DOCTYPE configuration
  PUBLIC  \"-//EXNUNC//Configuration File//EN\"
          \"ex-nunc-config.dtd\">
<configuration>
  <application id=\"demo1\">
    <runtime>
      <start-page>start_page.xnunc</start-page>
      <error-page>error_page.xnunc</error-page>
      <not-found-page>not_found_page.xnunc</not-found-page>
    </runtime>
    <connector type=\"fcgi\">
      <socket type=\"local\">
        <file-name>/tmp/local_socket</file-name>
      </socket>
    </connector>
  </application>
  <application id=\"demo2\">
    <runtime>
      <start-page>start_page.xnunc</start-page>
      <error-page>error_page.xnunc</error-page>
      <state-manager page-parameter-storage=\"client\"
                     session-storage=\"in_memory\" />
      <hidden-fields current-page-field=\"xnunc_page\"
                     page-parameter-field=\"xnunc_page_parameters\"
                     session-field=\"xnunc_session\" />
    </runtime>
    <connector type=\"ajp\">
      <socket type=\"internet\">
        <address>192.168.1.2</address>
        <port>9876</port>
      </socket>
    </connector>
  </application>
  <application id=\"demo3\">
    <runtime>
      <start-page>start_page.xnunc</start-page>
    </runtime>
    <connector type=\"netplex\">
      <netplex configuration=\"embedded\"
               path=\"/tmp/netplex.cfg\">
        <content>netplex { }</content>
      </netplex>
    </connector>
  </application>
</configuration>"
;;

let tear_down _ =
  ()
;;

(* tests *)
let test_load in_ch =
  let resolver =
    new Pxp_reader.lookup_id_as_file
      ~fixenc:`Enc_iso88591
      [
        Pxp_types.Public ("-//EXNUNC//Configuration File//EN",
                               "ex-nunc-config.dtd"),
        "../ex-nunc-config.dtd";
      ] in
  let data = Srv_configuration_loader.load ~resolvers:[resolver] in_ch in
    OUnit.assert_equal
      ~cmp:S.equal
      ~printer:S.string_of_field
      ~msg:"Unexpected data in data loader"
      (S.new_field "configuration"
         [| S.new_field "application"
              [| S.new_field
                   ~properties:(P.singleton P.attribute)
                   "id" "demo1" S.wrap_string;
                 S.new_field "runtime"
                   [| S.new_field "start-page" "start_page.xnunc" S.wrap_string;
                      S.new_field "error-page" "error_page.xnunc" S.wrap_string;
                      S.new_field "not-found-page" "not_found_page.xnunc"
                        S.wrap_string;
                   |] S.wrap_array;
                 S.new_field "connector"
                   [| S.new_field
                        ~properties:(P.singleton P.attribute)
                        "type" "fcgi" S.wrap_string;
                      S.new_field "socket"
                        [ S.new_field
                            ~properties:(P.singleton P.attribute)
                            "type" "local" S.wrap_string;
                          S.new_field
                            "file-name" "/tmp/local_socket" S.wrap_string;
                        ] S.wrap_list;
                   |] S.wrap_array;
              |] S.wrap_array;
            S.new_field "application"
              [| S.new_field
                   ~properties:(P.singleton P.attribute)
                   "id" "demo2" S.wrap_string;
                 S.new_field "runtime"
                   [| S.new_field "start-page" "start_page.xnunc" S.wrap_string;
                      S.new_field "error-page" "error_page.xnunc" S.wrap_string;
                      S.new_field "state-manager"
                        [ S.new_field
                             ~properties:(P.singleton P.attribute)
                             "page-parameter-storage" "client"
                             S.wrap_string;
                           S.new_field
                             ~properties:(P.singleton P.attribute)
                             "page-parameter-persistence" "hidden_field"
                             S.wrap_string;
                           S.new_field
                             ~properties:(P.singleton P.attribute)
                             "page-parameter-timeout" 30 S.wrap_int;
                           S.new_field
                             ~properties:(P.singleton P.attribute)
                             "session-storage" "in_memory" S.wrap_string;
                           S.new_field
                             ~properties:(P.singleton P.attribute)
                             "session-persistence" "hidden_field"
                             S.wrap_string;
                           S.new_field
                             ~properties:(P.singleton P.attribute)
                             "session-timeout" 30 S.wrap_int;
                           S.new_field
                             ~properties:(P.singleton P.attribute)
                             "sqlite3-path" "/tmp/state.db" S.wrap_string;
                        ] S.wrap_list;
                      S.new_field "hidden-fields"
                        [| S.new_field
                              ~properties:(P.singleton P.attribute)
                              "current-page-field" "xnunc_page" S.wrap_string;
                           S.new_field
                             ~properties:(P.singleton P.attribute)
                             "page-parameter-field" "xnunc_page_parameters"
                             S.wrap_string;
                           S.new_field
                             ~properties:(P.singleton P.attribute)
                             "session-field" "xnunc_session" S.wrap_string;
                        |] S.wrap_array;
                   |] S.wrap_array;
                 S.new_field "connector"
                   [| S.new_field
                         ~properties:(P.singleton P.attribute)
                         "type" "ajp" S.wrap_string;
                      S.new_field "socket"
                        [ S.new_field
                            ~properties:(P.singleton P.attribute)
                            "type" "internet" S.wrap_string;
                          S.new_field "address" "192.168.1.2"
                            S.wrap_string;
                          S.new_field "port" 9876 S.wrap_int;
                        ] S.wrap_list;
                   |] S.wrap_array;
              |] S.wrap_array;
            S.new_field "application"
              [| S.new_field
                   ~properties:(P.singleton P.attribute)
                   "id" "demo3" S.wrap_string;
                 S.new_field "runtime"
                   [| S.new_field "start-page" "start_page.xnunc" S.wrap_string;
                   |] S.wrap_array;
                 S.new_field "connector"
                   [| S.new_field
                        ~properties:(P.singleton P.attribute)
                        "type" "netplex" S.wrap_string;
                      S.new_field "netplex"
                        [ S.new_field
                             ~properties:(P.singleton P.attribute)
                             "configuration" "embedded" S.wrap_string;
                          S.new_field
                            ~properties:(P.singleton P.attribute)
                            "path" "/tmp/netplex.cfg" S.wrap_string;
                          S.new_field
                            ~properties:(P.singleton P.attribute)
                            "enable-netcgi" true S.wrap_bool;
                          S.new_field
                            ~properties:(P.singleton P.attribute)
                            "enable-nethttpd" false S.wrap_bool;
                          S.new_field
                            ~properties:(P.singleton P.attribute)
                            "nethttpd-handler" "ex-nunc" S.wrap_string;
                          S.new_field
                            ~properties:(P.singleton P.attribute)
                            "nethttpd-path" "/ex-nunc" S.wrap_string;
                          S.new_field "content" "netplex { }" S.wrap_string;
                        ] S.wrap_list;
                   |] S.wrap_array;
              |] S.wrap_array;
         |] S.wrap_array)
      data;
;;

(* test suite *)
let suite = "Configuration loader test" >:::
  ["test_load" >:: (bracket set_up test_load tear_down)]
;;

