open OUnit

(* tests *)
let test_is_valid_custom _ =
  let v = new Srv_input_validator.custom_validator
            "value"
            (fun v -> v = "value")
            "Error during field validation" in
    OUnit.assert_bool
      "Validation of \"value\" failed"
      (v # is_valid ())
;;

let test_is_not_valid_custom _ =
  let v = new Srv_input_validator.custom_validator
            "value"
            (fun v -> v = "test")
            "Error during field validation" in
    OUnit.assert_bool
      "Validation of \"value\" should fail"
      (not (v # is_valid ()))
;;

let test_get_error_message _ =
  let v = new Srv_input_validator.custom_validator
            "value"
            (fun v -> v = "value")
            "Error during field validation" in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected error message"
      "Error during field validation"
      (v # get_error_message)
;;

let test_is_valid_required _ =
  let v = new Srv_input_validator.required_field_validator
            "value"
            "Error during required field validation" in
    OUnit.assert_bool
      "Validation of \"value\" failed using a required field validator"
      (v # is_valid ())
;;

let test_is_not_valid_required _ =
  let v = new Srv_input_validator.required_field_validator
            ""
            "Error during required field validation" in
    OUnit.assert_bool
      "Validation of \"\" should fail using a required field validator"
      (not (v # is_valid ()))
;;

let test_is_valid_int_type _ =
  let v = new Srv_input_validator.type_validator
            "10"
            "int"
            "Error during type validation" in
    OUnit.assert_bool
      "Validation of \"10\" failed using an int type validator"
      (v # is_valid ())
;;

let test_is_not_valid_int_type _ =
  let v = new Srv_input_validator.type_validator
            "test"
            "int"
            "Error during type validation" in
    OUnit.assert_bool
      "Validation of \"test\" should fail using an int type validator"
      (not (v # is_valid ()))
;;

let test_is_valid_float_type _ =
  let v = new Srv_input_validator.type_validator
            "0.5"
            "float"
            "Error during type validation" in
    OUnit.assert_bool
      "Validation of \"0.5\" failed using a float type validator"
      (v # is_valid ())
;;

let test_is_not_valid_float_type _ =
  let v = new Srv_input_validator.type_validator
            "test"
            "float"
            "Error during type validation" in
    OUnit.assert_bool
      "Validation of \"test\" should fail using a float type validator"
      (not (v # is_valid ()))
;;

let test_is_valid_char_type _ =
  let v = new Srv_input_validator.type_validator
            "a"
            "char"
            "Error during type validation" in
    OUnit.assert_bool
      "Validation of \"a\" failed using a char type validator"
      (v # is_valid ())
;;

let test_is_not_valid_char_type _ =
  let v = new Srv_input_validator.type_validator
            "test"
            "char"
            "Error during type validation" in
    OUnit.assert_bool
      "Validation of \"test\" should fail using a char type validator"
      (not (v # is_valid ()))
;;

let test_is_valid_string_type _ =
  let v = new Srv_input_validator.type_validator
            "any"
            "string"
            "Error during type validation" in
    OUnit.assert_bool
      "Validation of \"any\" failed using a string type validator"
      (v # is_valid ())
;;

let test_is_valid_bool_type _ =
  let v = new Srv_input_validator.type_validator
            "false"
            "bool"
            "Error during type validation" in
    OUnit.assert_bool
      "Validation of \"false\" failed using a bool type validator"
      (v # is_valid ())
;;

let test_is_not_valid_bool_type _ =
  let v = new Srv_input_validator.type_validator
            "test"
            "bool"
            "Error during type validation" in
    OUnit.assert_bool
      "Validation of \"test\" should fail using a bool type validator"
      (not (v # is_valid ()))
;;

(* test suite *)
let suite = "Form validator test" >:::
  ["test_is_valid_custom" >:: test_is_valid_custom;
   "test_is_not_valid_custom" >:: test_is_not_valid_custom;
   "test_get_error_message" >:: test_get_error_message;
   "test_is_valid_required" >:: test_is_valid_required;
   "test_is_not_valid_required" >:: test_is_not_valid_required;
   "test_is_valid_int_type" >:: test_is_valid_int_type;
   "test_is_not_valid_int_type" >:: test_is_not_valid_int_type;
   "test_is_valid_float_type" >:: test_is_valid_float_type;
   "test_is_not_valid_float_type" >:: test_is_not_valid_float_type;
   "test_is_valid_char_type" >:: test_is_valid_char_type;
   "test_is_not_valid_char_type" >:: test_is_not_valid_char_type;
   "test_is_valid_string_type" >:: test_is_valid_string_type;
   "test_is_valid_bool_type" >:: test_is_valid_bool_type;
   "test_is_not_valid_bool_type" >:: test_is_not_valid_bool_type]
;;

