open OUnit

module Concrete_page_parameters =
  Test_types.Concrete_page_parameters
;;

module Concrete_page_behavior =
  Test_page_behavior.Concrete_page_behavior
;;

module Concrete_error_page =
  Srv_standard_error_page.Make
    (Concrete_page_parameters)
    (Concrete_page_behavior)
;;

module Concrete_page_factory_builder =
struct
  type t = Concrete_page_behavior.Factory.t

  let create () =
    let repository = Concrete_page_behavior.Factory.create () in
    let repository = Concrete_page_behavior.Factory.add_page_constructor
                       repository
                       "test_page.xnunc"
                       (new Test_page_behavior.test_page) in
      Concrete_page_behavior.Factory.add_page_constructor
                       repository
                       "standard_error_page.xnunc"
                       (new Concrete_error_page.standard_error_page)

end;;

(* tests *)
let test_create _ =
  let cgi = As_test_helper.create_test_activation [] in
  let resource_list = Srv_resource_list.create
                        [
                          { Srv_resource_list.name = "test_page.xnunc";
                            Srv_resource_list.resource_type = "xnc:page"
                          }
                        ] in
  let serialized_state = Srv_state_persisting.Serialized.retrieve_state
                           cgi
                           Srv_configuration.default_conf
                           resource_list in
  let page_factory = Concrete_page_factory_builder.create () in
  let page_constructor =
    Concrete_page_behavior.Factory.find_page_constructor
      page_factory "test_page.xnunc" in
  let page = page_constructor
               { Srv_page_state.cgi = cgi;
                 Srv_page_state.state_serialization = serialized_state;
                 Srv_page_state.is_self_submitted = false
               } in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected page name"
      "test_page.xnunc"
      (page # get_name);
;;

(* test suite *)
let suite = "Page factory builder test" >:::
  ["test_create" >:: test_create]
;;

