open OUnit

(* tests *)
let test_render_name_value_table _ =
  let orig_list = [("a", 1); ("b", 2); ("c", 3)] in
  let name_value_list_builder () =
    List.map
      (fun (n, v) -> (n, string_of_int (succ v)))
      orig_list in
  let e = new Srv_page_helper.name_value_table "Test" name_value_list_builder in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected name-value table element rendered output"
      "<table>\
         <caption>Test</caption>\
         <col class=\"name\" />\
         <col class=\"value\" />\
         <tr>\
           <th>Name</th>\
           <th>Value</th>\
         </tr>\
         <tr class=\"even\">\
           <td>a</td>\
           <td>2</td>\
         </tr>\
         <tr class=\"odd\">\
           <td>b</td>\
           <td>3</td>\
         </tr>\
         <tr class=\"even\">\
           <td>c</td>\
           <td>4</td>\
         </tr>\
       </table>"
      (As_test_helper.render_element e)
;;

let test_render_trace_table _ =
  let logger = Srv_page_helper.create_page_logger () in
  let cgi = As_test_helper.create_test_activation [] in
  let e = new Srv_page_helper.trace_table cgi logger in
    Xnc_logger.warn logger "Warning!";
    OUnit.assert_bool
      "Log HTML output does not match"
      (Str.string_match
         (Str.regexp
            "^<table id=\"pagetrace\">\
                <caption>Trace</caption>\
                <col class=\"date\" />\
                <col class=\"level\" />\
                <col class=\"message\" />\
                <col class=\"elapsed\" />\
                <tr>\
                  <th>Date</th>\
                  <th>Level</th>\
                  <th>Message</th>\
                  <th>Elapsed (sec)</th>\
                </tr>\
                <tr class=\"warn\">\
                  <td>\
                    [0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9] \
                    [0-9][0-9]:[0-9][0-9]:[0-9][0-9]\
                  </td>\
                  <td>warn</td>\
                  <td>Warning!</td>\
                  <td>\
                    [0-9]\\.[0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9][0-9]\
                  </td>\
                </tr>\
              </table>$")
         (As_test_helper.render_element e)
         0
      )
;;

(* test suite *)
let suite = "Page helper test" >:::
  ["test_render_name_value_table" >:: test_render_name_value_table;
   "test_render_trace_table" >:: test_render_trace_table]
;;

