open OUnit

module Concrete_context =
  Test_runtime_context.Runtime_context
;;

module Concrete_session_data =
  Test_types.Concrete_session_data
;;

module Concrete_page_parameters =
  Test_types.Concrete_page_parameters
;;

module Concrete_page_behavior =
  Test_page_behavior.Concrete_page_behavior
;;

module Concrete_page_state_machine =
  Srv_page_state_machine.Make
    (Concrete_page_behavior)
;;

class test_page
  page_context =
object(self)
  inherit Concrete_page_behavior.page_behavior_base
        "test_page.xnunc"
        Concrete_page_parameters.No_parameters
        page_context

  method build_validator_collection () =
    ()
  method read_user_input () =
    ()
  method build_page_structure () =
    ()
  method render () =
    ()
end;;

(* fixture setup and teardown *)
let set_up _ =
  let context = Concrete_context.get_current () in
  let logger = Xnc_logger.create () in
  let context' = { context with
                       Srv_runtime_context.logger = logger
                 } in
  let () = Concrete_context.update_current context' in
  let context = Concrete_context.get_current () in
  let config = context.Srv_runtime_context.configuration in
  let curr_page = Netcgi.Argument.simple
                    (Srv_configuration.current_page_field_name config)
                    "test_page.xnunc" in
  let state = Netcgi.Argument.simple
                    (Srv_configuration.session_field_name config)
                    (Xnc_field_collection.to_string
                       (Concrete_session_data.to_field_collection
                          (Concrete_session_data.create ()))) in
  let page_params = Netcgi.Argument.simple
                      (Srv_configuration.page_parameters_field_name config)
                      (Xnc_field_collection.to_string
                         (Concrete_page_parameters.to_field_collection
                            Concrete_page_parameters.No_parameters)) in
  let cgi = As_test_helper.create_test_activation
              [curr_page;
               page_params;
               state] in
  let resource_list = Srv_resource_list.create
                        [
                          { Srv_resource_list.name = "test_page.xnunc";
                            Srv_resource_list.resource_type = "xnc:page"
                          }
                        ] in
  let serialized_state = Srv_state_persisting.Serialized.retrieve_state
                           cgi
                           config
                           resource_list in
  let repository = Concrete_page_behavior.Factory.create () in
  let repository = Concrete_page_behavior.Factory.add_page_constructor
                     repository
                     "test_page.xnunc"
                     (new test_page) in
    (repository, logger, cgi, serialized_state);;

let tear_down _ =
  ()
;;

(* tests *)
let test_create (repository, logger, cgi, serialized_state) =
  let state_machine = Concrete_page_state_machine.create
                        repository
                        logger
                        cgi
                        serialized_state
                        "test_page.xnunc"
                        (Some Concrete_page_parameters.No_parameters)
                        true
                        false in
    OUnit.assert_equal
      ~msg:"Expected Uninitialized state"
      Concrete_page_state_machine.Uninitialized
      (Concrete_page_state_machine.current_state state_machine)
;;

let test_do_transition_first_call_start_page
      (repository, logger, cgi, serialized_state) =
  let state_machine = Concrete_page_state_machine.create
                        repository
                        logger
                        cgi
                        serialized_state
                        "test_page.xnunc"
                        (Some Concrete_page_parameters.No_parameters)
                        false
                        false in
    OUnit.assert_raises
      ~msg:"Exception Invalid_transition not raised"
      Srv_page_state_machine.Invalid_transition
      (fun () -> Concrete_page_state_machine.do_transition
                   state_machine
                   Concrete_page_state_machine.Render);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Initialize in
    OUnit.assert_equal
      ~msg:"Expected Initialized state"
      Concrete_page_state_machine.Initialized
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Init_page_parameters in
    OUnit.assert_equal
      ~msg:"Expected State_loaded state"
      Concrete_page_state_machine.State_loaded
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Build_page_structure in
    OUnit.assert_equal
      ~msg:"Expected Ready_to_save_state state"
      Concrete_page_state_machine.Ready_to_save_state
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Save_state in
    OUnit.assert_equal
      ~msg:"Expected State_saved state"
      Concrete_page_state_machine.State_saved
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Handle_page_change in
    OUnit.assert_equal
      ~msg:"Expected Ready_to_render state"
      Concrete_page_state_machine.Ready_to_render
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Render in
    OUnit.assert_equal
      ~msg:"Expected Rendered state"
      Concrete_page_state_machine.Rendered
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Complete in
    OUnit.assert_equal
      ~msg:"Unexpected Completed state"
      Concrete_page_state_machine.Completed
      (Concrete_page_state_machine.current_state state_machine);
;;

let test_do_transition_first_call (repository, logger, cgi, serialized_state) =
  let state_machine = Concrete_page_state_machine.create
                        repository
                        logger
                        cgi
                        serialized_state
                        "test_page.xnunc"
                        (Some Concrete_page_parameters.No_parameters)
                        false
                        true in
    OUnit.assert_raises
      ~msg:"Exception Invalid_transition not raised"
      Srv_page_state_machine.Invalid_transition
      (fun () -> Concrete_page_state_machine.do_transition
                   state_machine
                   Concrete_page_state_machine.Render);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Initialize in
    OUnit.assert_equal
      ~msg:"Expected Ready_to_load_session state"
      Concrete_page_state_machine.Ready_to_load_session
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Load_session in
    OUnit.assert_equal
      ~msg:"Expected Initialized state"
      Concrete_page_state_machine.Initialized
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Init_page_parameters in
    OUnit.assert_equal
      ~msg:"Expected State_loaded state"
      Concrete_page_state_machine.State_loaded
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Build_page_structure in
    OUnit.assert_equal
      ~msg:"Expected Ready_to_save_state state"
      Concrete_page_state_machine.Ready_to_save_state
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Save_state in
    OUnit.assert_equal
      ~msg:"Expected State_saved state"
      Concrete_page_state_machine.State_saved
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Handle_page_change in
    OUnit.assert_equal
      ~msg:"Expected Ready_to_render state"
      Concrete_page_state_machine.Ready_to_render
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Render in
    OUnit.assert_equal
      ~msg:"Expected Rendered state"
      Concrete_page_state_machine.Rendered
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Complete in
    OUnit.assert_equal
      ~msg:"Unexpected Completed state"
      Concrete_page_state_machine.Completed
      (Concrete_page_state_machine.current_state state_machine);
;;

let test_do_transition_post_back (repository, logger, cgi, serialized_state) =
  let state_machine = Concrete_page_state_machine.create
                        repository
                        logger
                        cgi
                        serialized_state
                        "test_page.xnunc"
                        None
                        true
                        true in
    OUnit.assert_raises
      ~msg:"Exception Invalid_transition not raised"
      Srv_page_state_machine.Invalid_transition
      (fun () -> Concrete_page_state_machine.do_transition
                   state_machine
                   Concrete_page_state_machine.Render);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Initialize in
    OUnit.assert_equal
      ~msg:"Expected Ready_to_load_session state"
      Concrete_page_state_machine.Ready_to_load_session
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Load_session in
    OUnit.assert_equal
      ~msg:"Expected Initialized state"
      Concrete_page_state_machine.Initialized
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Load_page_parameters in
    OUnit.assert_equal
      ~msg:"Expected Ready_to_validate_user_input state"
      Concrete_page_state_machine.Ready_to_validate_user_input
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Validate_user_input in
    OUnit.assert_equal
      ~msg:"Expected Ready_to_read_user_input state"
      Concrete_page_state_machine.Ready_to_read_user_input
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Read_user_input in
    OUnit.assert_equal
      ~msg:"Expected State_loaded state"
      Concrete_page_state_machine.State_loaded
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Build_page_structure in
    OUnit.assert_equal
      ~msg:"Expected Ready_to_handle_ui_events state"
      Concrete_page_state_machine.Ready_to_handle_ui_events
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Handle_ui_events in
    OUnit.assert_equal
      ~msg:"Expected Ready_to_save_state state"
      Concrete_page_state_machine.Ready_to_save_state
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Save_state in
    OUnit.assert_equal
      ~msg:"Expected State_saved state"
      Concrete_page_state_machine.State_saved
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Handle_page_change in
    OUnit.assert_equal
      ~msg:"Expected Ready_to_render state"
      Concrete_page_state_machine.Ready_to_render
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Render in
    OUnit.assert_equal
      ~msg:"Expected Rendered state"
      Concrete_page_state_machine.Rendered
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.do_transition
                        state_machine
                        Concrete_page_state_machine.Complete in
    OUnit.assert_equal
      ~msg:"Unexpected Completed state"
      Concrete_page_state_machine.Completed
      (Concrete_page_state_machine.current_state state_machine);
;;

let test_next_step (repository, logger, cgi, serialized_state) =
  let state_machine = Concrete_page_state_machine.create
                        repository
                        logger
                        cgi
                        serialized_state
                        "test_page.xnunc"
                        (Some Concrete_page_parameters.No_parameters)
                        false
                        false in
    OUnit.assert_equal
      ~msg:"Expected Uninitialized state"
      Concrete_page_state_machine.Uninitialized
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.next_step state_machine in
    OUnit.assert_equal
      ~msg:"Expected Initialized state"
      Concrete_page_state_machine.Initialized
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.next_step state_machine in
    OUnit.assert_equal
      ~msg:"Expected State_loaded state"
      Concrete_page_state_machine.State_loaded
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.next_step state_machine in
    OUnit.assert_equal
      ~msg:"Expected Ready_to_save_state state"
      Concrete_page_state_machine.Ready_to_save_state
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.next_step state_machine in
    OUnit.assert_equal
      ~msg:"Expected State_saved state"
      Concrete_page_state_machine.State_saved
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.next_step state_machine in
    OUnit.assert_equal
      ~msg:"Expected Ready_to_render state"
      Concrete_page_state_machine.Ready_to_render
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.next_step state_machine in
    OUnit.assert_equal
      ~msg:"Expected Rendered state"
      Concrete_page_state_machine.Rendered
      (Concrete_page_state_machine.current_state state_machine);
  let state_machine = Concrete_page_state_machine.next_step state_machine in
    OUnit.assert_equal
      ~msg:"Unexpected Completed state"
      Concrete_page_state_machine.Completed
      (Concrete_page_state_machine.current_state state_machine);
    OUnit.assert_raises
      ~msg:"Exception Final_state_reached not raised"
      Srv_page_state_machine.Final_state_reached
      (fun () -> Concrete_page_state_machine.next_step state_machine);
;;

(* test suite *)
let suite = "Page state machine" >:::
  ["test_create" >:: (bracket set_up test_create tear_down);
   "test_do_transition_first_call_start_page"
     >:: (bracket set_up test_do_transition_first_call_start_page tear_down);
   "test_do_transition_first_call"
     >:: (bracket set_up test_do_transition_first_call tear_down);
   "test_do_transition_post_back"
     >:: (bracket set_up test_do_transition_post_back tear_down);
   "test_next_step" >:: (bracket set_up test_next_step tear_down)]
;;

