open OUnit

(* tests *)
let test_get_name_list _ =
  let resource_list =
    Srv_resource_list.create
      [
        { Srv_resource_list.name = "page1";
          Srv_resource_list.resource_type = "xnc:page"
        };
        { Srv_resource_list.name = "page2";
          Srv_resource_list.resource_type = "xnc:page"
        };
        { Srv_resource_list.name = "control1";
          Srv_resource_list.resource_type = "xnc:control"
        };
      ] in
  let name_list = Srv_resource_list.get_name_list resource_list in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected name list"
      [ "page1"; "page2"; "control1" ]
      name_list;
;;

let test_get_filtered_name_list _ =
  let resource_list =
    Srv_resource_list.create
      [
        { Srv_resource_list.name = "page1";
          Srv_resource_list.resource_type = "xnc:page"
        };
        { Srv_resource_list.name = "page2";
          Srv_resource_list.resource_type = "xnc:page"
        };
        { Srv_resource_list.name = "control1";
          Srv_resource_list.resource_type = "xnc:control"
        };
      ] in
  let name_list =
    Srv_resource_list.get_filtered_name_list
      (fun r -> r.Srv_resource_list.resource_type = "xnc:control")
      resource_list
  in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected name list"
      [ "control1" ]
      name_list;
;;

let test_get_page_name_list _ =
  let resource_list =
    Srv_resource_list.create
      [
        { Srv_resource_list.name = "page1";
          Srv_resource_list.resource_type = "xnc:page"
        };
        { Srv_resource_list.name = "page2";
          Srv_resource_list.resource_type = "xnc:page"
        };
        { Srv_resource_list.name = "control1";
          Srv_resource_list.resource_type = "xnc:control"
        };
      ] in
  let name_list =
    Srv_resource_list.get_page_name_list
      resource_list
  in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected name list"
      [ "page1"; "page2" ]
      name_list;
;;

(* test suite *)
let suite = "Resource list test" >:::
  ["test_get_name_list" >:: test_get_name_list;
   "test_get_filtered_name_list" >:: test_get_filtered_name_list;
   "test_get_page_name_list" >:: test_get_page_name_list]
;;

