open OUnit

module Runtime_context =
  Srv_runtime_context.Make
    (Xnc_core.Null_mutex)
    (Xnc_scheduler.Null)
;;

(* fixture setup and teardown *)
let set_up _ =
  ()
;;

let tear_down _ =
  Runtime_context.clear ()
;;

(* tests *)
let test_get_current _ =
  let context = Runtime_context.get_current () in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected default application name"
      "demo.fcgi"
      context.
       Srv_runtime_context.command_line.
       Srv_command_line.application_name;
;;

let test_update_current _ =
  let context = Runtime_context.get_current () in
  let context =
    { context with
          Srv_runtime_context.command_line =
            { Srv_command_line.default_args with
                  Srv_command_line.application_name = "test.fcgi"
            }
    } in
  let () = Runtime_context.update_current context in
  let context = Runtime_context.get_current () in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected updated application name"
      "test.fcgi"
      context.
       Srv_runtime_context.command_line.
       Srv_command_line.application_name;
;;

let test_clear _ =
  let context = Runtime_context.get_current () in
  let context =
    { context with
          Srv_runtime_context.command_line =
            { Srv_command_line.default_args with
                  Srv_command_line.application_name = "test.fcgi"
            }
    } in
  let () = Runtime_context.update_current context in
  let () = Runtime_context.clear () in
  let context = Runtime_context.get_current () in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected default application name"
      "demo.fcgi"
      context.
       Srv_runtime_context.command_line.
       Srv_command_line.application_name;
;;

let test_fc_rep_get _ =
  let counter = ref 0 in
  let create () =
    let fc = Xnc_field_collection.create "Test" in
      incr counter;
      Xnc_field_collection.register_int fc "int" !counter;
      fc in
  let fc = Runtime_context.Field_collection_repository.get create "fc" in
  let fc' = Runtime_context.Field_collection_repository.get create "fc" in
  let ofc = Runtime_context.Field_collection_repository.get create "ofc" in
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected registered int value"
      1
      (Xnc_field_collection.get_int fc "int");
    OUnit.assert_bool
      "fc should be phisically equal to fc'"
      (fc == fc');
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected registered int value"
      2
      (Xnc_field_collection.get_int ofc "int");
    OUnit.assert_bool
      "ofc should be phisically different from fc'"
      (ofc != fc');
;;

let test_fc_rep_clear _ =
  let counter = ref 0 in
  let create () =
    let fc = Xnc_field_collection.create "Test" in
      incr counter;
      Xnc_field_collection.register_int fc "int" !counter;
      fc in
  let fc = Runtime_context.Field_collection_repository.get create "fc" in
  let () = Runtime_context.clear () in
  let fc' = Runtime_context.Field_collection_repository.get create "fc" in
  let () = Runtime_context.clear () in
  let fc'' = Runtime_context.Field_collection_repository.get create "fc" in
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected registered int value"
      1
      (Xnc_field_collection.get_int fc "int");
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected registered int value"
      2
      (Xnc_field_collection.get_int fc' "int");
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected registered int value"
      3
      (Xnc_field_collection.get_int fc'' "int");
;;

(* test suite *)
let suite = "Runtime context test" >:::
  ["test_get_current" >:: (bracket set_up test_get_current tear_down);
   "test_update_current" >:: (bracket set_up test_update_current tear_down);
   "test_clear" >:: (bracket set_up test_clear tear_down);
   "test_fc_rep_get" >:: (bracket set_up test_fc_rep_get tear_down);
   "test_fc_rep_clear" >:: (bracket set_up test_fc_rep_clear tear_down)]
;;

