open OUnit

module Concrete_session_data =
  Test_types.Concrete_session_data
;;

module Concrete_page_parameters =
  Test_types.Concrete_page_parameters
;;

let resource_list =
  Srv_resource_list.create
    [
      { Srv_resource_list.name = "test_page.xnunc";
        Srv_resource_list.resource_type = "xnc:page"
      };
      { Srv_resource_list.name = "param";
        Srv_resource_list.resource_type = "xnc:page"
      }
    ]
;;

(* tests *)
let test_retrieve_serialized_state_argument_value _ =
  let config = Srv_configuration.default_conf in
  let page_name = Netcgi.Argument.simple
                    (Srv_configuration.current_page_field_name config)
                    "test_page.xnunc" in
  let session = Netcgi.Argument.simple
                  (Srv_configuration.session_field_name config)
                  "Session=[a_int=42;\
                            a_float=3.14;\
                            a_char=a;\
                            a_string=&quot;test&quot;;\
                            a_bool=true]" in
  let page_parameters = Netcgi.Argument.simple
                          (Srv_configuration.page_parameters_field_name config)
                          "Parameters=[integer=10;boolean=false]" in
  let cgi = As_test_helper.create_test_activation
              [page_name;
               session;
               page_parameters] in
  let serialized_state =
    Srv_state_persisting.Serialized.retrieve_state
      cgi
      config
      resource_list in
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state
            Srv_state_persisting.Page_name in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected page name location"
      Srv_state_persisting.Argument_value
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected page name serialized state"
      "test_page.xnunc"
      s.Srv_state_persisting.Serialized.content;
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state Srv_state_persisting.Page_parameters in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected page parameter location"
      Srv_state_persisting.Argument_value
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected page parameter serialized state"
      "Parameters=[integer=10;boolean=false]"
      s.Srv_state_persisting.Serialized.content;
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state Srv_state_persisting.Session in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected session location"
      Srv_state_persisting.Argument_value
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected session serialized state"
      "Session=[a_int=42;\
                a_float=3.14;\
                a_char=a;\
                a_string=&quot;test&quot;;\
                a_bool=true]"
      s.Srv_state_persisting.Serialized.content;
;;

let test_retrieve_serialized_state_path_info _ =
  let config = Srv_configuration.default_conf in
  let cgi = As_test_helper.create_test_activation
              ~path_info:"/test_page.xnunc/\
                             S42,3.14,a,test,true/\
                             12/false"
              [] in
  let serialized_state =
    Srv_state_persisting.Serialized.retrieve_state
      cgi
      config
      resource_list in
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state
            Srv_state_persisting.Page_name
  in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected page name location"
      Srv_state_persisting.Path_info
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected page name serialized state"
      "test_page.xnunc"
      s.Srv_state_persisting.Serialized.content;
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state Srv_state_persisting.Page_parameters in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected page parameter location"
      Srv_state_persisting.Path_info
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected page parameter serialized state"
      "12/false"
      s.Srv_state_persisting.Serialized.content;
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state Srv_state_persisting.Session in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected session location"
      Srv_state_persisting.Path_info
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected session serialized state"
      "42,3.14,a,test,true"
      s.Srv_state_persisting.Serialized.content;
;;

let test_retrieve_serialized_state_script_name _ =
  let config = Srv_configuration.default_conf in
  let cgi = As_test_helper.create_test_activation
              ~script_name:"/ex-nunc/\
                             test_page.xnunc/\
                             S42,3.14,a,test,true/\
                             12/false/param"
              ~server_software:"nethttpd"
              [] in
  let serialized_state =
    Srv_state_persisting.Serialized.retrieve_state
      cgi
      config
      resource_list in
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state
            Srv_state_persisting.Page_name
  in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected page name location"
      Srv_state_persisting.Path_info
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected page name serialized state"
      "test_page.xnunc"
      s.Srv_state_persisting.Serialized.content;
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state Srv_state_persisting.Page_parameters in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected page parameter location"
      Srv_state_persisting.Path_info
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected page parameter serialized state"
      "12/false/param"
      s.Srv_state_persisting.Serialized.content;
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state Srv_state_persisting.Session in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected session location"
      Srv_state_persisting.Path_info
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected session serialized state"
      "42,3.14,a,test,true"
      s.Srv_state_persisting.Serialized.content;
;;

let test_retrieve_serialized_state_script_name_without_session _ =
  let config = Srv_configuration.create
                 ~session_manager_mode:Srv_types.Off () in
  let cgi = As_test_helper.create_test_activation
              ~script_name:"/ex-nunc/\
                             test_page.xnunc/\
                             12/false/param"
              ~server_software:"nethttpd"
              [] in
  let serialized_state =
    Srv_state_persisting.Serialized.retrieve_state
      cgi
      config
      resource_list in
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state
            Srv_state_persisting.Page_name
  in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected page name location"
      Srv_state_persisting.Path_info
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected page name serialized state"
      "test_page.xnunc"
      s.Srv_state_persisting.Serialized.content;
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state Srv_state_persisting.Page_parameters in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected page parameter location"
      Srv_state_persisting.Path_info
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected page parameter serialized state"
      "12/false/param"
      s.Srv_state_persisting.Serialized.content;
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state Srv_state_persisting.Session in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected session location"
      Srv_state_persisting.Not_present
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected session serialized state"
      ""
      s.Srv_state_persisting.Serialized.content;
;;

let test_retrieve_serialized_state_mixed_location _ =
  let config = Srv_configuration.default_conf in
  let page_parameters = Netcgi.Argument.simple
                          (Srv_configuration.page_parameters_field_name config)
                          "Parameters=[integer=15;boolean=true]" in
  let cgi = As_test_helper.create_test_activation
              ~path_info:"/test_page.xnunc/\
                             S42,3.14,a,test,true"
              [page_parameters] in
  let serialized_state =
    Srv_state_persisting.Serialized.retrieve_state
      cgi
      config
      resource_list in
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state Srv_state_persisting.Page_name in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected page name location"
      Srv_state_persisting.Path_info
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected page name serialized state"
      "test_page.xnunc"
      s.Srv_state_persisting.Serialized.content;
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state Srv_state_persisting.Page_parameters in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected page parameter location"
      Srv_state_persisting.Argument_value
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected page parameter serialized state"
      "Parameters=[integer=15;boolean=true]"
      s.Srv_state_persisting.Serialized.content;
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state Srv_state_persisting.Session in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected session location"
      Srv_state_persisting.Path_info
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected session serialized state"
      "42,3.14,a,test,true"
      s.Srv_state_persisting.Serialized.content;
;;

let test_retrieve_serialized_state_path_info_with_cookie _ =
  let config = Srv_configuration.create
                 ~session_persistence_mode:Srv_types.Cookie () in
  let cgi = As_test_helper.create_test_activation
              ~path_info:"/test_page.xnunc/\
                             12/false"
              [] in
  let serialized_state =
    Srv_state_persisting.Serialized.retrieve_state
      cgi
      config
      resource_list in
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state
            Srv_state_persisting.Page_name
  in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected page name location"
      Srv_state_persisting.Path_info
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected page name serialized state"
      "test_page.xnunc"
      s.Srv_state_persisting.Serialized.content;
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state Srv_state_persisting.Page_parameters in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected page parameter location"
      Srv_state_persisting.Path_info
      s.Srv_state_persisting.Serialized.location;
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected page parameter serialized state"
      "12/false"
      s.Srv_state_persisting.Serialized.content;
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state Srv_state_persisting.Session in
    OUnit.assert_equal
      ~printer:Std.dump
      ~msg:"Unexpected session location"
      Srv_state_persisting.Not_present
      s.Srv_state_persisting.Serialized.location;
;;

let test_get_serializer _ =
  let config = Srv_configuration.default_conf in
  let serialized_state =
    { Srv_state_persisting.Serialized.part =
        Srv_state_persisting.Page_parameters;
      Srv_state_persisting.Serialized.location =
        Srv_state_persisting.Path_info;
      Srv_state_persisting.Serialized.content =
        "10/false"
    } in
  let serializer = Srv_state_persisting.get_serializer
                     serialized_state.Srv_state_persisting.Serialized.location
                     serialized_state.Srv_state_persisting.Serialized.part
                     config in
  let fc = Concrete_page_parameters.to_field_collection
             (Concrete_page_parameters.Test_page_parameters
                (0, true)) in
    Xnc_field_collection.from_string
      ~serializer
      fc
      serialized_state.Srv_state_persisting.Serialized.content;
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected integer field"
      10
      (Xnc_field_collection.get_int fc "integer");
    OUnit.assert_equal
      ~printer:string_of_bool
      ~msg:"Unexpected boolean field"
      false
      (Xnc_field_collection.get_bool fc "boolean");
;;

let test_serialize_state _ =
  let config = Srv_configuration.default_conf in
  let page_param_fc = Concrete_page_parameters.to_field_collection
                        (Concrete_page_parameters.Test_page_parameters
                           (10, false)) in
  let session_fc = Concrete_session_data.to_field_collection
                     (Concrete_session_data.create ()) in
  let deserialized_state = Srv_state_persisting.Deserialized.create
    [ { Srv_state_persisting.Deserialized.part =
          Srv_state_persisting.Page_name;
        Srv_state_persisting.Deserialized.location =
          Srv_state_persisting.Argument_value;
        Srv_state_persisting.Deserialized.content =
          Srv_state_persisting.Deserialized.String "test_page.xnunc"
      };
      { Srv_state_persisting.Deserialized.part =
          Srv_state_persisting.Session;
        Srv_state_persisting.Deserialized.location =
          Srv_state_persisting.Path_info;
        Srv_state_persisting.Deserialized.content =
          Srv_state_persisting.Deserialized.Field_collection session_fc
      };
      { Srv_state_persisting.Deserialized.part =
          Srv_state_persisting.Page_parameters;
        Srv_state_persisting.Deserialized.location =
          Srv_state_persisting.Path_info;
        Srv_state_persisting.Deserialized.content =
          Srv_state_persisting.Deserialized.Field_collection page_param_fc
      } ] in
  let serialized_state = Srv_state_persisting.Deserialized.serialize_state
                           deserialized_state
                           config in
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state Srv_state_persisting.Page_name in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected serialized page name"
      "test_page.xnunc"
      s.Srv_state_persisting.Serialized.content;
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state Srv_state_persisting.Page_parameters in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected serialized page parameters"
      "10/false"
      s.Srv_state_persisting.Serialized.content;
  let s = Srv_state_persisting.Serialized.get_state_part
            serialized_state Srv_state_persisting.Session in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected serialized session"
      "42,3.14,a,test,true"
      s.Srv_state_persisting.Serialized.content;
;;

let test_build_url _ =
  let config = Srv_configuration.default_conf in
  let cgi = As_test_helper.create_test_activation
              ~script_name:"/ex-nunc/\
                             test_page.xnunc/\
                             S42,3.14,a,test,true/\
                             12/false"
              ~server_software:"nethttpd"
              [] in
  let serialized_state = Srv_state_persisting.Serialized.create
    [ { Srv_state_persisting.Serialized.part =
          Srv_state_persisting.Page_name;
        Srv_state_persisting.Serialized.location =
          Srv_state_persisting.Path_info;
        Srv_state_persisting.Serialized.content =
          "test_page.xnunc"
      };
      { Srv_state_persisting.Serialized.part =
          Srv_state_persisting.Session;
        Srv_state_persisting.Serialized.location =
          Srv_state_persisting.Path_info;
        Srv_state_persisting.Serialized.content =
          "42,3.14,a,test,true"
      };
      { Srv_state_persisting.Serialized.part =
          Srv_state_persisting.Page_parameters;
        Srv_state_persisting.Serialized.location =
          Srv_state_persisting.Path_info;
        Srv_state_persisting.Serialized.content =
          "10/false"
      } ] in
  let url = Srv_state_persisting.Serialized.build_url
              serialized_state
              cgi
              config in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected URL"
      "http:///ex-nunc/test_page.xnunc/S42,3.14,a,test,true/10/false"
      url;
;;

let test_build_url_without_session _ =
  let config = Srv_configuration.create
                 ~session_manager_mode:Srv_types.Off () in
  let cgi = As_test_helper.create_test_activation
              ~script_name:"/ex-nunc/\
                             test_page.xnunc/\
                             12/false"
              ~server_software:"nethttpd"
              [] in
  let serialized_state = Srv_state_persisting.Serialized.create
    [ { Srv_state_persisting.Serialized.part =
          Srv_state_persisting.Page_name;
        Srv_state_persisting.Serialized.location =
          Srv_state_persisting.Path_info;
        Srv_state_persisting.Serialized.content =
          "test_page.xnunc"
      };
      { Srv_state_persisting.Serialized.part =
          Srv_state_persisting.Session;
        Srv_state_persisting.Serialized.location =
          Srv_state_persisting.Not_present;
        Srv_state_persisting.Serialized.content =
          ""
      };
      { Srv_state_persisting.Serialized.part =
          Srv_state_persisting.Page_parameters;
        Srv_state_persisting.Serialized.location =
          Srv_state_persisting.Path_info;
        Srv_state_persisting.Serialized.content =
          "10/false"
      } ] in
  let url = Srv_state_persisting.Serialized.build_url
              serialized_state
              cgi
              config in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected URL"
      "http:///ex-nunc/test_page.xnunc/10/false"
      url;
;;

(* test suite *)
let suite = "State persisting test" >:::
  ["test_retrieve_serialized_state_argument_value"
     >:: test_retrieve_serialized_state_argument_value;
   "test_retrieve_serialized_state_path_info"
     >:: test_retrieve_serialized_state_path_info;
   "test_retrieve_serialized_state_script_name"
     >:: test_retrieve_serialized_state_script_name;
   "test_retrieve_serialized_state_script_name_without_session"
     >:: test_retrieve_serialized_state_script_name_without_session;
   "test_retrieve_serialized_state_mixed_location"
     >:: test_retrieve_serialized_state_mixed_location;
   "test_retrieve_serialized_state_path_info_with_cookie"
     >:: test_retrieve_serialized_state_path_info_with_cookie;
   "test_get_serializer"
     >:: test_get_serializer;
   "test_serialize_state"
     >:: test_serialize_state;
   "test_build_url_without_session"
     >:: test_build_url_without_session;
   "test_build_url"
     >:: test_build_url]
;;

