open OUnit

(* tests *)
let test_count _ =
  let c = Srv_validator_collection.create () in
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected number of validators"
      0
      (Srv_validator_collection.count c)
;;

let test_add_validator _ =
  let validator = new Srv_input_validator.required_field_validator
                    "test"
                    "Error" in
  let c = Srv_validator_collection.create () in
    Srv_validator_collection.add_validator c
      "test_field"
      validator;
    OUnit.assert_equal
      ~printer:string_of_int
      ~msg:"Unexpected number of validators"
      1
      (Srv_validator_collection.count c)
;;

let test_is_valid _ =
  let c = Srv_validator_collection.create () in
    Srv_validator_collection.add_validator c
      "test_field1"
      (new Srv_input_validator.required_field_validator
         "test"
         "Error 1");
    Srv_validator_collection.add_validator c
      "test_field2"
      (new Srv_input_validator.required_field_validator
         "10"
         "Error 2");
    OUnit.assert_bool
      "Validation of test_field1 (valid) and test_field2 (valid) \
       should not fail"
      (Srv_validator_collection.is_valid c)
;;

let test_is_not_valid _ =
  let c = Srv_validator_collection.create () in
    Srv_validator_collection.add_validator c
      "test_field1"
      (new Srv_input_validator.required_field_validator
         ""
         "Error 1");
    Srv_validator_collection.add_validator c
      "test_field2"
      (new Srv_input_validator.required_field_validator
         "10"
         "Error 2");
    OUnit.assert_equal
      ~printer:string_of_bool
      ~msg:"Validation of test_field1 (valid) and test_field2 (invalid) \
            should fail"
      false
      (Srv_validator_collection.is_valid c)
;;

let test_error_message_list _ =
  let c = Srv_validator_collection.create () in
    Srv_validator_collection.add_validator c
      "test_field1"
      (new Srv_input_validator.required_field_validator
         ""
         "Error 1");
    Srv_validator_collection.add_validator c
      "test_field2"
      (new Srv_input_validator.required_field_validator
         "10"
         "Error 2");
    Srv_validator_collection.add_validator c
      "test_field3"
      (new Srv_input_validator.required_field_validator
         ""
         "Error 3");
    OUnit.assert_equal
      ~msg:"Unexpected error message list"
      ["Error 1"; "Error 3"]
      (Srv_validator_collection.error_message_list c)
;;

let test_find_validator _ =
  let c = Srv_validator_collection.create () in
  let () = Srv_validator_collection.add_validator c
             "test_field1"
             (new Srv_input_validator.required_field_validator
                ""
                "Error 1") in
  let () = Srv_validator_collection.add_validator c
             "test_field2"
             (new Srv_input_validator.required_field_validator
                "10"
                "Error 2") in
  let () = Srv_validator_collection.add_validator c
             "test_field3"
             (new Srv_input_validator.required_field_validator
                ""
                "Error 3") in
  let validator = Srv_validator_collection.find_validator c "test_field2" in
    OUnit.assert_equal
      ~printer:Std.identity
      ~msg:"Unexpected error message for test_field2"
      "Error 2"
      (validator # get_error_message)
;;

(* test suite *)
let suite = "Validator collection test" >:::
  ["test_count" >:: test_count;
   "test_add_validator" >:: test_add_validator;
   "test_is_valid" >:: test_is_valid;
   "test_is_not_valid" >:: test_is_not_valid;
   "test_error_message_list" >:: test_error_message_list;
   "test_find_validator" >:: test_find_validator]
;;

