#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import os
import glob
import shutil

sys.path.append("build")
import build_lib
from build_lib import *


#############
# FUNCTIONS #
#############
def print_help():
    print """This is the Ex-nunc build system, you are welcome.
Usage: build.py target [target2 target3...]

Available targets:

Cleaning targets:
  clean     - remove generated files

Compilation  targets:
  libs           - compiles Ex-nunc libraries (native and bytecode, NO INSTALL)
  install.libs   - INSTALL Ex-nunc libraries (native and bytecode)
  install        - INSTALL Ex-nunc libraries (native and bytecode) and
                    compiler (native). See --install-dir option
  uninstall.libs - UNINSTALL Ex-nunc libraries (native and bytecode)
  uninstall      - UNINSTALL Ex-nunc libraries (native and bytecode) and
                    compiler. See --install-dir option
  compiler       - compiles Ex-nunc compiler (native ONLY)
  all            - compiles everything both native and bytecode (NO INSTALL)
  test           - compiles and runs all the test suites (native AND bytecode)
  doc            - generates the project documentation with ocamldoc
  help           - show this help message

Options:
  --install-dir <dir>   - install the compiler executable in "dir".
                            (default: /usr/local/bin)
  --help                - show this help message"""

    sys.exit(0)

def clean_xnc_core():
    def job():
        print_flush("--- cleaning xnc_core")
        run("./build.py clean")
    enter_dir_and_do("xnc_core", job)

def clean_serialization():
    def job():
        print_flush("--- cleaning serialization")
        run("./build.py clean")
    enter_dir_and_do("serialization", job)

def clean_application_framework():
    def job():
        print_flush("-- cleaning application_framework")
        clean_xnc_core()
        clean_serialization()
    enter_dir_and_do("application_framework", job)

def clean_application_server():
    def job():
        print_flush("-- cleaning application_server")
        run("./build.py clean")
    enter_dir_and_do("application_server", job)

def clean_compiler():
    print_flush("- cleaning Ex-nunc compiler")
    def job():
        run("./build.py clean")
    enter_dir_and_do("page_compiler", job)

def build_xnc_core():
    def job():
        print_flush("--- building xnc_core")
        ret = run("./build.py lib.opt")
        if ret != 0:
            sys.exit(1)
        delete_ocamlbuild_objs()
    enter_dir_and_do("xnc_core", job)
    
def build_serialization():
    def job():
        print_flush("--- building serialization")
        ret = run("./build.py lib.opt")
        if ret != 0:
            sys.exit(1)
        delete_ocamlbuild_objs()
    enter_dir_and_do("serialization", job)

def build_application_framework():
    def job():
        print_flush("-- building application_framework")
        build_xnc_core()
        build_serialization()
    enter_dir_and_do("application_framework", job)

def build_application_server():
    def job():
        print_flush("-- building application_server")
        run("./build.py lib.opt")
        delete_ocamlbuild_objs()
    enter_dir_and_do("application_server", job)

def install_xnc_core():
    def job():
        print_flush("--- instaling xnc_core")
        run("./build.py install")
    enter_dir_and_do("xnc_core", job)

def install_serialization():
    def job():
        print_flush("--- instaling serialization")
        run("./build.py install")
    enter_dir_and_do("serialization", job)

def install_application_framework():
    def job():
        print_flush("-- installing application_framework")
        install_xnc_core()
        install_serialization()
    enter_dir_and_do("application_framework", job)

def install_application_server():
    def job():
        print_flush("-- instaling application_server")
        run("./build.py install")
    enter_dir_and_do("application_server", job)
        
def build_xnc_core_test():
    def job():
        print_flush("--- building xnc_core test suite")
        run("./build.py test")
    enter_dir_and_do("xnc_core", job)
        
def build_serialization_test():
    def job():
        print_flush("--- building serialization test suite")
        run("./build.py test")
    enter_dir_and_do("serialization", job)

def build_app_framew_test():
    def job():
        print_flush("-- building application_framework test suite")
        build_xnc_core_test()
        build_serialization_test()
    enter_dir_and_do("application_framework", job)

def build_app_server_test():
    def job():
        print_flush("-- building application_server test suite")
        run("./build.py test")
    enter_dir_and_do("application_server", job)

def build_compiler_test():
    def job():
        print_flush("-- building compiler test suite")
        run("./build.py test")
    enter_dir_and_do("page_compiler", job)


##########################
# THE SCRIPT BEGINS HERE #
##########################
install_dir = "/usr/local/bin"
if (len(sys.argv) == 1
        or
    find_list2(["--help", "-help", "-h", "help"], sys.argv)):
    print_help()

cmd_install_dir = process_option("--install-dir")
if cmd_install_dir is not None:
    install_dir = cmd_install_dir

# Copy the myocamlbuild.ml plugin everywhere, with the exception of the page
# compiler, which has its own myocamlbuild.ml
shutil.copy("myocamlbuild.ml", "application_framework/xnc_core/")
shutil.copy("myocamlbuild.ml", "application_framework/xnc_core/test/")
shutil.copy("myocamlbuild.ml", "application_framework/serialization/")
shutil.copy("myocamlbuild.ml", "application_server/")

for arg in sys.argv[1:]:
    if arg == "clean":
        print_flush("- cleaning Ex-nunc")
        clean_application_framework()
        clean_application_server()
        clean_compiler()
        run("ocamlbuild -clean")
        os.unlink(os.path.join("build", "build_lib.pyc"))

    elif arg == "all":
        print_flush("- building Ex-nunc")
        build_application_framework()
        build_application_server()
        run("./build.py compiler")

    elif arg == "libs":
        print_flush("- building Ex-nunc libraries")
        build_application_framework()
        build_application_server()

    elif arg == "install.libs":
        run("./build.py uninstall.libs")
        run("./build.py libs")
        print_flush("- installing Ex-nunc libraries")
        install_application_framework()
        install_application_server()

    elif arg == "install":
        run("./build.py install.libs")
        print_flush("- installing Ex-nunc compiler")
        enter_dir_and_do("page_compiler", lambda :
            run("./build.py install --install-dir " + install_dir))
    
    elif arg == "uninstall.libs":
        print_flush("- uninstalling Ex-nunc libraries")
        run("ocamlfind remove xnc-server")
        run("ocamlfind remove xnc-serialization")
        run("ocamlfind remove xnc-core")

    elif arg == "uninstall":
        run("./build.py uninstall.libs")
        print_flush("- uninstalling Ex-nunc compiler")
        enter_dir_and_do("page_compiler", lambda :
            run("./build.py uninstall --install-dir " + install_dir))

    elif arg == "compiler":
        run("./build.py install.libs")
        print_flush("- building Ex-nunc compiler, xnuncc")
        def job():
            run("./build.py compiler.opt")
        enter_dir_and_do("page_compiler", job)

    elif arg == "test":
        run("./build.py install.libs")
        print_flush("- building Ex-nunc unit test suite")
        build_app_framew_test()
        build_app_server_test()
        build_compiler_test()
    
    elif arg == "doc":
        print_flush("- Compiling Ex-nunc documentation")
        run("ocamlbuild documentation/ex_nunc.docdir/index.html")

    else:
        message = "%s: unknown action \"%s\": try with --help\n" % (sys.argv[0], arg)
        sys.stderr.write(message)
        sys.exit(1)

# vim:set encoding=utf8:

