#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import os
import glob

def expand_exts(name, exts):
    """expand_exts("foo", ["bar", "baz", "42"]) returns the string
    'foo.bar foo.baz foo.42'"""
    return " ".join(map(lambda ext: name + "." + ext, exts))

def enter_dir_and_do(dir, job):
    """Enters in the directory dir executes the function job,
    and exits the dir"""
    cwd = os.getcwd()
    os.chdir(dir)
    job()
    os.chdir(cwd)

def run(command, exit_on_failure = False):
    """run() is an alias for os.system(). If exit_on_failure is True and
    the command return value is <> 0, the program exits with the same value.
    """
    result = os.system(command)
    if exit_on_failure is True and result != 0:
        sys.exit(result >> 8)
    else:
        return result

def find_list2(l1, l2):
    """Finds elements of l1 in l2.
    Returns True if any of the l1 elements are present in l2."""
    return reduce( lambda acc, el : (el in l1) or acc, l2, False)

def print_flush(message):
    """Prints message on stdout and flushes stdout"""
    print message
    sys.stdout.flush()

def delete_ocamlbuild_objs():
    """Deletes "_build/myocamlbuild*" files from the current working dir"""
    to_delete = glob.glob("_build/myocamlbuild*")
    map(lambda f : os.unlink(f), to_delete)

def process_option(option_name):
    """If option_name exists in the command line:
    1) the option and the value are removed from sys.argv
    2) the value is returned
Example: ./build.py a b c --option value d e
process_option("--option") returns the string "value\""""
    index = None
    for i, arg in enumerate(sys.argv[1:]):
        if arg == option_name:
            index = i+1
            try:
                option_value = sys.argv[index+1]
            except IndexError:
                sys.stderr.write(
                    "%s: option --install-dir requires an argument\n" % sys.argv[0]
                )
    if index is not None:
        sys.argv[index:index+2] = []
        return option_value
    return None

# vim:set encoding=utf8:

