#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import os
from glob import glob
import shutil
import stat

sys.path.append(os.path.join("..", "build"))
import build_lib
from build_lib import *


#############
# VARIABLES #
#############

dot_files = """utility.ml \
xnunc_lexer.mli \
tree_validation.mli \
common_types.ml \
element_validations.ml \
validity_rules.ml \
tree_validation.ml \
codegen/attributes.ml \
codegen/codegen_utils.ml \
codegen/body.ml \
codegen/button.ml \
codegen/call.ml \
codegen/field.ml \
codegen/form.ml \
codegen/form_data.ml \
codegen/head.ml \
codegen/if_invalid.ml \
codegen/input.ml \
codegen/onclick.ml \
codegen/standard_page.ml \
codegen/text_blocks.ml \
codegen/validation_summary.ml \
codegen/validator.ml \
codegen/validator_param.ml \
codegen/root.ml \
codegen/codegen_common.ml \
files/files_common.ml \
files/main_file.ml \
files/page_parameters_file.ml \
files/page_factory_file.ml \
files/main_page_file.ml \
files/page_modules_file.ml \
files/resource_list_builder_file.ml \
files/session_data_file.ml \
files/files_factory.ml"""
# TODO insert when necessary
#fragment_composer/fragment_composer.mli \
#fragment_composer/fragment_composer.ml \
            
dot_command1 = "ocamlfind ocamldoc -package str,netstring,oUnit,extlib,xnc-core \
-dot \
-I codegen/ -I files/ -I fragment_composer/ \
-dot-reduce \
-dump xnuncc.docinfo \
xnuncc.ml"

dot_command2 = "ocamlfind ocamldoc -package str,netstring,oUnit,extlib,xnc-core \
-dot \
-I codegen/ -I files/ -I fragment_composer/ \
-dot-reduce \
" + dot_files + " \
-load xnuncc.docinfo \
-o ../modules_graph.dot"

#############
# FUNCTIONS #
#############
def print_help():
    print """This is the Ex-nunc build system, you are welcome.
Usage: build.py target [target2 target3...] [options]

Available targets:

Cleaning targets:
  clean     - remove generated files

Compilation targets:
  compiler      - builds compiler (bytecode, NO INSTALL) (DEFAULT ACTION)
  compiler.opt  - builds compiler (native and bytecode, NO INSTALL)
  install       - INSTALL compiler (native ONLY)
  uninstall     - UNINSTALL compiler
  test          - builds and executes compiler test suite (native and bytecode)
  dot           - create the dependency graph of the compiler modules.
                    (The output is the file modules_graph.svg)
  help          - show this help message

Options:
  --install-dir <dir>   - install the compiler executable in "dir".
                            (default: /usr/local/bin)
  --help                - show this help message"""

    sys.exit(0)


##########################
# THE SCRIPT BEGINS HERE #
##########################
if (find_list2(["--help", "-help", "-h", "help"], sys.argv)):
    print_help()

if (len(sys.argv) == 1):
    sys.argv.append("compiler")

install_dir = process_option("--install-dir")
if install_dir is None:
    install_dir = "/usr/local/bin"

for arg in sys.argv[1:]:
    if arg == "clean":
        run("ocamlbuild -clean", True)
        for file in glob.glob("modules_graph.*"):
            os.unlink(file)

    elif arg == "compiler":
        run("ocamlbuild xnuncc.d.byte", True)

    elif arg == "compiler.opt":
        run("ocamlbuild xnuncc.d.byte xnuncc.native", True)

    elif arg == "test":
        run("ocamlbuild test_suite.byte test_suite.native", True)
        run("./test_suite.byte", True)
        run("./test_suite.native", True)

    elif arg == "dot":
        run("./build.py compiler.opt", True)
        cwd = os.getcwd()
        os.chdir("_build")
        print dot_command1
        run(dot_command1, True)
        print dot_command2
        run(dot_command2, True)
        os.chdir(cwd)
        run("dot -Tsvg modules_graph.dot > modules_graph.svg", True)

    elif arg == "install":
        from stat import S_IRWXU, S_IRGRP, S_IXGRP, S_IROTH, S_IXOTH
        run("./build.py compiler.opt", True)
        exe_name = os.path.join(install_dir, "xnuncc")
        shutil.copyfile("xnuncc.native", exe_name)
        mode = os.stat(exe_name)[stat.ST_MODE]
        os.chmod(exe_name, mode | S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH)
        run("strip " + exe_name, True)

    elif arg == "uninstall":
        to_delete = os.path.join(install_dir, "xnuncc")
        if os.path.exists(to_delete):
            os.unlink(to_delete)

    else:
        sys.stderr.write(
            "%s: unknown action \"%s\": try with --help\n" % (sys.argv[0], arg)
        )
        sys.exit(1)

# vim:set encoding=utf8:

