open Camlp4.PreCast
open Xnc_stdlib
open Xnc_functional
open Common_types
open Codegen_common

let (-->) map key = String_map.find key map
let (<--) map (key, value) = String_map.add key value map
let keys map = String_map.fold (fun k v acc -> k::acc) map [] |> List.rev


type attribute_data =
    {
      pos : position;
      value : string;
      value_pos : position;
      param : bool;
    }

let create_attribute_map element =
  List.fold_left
    (
      fun map attr ->
        let name, data =
          match attr with
            | ((name, pos), (Literal(value), val_pos)) ->
                name, { pos = pos; value = value;
                        value_pos = val_pos; param = false; }
            | ((name, pos), (Parametric(value), val_pos)) ->
                name, { pos = pos; value = value;
                        value_pos = val_pos; param = true; } in
          map <-- (name, data)
    )
    String_map.empty
    element.attributes

class attributes element =
object (self)
  val map = create_attribute_map element

  method pos name = (map-->name).pos
  method value name = (map-->name).value
  method value_opt name = Common_lib.optify_not_found self#value name
  method value_pos name = (map-->name).value_pos
  method value_and_pos name =
    let line, pos = (map-->name).value_pos in
      (map-->name).value, line, pos
  method parametric name = (map-->name).param
  method exists name = String_map.mem name map

  method private is_implied name =
    let line, pos = (map-->name).value_pos in
      if line = -1 && pos = -1 then true else false

  method render ?(before="") ?(after="") ?(tr=fun (s : string) -> s) name comp =
    let v, (l, p) = (map-->name).value, (map-->name).value_pos in
    let tranf_value = tr v in
      if self#is_implied name then
        let s = Printf.sprintf "%s%s%s" before tranf_value after in
          Fragment_composer.append_gen s comp
      else
        let orig_len = String.length v in
        let transf_len = String.length tranf_value in
        let new_pos = p - (transf_len - orig_len)/2 in
          comp |> Fragment_composer.append_gen before |>
            Fragment_composer.append_source tranf_value l new_pos |>
            Fragment_composer.append_gen after

  method attribute_method_map =
    List.fold_left
      (fun m attr -> m <-- (attr, "set_"^attr)) String_map.empty (keys map)

  method render_all_attributes ?(attr_method_map=None) template_obj_id comp =
    let map = match attr_method_map with
      | None -> self#attribute_method_map
      | Some m -> m in

      String_map.fold
      (
        fun attr meth comp ->
          self#render ~before:(template_obj_id ^ "#" ^ meth ^ " (Some ")
                      ~after:");\n" ~tr:add_quotes attr comp
      )
      map
      comp
end

