open Camlp4.PreCast
open Xnc_functional
open Common_types
open Codegen_common

module FC = Fragment_composer

class body depth idx parsed_element parent_obj node =
object (self)
  inherit Elements.element depth idx parsed_element parent_obj node

  val module_name = "Body"
  val class_name = "body"
  
  method tag = Codegen_common.Body

  method target_files =
    let children_target =
      List.fold_left
        (fun acc (c : #Elements.element_type) -> acc @ (c#target_files))
        [] self#children in
      [ Page_target ] @ children_target

  method private render_page_structure comp =
    comp |> FC.append_gen ("let " ^ self#unique_id ^ " = Srv_page_structure" ^
                           ".create_body_element " ^
                           "(self#get_page_structure) in\n") |>
    fold_left (fun el -> el#render_main_page Build_page_structure)
              self#children
end

let make_children depth sibl_num block =
  match block with
    | Proc_instr _
    | Print_proc_instr _
    | Common_types.Cdata _ -> Text_blocks.make depth sibl_num block
    | Element e -> begin
        match e.name with
          | "form" -> Form.make depth sibl_num block
          | n -> Codegen_common.unknown_element n __LOCATION__
      end

let make depth idx (block : Common_types.parsed_block) =
  let element = Common_lib.unwrap_element block in
  let constructor parent_obj node =
    let this_obj = new body depth idx element parent_obj node in
      (this_obj :> Elements.element_type) in
    Tree.Comp (constructor, make_children)

