open Camlp4.PreCast
open Xnc_functional
open Common_types
open Codegen_common

module FC = Fragment_composer

class button depth idx parsed_element parent_obj node =
object (self)
  inherit Elements.element depth idx parsed_element parent_obj node

  val module_name = "Button"
  val class_name = "button"

  method tag = Codegen_common.Button

  method target_files =
    let children_target =
      List.fold_left
        (fun acc (c : #Elements.element_type) -> acc @ (c#target_files))
        [] self#children in
      [ Page_target ] @ children_target

  method private render_page_structure comp =
    comp |>
      attrs#render ~before:("let " ^ self#unique_id ^ " = new Srv_page_element.button_element ")
                   "name"
                   ~after:" in\n" ~tr:add_quotes |>
      attrs#render_all_attributes (self#unique_id) |>
      fold_left (fun el -> el#render_main_page Build_page_structure)
                self#children |>
      FC.append_gen ("Srv_page_structure.add_element\n  " ^
                    self#parent_id ^ "\n    (" ^
                    self#unique_id ^ " :> Srv_page_element.page_element);\n\n")

end

let make_children depth sibl_num block =
  match block with
    | Proc_instr _
    | Print_proc_instr _
    | Common_types.Cdata _ ->
        failwith "Button.make_children: this exception should never appear"
    | Element e -> begin
        match e.name with
          | "onclick" -> Onclick.make depth sibl_num block
          | n -> Codegen_common.unknown_element n __LOCATION__
      end

let make depth idx (block : Common_types.parsed_block) =
  let element = Common_lib.unwrap_element block in
  let constructor parent_obj node =
    let this_obj = new button depth idx element parent_obj node in
      (this_obj :> Elements.element_type) in
    Tree.Comp (constructor, make_children)

