open Camlp4.PreCast
open Xnc_functional
open Common_types
open Codegen_common

module FC = Fragment_composer

class call depth idx parsed_element parent_obj node =
object (self)
  inherit Jump_element.jump_element depth idx parsed_element parent_obj node

  val module_name = "Call"
  val class_name = "call"

  method tag = Codegen_common.Call

  method private render_page_structure comp =
    let if_invalid comp =
      match  If_invalid.find self#node with
        | None -> comp
        | Some inv -> begin
            comp |>
              FC.append_gen "~on_invalid_input:" |>
              inv#render_main_page Build_page_structure |>
              FC.append_gen "\n"
        end in

    comp |>
      FC.append_gen ("\n(* Expanding call element " ^ self#unique_id ^ " *)\n") |>
      FC.append_gen "self#set_target_page" |>
      FC.append_gen (
        match attrs#value "mode" with
          | "post" -> "\n  "
          | "cgi" -> "\n  ~redirect_mode:(Srv_types.Client_side Srv_types.Cgi_style)\n  "
          | "get" -> "\n  ~redirect_mode:(Srv_types.Client_side Srv_types.Plain_url)\n  "
          | _ -> failwith "call#render_page_structure: redirect method unknown\n"
      ) |> if_invalid |> self#build_goto
      
end

let make_children depth sibl_num block =
  match block with
    | Proc_instr _
    | Print_proc_instr _ -> failwith "Call.make_children: this exception should never appear"
    | Common_types.Cdata _ -> Text_blocks.make depth sibl_num block
    | Element e -> begin
        match e.name with
          | "if-invalid" -> If_invalid.make depth sibl_num block
          | n -> Codegen_common.unknown_element n __LOCATION__
      end

let make depth idx (block : Common_types.parsed_block) =
  let element = Common_lib.unwrap_element block in
  let constructor parent_obj node =
    let this_obj = new call depth idx element parent_obj node in
      Jump_element.jump_memo#add this_obj;
      (this_obj :> Elements.element_type) in
    Tree.Comp (constructor, make_children)

