exception Cast_error of string

type target_file_type =
  | Session_target
  | Parameters_target
  | Page_target
  | Page_factory_builder_target
  | Resource_list_builder_target
  | ML_source_target

type page_param_file_section =
  | Start_page_param
  | Param_type_def
  | Default_generators
  | Serialize_param
  | Deserialize_param
  | Params_builder

type page_file_section =
  | Global_scripts
  | Form_data_module
  | Class_header
  | Build_validators
  | Class_scripts
  | Build_page_structure
  | Class_footer

type element_tag =
  | Body
  | Button
  | Call
  | Cdata
  | Error_page
  | Field
  | Form
  | Form_data
  | Head
  | If_invalid
  | Input
  | Not_found_page
  | Onclick
  | Page_parameters
  | Page_parameters_variable
  | Print_proc_instruction
  | Proc_instruction
  | Script
  | Session
  | Session_variable
  | Standard_page
  | Start_page
  | Template
  | Validation_summary
  | Validator
  | Validator_param
  | Custom of string

let location ?(message = "") loc =
  Printf.sprintf "file %s, line %d%s"
    (Camlp4.PreCast.Loc.file_name loc)
    (Camlp4.PreCast.Loc.start_line loc)
      message

let unknown_element name loc =
  let m = ": unknown element \"" ^ name ^ "\"" in
    failwith (location ~message:m loc)

let asterisks = "(***************************************\
***************************************)\n"

let (!!) opt = match opt with
                | Some v -> v
                | None -> raise Not_found

let fold_left f l a = List.fold_left (fun acc el -> f el acc) a l

let add_quotes s = "\"" ^ s ^ "\""

