INCLUDE "../camlp4_macros.ml"

open Camlp4.PreCast
open Xnc_functional
open Common_types
open Codegen_common

module FC = Fragment_composer

class form depth idx parsed_element parent_obj node =
object (self)
  inherit Elements.element depth idx parsed_element parent_obj node

  val module_name = "Form"
  val class_name = "form"
  
  method tag = Codegen_common.Form

  method target_files =
    let children_target =
      List.fold_left
        (fun acc (c : #Elements.element_type) -> acc @ (c#target_files))
        [] self#children in
      [ Page_target ] @ children_target

  method private render_validators comp =
    comp |> (
      match Validator.find self#node with
        | [] -> FC.append_gen "    ()\n"
        | l -> fold_left (fun v -> v#render_main_page Build_validators) l
    ) |> FC.append_gen "\n"

  method private render_page_structure comp =
      comp |> FC.append_gen ("(* Expanding form element " ^ self#unique_id ^
                             " *)\nlet " ^ self#unique_id ^ " = Srv_page_structure." ^
                             "create_form_element (self#get_page_structure) " ^
                             "(self#get_cgi_context) in\n\n") |>
      attrs#render_all_attributes (self#unique_id) |>
      fold_left (fun el -> el#render_main_page Build_page_structure)
                self#children
end
  
let make_children depth sibl_num block =
  match block with
    | Proc_instr _
    | Print_proc_instr _
    | Common_types.Cdata _ -> Text_blocks.make depth sibl_num block
    | Element e -> begin
        match e.name with
          | "form-data" -> Form_data.make depth sibl_num block
          | "input" -> Input.make depth sibl_num block
          | "validator" -> Validator.make depth sibl_num block
          | "validation-summary" -> Validation_summary.make depth sibl_num block
          | "button" -> Button.make depth sibl_num block
          | n -> Codegen_common.unknown_element n __LOCATION__
      end

let make depth idx (block : Common_types.parsed_block) =
  let element = Common_lib.unwrap_element block in
  let constructor parent_obj node =
    let this_obj = new form depth idx element parent_obj node in
      (this_obj :> Elements.element_type) in
    Tree.Comp (constructor, make_children)

let find t = FIND_UNIQUE(Form)

