INCLUDE "../camlp4_macros.ml"

open Camlp4.PreCast
open Xnc_functional
open Common_types
open Common_lib
open Codegen_common

module FC = Fragment_composer

class form_data depth idx parsed_element parent_obj node =
object (self)
  inherit Elements.element depth idx parsed_element parent_obj node

  val module_name = "Form_data"
  val class_name = "form_data"
  
  method tag = Codegen_common.Form_data

  method target_files =
    let children_target =
      List.fold_left
        (fun acc (c : #Elements.element_type) -> acc @ (c#target_files))
        [] self#children in
      [ Page_target ] @ children_target

  method private form_data_header comp =
    FC.append_gen "module Form_data =\nstruct\n" comp

  method private type_definition comp =
    comp |> FC.append_gen "  type t =\n      {\n" |>
    fold_left
      (
        fun el comp ->
          let field = Field.cast el in
          let f_name = field#attrs#value "name" in
          let n_line, n_pos = field#attrs#value_pos "name" in
          let f_type = field#attrs#value "type" in
          let t_line, t_pos = field#attrs#value_pos "type" in
            comp |> FC.append_source f_name n_line n_pos |>
            FC.append_gen " : " |>
            FC.append_source f_type t_line t_pos |>
            FC.append_gen ";\n"
      )
      self#children |>
    FC.append_gen "      }\n\n"

  method private generate_create_function comp =
    comp |> FC.append_gen "  let create () =\n    {\n" |>
    fold_left
      (
        fun el comp ->
          let field = Field.cast el in
          let f_name = field#attrs#value "name" in
          let n_line, n_pos = field#attrs#value_pos "name" in
          let f_def = field#attrs#value "default" in
          let d_line, d_pos = field#attrs#value_pos "default" in
            comp |> FC.append_source f_name n_line n_pos |>
            FC.append_gen " = " |>
            FC.append_source f_def d_line d_pos |>
            FC.append_gen ";\n"
      )
      self#children |>
    FC.append_gen "    }\n\n"

  method private generate_from_cgi_context_function comp =
    comp |> FC.append_gen "  let from_cgi_context (cgi : Netcgi.cgi_activation) =\n" |>
    fold_left
      (
        fun el comp ->
          let field = Field.cast el in
          let f_name = field#attrs#value "name" in
          let n_line, n_pos = field#attrs#value_pos "name" in
          let f_type = field#attrs#value "type" in
          let conversion_fun = match f_type with
            | "int" -> "int_of_string"
            | "float" -> "float_of_string"
            | "char" -> "Xnc_stdlib.char_of_string"
            | "string" -> "Std.identity"
            | "bool" -> "bool_of_string"
            | _ -> failwith ("form_data#generate_from_cgi_context_function: unsupported variable type") in
            comp |> FC.append_gen "let " |> FC.append_source f_name n_line n_pos |>
            FC.append_gen (" = " ^ conversion_fun ^ " (cgi # argument_value \"" ^ f_name ^ "\") in\n")
      )
      self#children |>
    FC.append_gen "      {\n" |>
    fold_left
      (
        fun el comp ->
          let field = Field.cast el in
          let f_name = field#attrs#value "name" in
          let n_line, n_pos = field#attrs#value_pos "name" in
            comp |>
            FC.append_source f_name n_line n_pos |> FC.append_gen " = " |>
            FC.append_source f_name n_line n_pos |> FC.append_gen ";\n"
      )
      self#children

  method private form_data_footer comp =
    FC.append_gen "      }\nend\n\n" comp

  method private render_form_data comp =
    comp |>
      self#form_data_header |>
      self#type_definition |>
      self#generate_create_function |>
      self#generate_from_cgi_context_function |>
      self#form_data_footer

  method private render_page_structure c = c
end

let make_children depth sibl_num block =
  match block with
    | Proc_instr _
    | Print_proc_instr _
    | Common_types.Cdata _ -> failwith "Form_data.make_children: this exception should never appear"
    | Element e -> begin
        match e.name with
          | "field" -> Field.make depth sibl_num block
          | n -> Codegen_common.unknown_element n __LOCATION__
      end

let make depth idx (block : Common_types.parsed_block) =
  let element = Common_lib.unwrap_element block in
  let constructor parent_obj node =
    let this_obj = new form_data depth idx element parent_obj node in
      (this_obj :> Elements.element_type) in
    Tree.Comp (constructor, make_children)

let find t = FIND(Form_data)

