open Camlp4.PreCast
open Xnc_functional
open Common_types
open Codegen_common

module FC = Fragment_composer

class input depth idx parsed_element parent_obj node =
object (self)
  inherit Elements.element depth idx parsed_element parent_obj node
  
  val module_name = "Input"
  val class_name = "input"

  method tag = Codegen_common.Input

  method target_files = [ Codegen_common.Page_target ]

  method private get_data_type =
    let find_ker field =
      if attrs#value "name" = field#attrs#value "name" then true else false in
    let fields = Field.find (!!(self#parent))#node |> List.map Field.cast in
    let field = List.find find_ker fields in
      field#attrs#value "type"

  method private render_page_structure comp =
    let conversion_function =
      match self#get_data_type with
        | "int" -> "string_of_int"
        | "float" -> "string_of_float"
        | "char" -> "Printf.sprintf \"%c\""
        | "string" -> ""
        | "bool" -> "string_of_bool"
        | _ -> failwith ("input#render_page_structure: type unknown") in

    let rendered_input = Printf.sprintf
      "(* Rendering XHTML input element \"%s\" *)
let %s = new Srv_page_element.literal_element
(\"<input type=\\\"%s\\\"
             name=\\\"%s\\\"
             id=\\\"%s\\\"
             value=\\\"\" ^
     (Srv_page_helper.text (
       if not self#is_self_submitted
       then
         %s form_data.Form_data.%s
       else
         self#get_cgi_context#argument_value \"%s\"
     )) ^ \"\\\" />\") in
Srv_page_structure.add_element %s %s;\n\n"
        self#unique_id
        self#unique_id
        (self#attrs#value "type")
        (self#attrs#value "name")
        (self#attrs#value "id")
        conversion_function
        (self#attrs#value "id")
        (self#attrs#value "id")
        !!(self#parent)#unique_id
        self#unique_id in
      FC.append_gen rendered_input comp
end

let make depth idx (block : Common_types.parsed_block) =
  let element = Common_lib.unwrap_element block in
  let constructor parent_obj node =
    let this_obj = new input depth idx element parent_obj node in
      (this_obj :> Elements.element_type) in
    Tree.Comp (constructor, Tree.null_computation)

