INCLUDE "../camlp4_macros.ml"

open Camlp4.PreCast
open Xnc_functional
open Common_types
open Codegen_common

module FC = Fragment_composer

class virtual jump_element depth idx parsed_element parent_obj node =
object (self)
  inherit Elements.element depth idx parsed_element parent_obj node

  val module_name = "Jump_element"
  val class_name = "jump_element"

  method target_files = [ Codegen_common.Page_target ]

  method target_page_name = attrs#value "page"

  method private param_creator =
    let target_page_name_label =
      try Filename.chop_extension self#target_page_name
      with Invalid_argument "Filename.chop_extension" ->
            self#target_page_name |> String.lowercase in
      "Page_parameters.create_" ^ target_page_name_label ^ "_page_parameters"

  method private arguments comp =
    try
      let cd = !!(Text_blocks.find_cdata self#node) |> Text_blocks.cast_cdata in
        FC.append_source cd#text cd#line cd#pos comp
    with Not_found -> comp

  method private build_goto comp =
    comp |>
      FC.append_gen ("\n   (Page_modules.Page_behavior.Goto\n" ^
                     "(\"" ^ self#target_page_name ^ "\", (" ^
                     self#param_creator) |>
      self#arguments |>
      FC.append_gen ")))\n"
end

let jump_memo : jump_element Elements.memo = new Elements.memo
let cast_jump_element (o : Elements.element_type) =
  CAST_OR_ERROR (jump_memo, o, __LOCATION__)

let find t =
  let f acc t =
    let obj = Tree.get_data t in
      match obj#tag with
        | If_invalid -> (cast_jump_element obj)::acc
        | Call -> (cast_jump_element obj)::acc
        | _ -> acc in
  Tree.depth_first_fold f [] t

