open Camlp4.PreCast
open Xnc_functional
open Common_types
open Codegen_common

module FC = Fragment_composer

class onclick depth idx parsed_element parent_obj node =
object (self)
  inherit Elements.element depth idx parsed_element parent_obj node
  
  val module_name = "Onclick"
  val class_name = "onclick"

  method tag = Codegen_common.Onclick

  method target_files =
    let children_target =
      List.fold_left
        (fun acc (c : #Elements.element_type) -> acc @ (c#target_files))
        [] self#children in
      [ Page_target ] @ children_target

  method private render_page_structure comp =
    comp |>
      FC.append_gen ("\n(* Expanding onclick element " ^ self#unique_id ^ " *)\n" ^
                     self#parent_id ^ "#add_click_handler\n" ^
                     "(fun () ->\n") |>
      fold_left (fun el comp ->
                  match el#tag with
                    | Cdata ->
                      let cdata = Text_blocks.cast_cdata el in
                      let text, line, pos = cdata#text, cdata#line, cdata#pos in
                        comp |>
                          FC.append_gen ("(* Expanding OCaml script inside <xnc:onclick> " ^
                                         self#unique_id ^ " *)\n") |>
                          FC.append_source text line pos |> FC.append_gen "\n"
                    | Call -> el#render_main_page Build_page_structure comp
                    | _ -> failwith("onclick#render_page_structure: impossible element found here")
          ) self#children |>
      FC.append_gen ");\n\n"
end

let make_children depth sibl_num block =
  match block with
    | Proc_instr _
    | Print_proc_instr _ -> failwith "Onclick.make_children: this exception should never appear"
    | Common_types.Cdata _ -> Text_blocks.make depth sibl_num block
    | Element e -> begin
        match e.name with
          | "call" -> Call.make depth sibl_num block
          | n -> Codegen_common.unknown_element n __LOCATION__
      end

let make depth idx (block : Common_types.parsed_block) =
  let element = Common_lib.unwrap_element block in
  let constructor parent_obj node =
    let this_obj = new onclick depth idx element parent_obj node in
      (this_obj :> Elements.element_type) in
    Tree.Comp (constructor, make_children)

