INCLUDE "../camlp4_macros.ml"

open Camlp4.PreCast
open Xnc_functional
open Common_types
open Codegen_common

module FC = Fragment_composer

class script depth idx parsed_element parent_obj node =
object (self)
  inherit Elements.element depth idx parsed_element parent_obj node
  
  val module_name = "Script"
  val class_name = "script"

  method tag = Codegen_common.Script

  method target_files = [ Codegen_common.Page_target ]

  method private cdata =
    !!(Text_blocks.find_cdata self#node) |> Text_blocks.cast_cdata

  method private render_global_scripts comp =
    if attrs#value "scope" = "module" then
      FC.append_source self#cdata#text self#cdata#line self#cdata#pos comp
    else comp

  method private render_class_scripts comp =
    if attrs#value "scope" = "class" then
      FC.append_source self#cdata#text self#cdata#line self#cdata#pos comp
    else comp

  method private render_page_structure c = c
end

let make_children depth sibl_num block =
  match block with
    | Common_types.Cdata _ -> Text_blocks.make depth sibl_num block
    | _ -> failwith "Script.make_children: this exception should never appear"

let make depth idx (block : Common_types.parsed_block) =
  let element = Common_lib.unwrap_element block in
  let constructor parent_obj node =
    let this_obj = new script depth idx element parent_obj node in
      (this_obj :> Elements.element_type) in
    Tree.Comp (constructor, make_children)

let find t = FIND(Script)

