INCLUDE "../camlp4_macros.ml"

open Camlp4.PreCast
open Xnc_functional
open Common_types
open Codegen_common

module FC = Fragment_composer


class session depth idx parsed_element parent_obj node =
object (self)
  inherit Elements.element depth idx parsed_element parent_obj node
  
  val module_name = "Session"
  val class_name = "session"

  method tag = Codegen_common.Session

  method target_files =
    let children_target =
      List.fold_left
        (fun acc (c : #Elements.element_type) -> acc @ (c#target_files))
        [] self#children in
      [ Session_target ] @ children_target

  method render_session_file c =
    let vars = List.map Variable.cast_session_var self#children in
      c |>
      (* Create the type t of the session *)
      FC.append_gen "type t = {\n" |>
      fold_left (fun v -> v#render_type_definition) vars |>
      FC.append_gen "};;\n\n" |>

      (* Build the "create" function *)
      FC.append_gen "let create () = {\n" |>
      fold_left (fun v -> v#render_create) vars |>
      FC.append_gen "};;\n\n" |>

      (* Build serializer *)
      FC.append_gen "let to_field_collection s =\n" |>
      FC.append_gen "let fc = Xnc_field_collection.create \"Session\" in\n" |>
      fold_left (fun v -> v#render_serializer) vars |>
      FC.append_gen "fc;;\n\n" |>

      (* Build deserializer *)
      FC.append_gen "let from_field_collection fc =\n" |>
      FC.append_gen "let session = create () in\n" |>
      fold_left (fun v -> v#render_deserializer) vars |>
      FC.append_gen "session;;\n\n"
end

let session_memo : session Elements.memo = new Elements.memo
let cast (o : Elements.element_type) =
  CAST_OR_ERROR (session_memo, o, __LOCATION__)

let make_children depth sibl_num block =
  match block with
    | Proc_instr _
    | Print_proc_instr _
    | Common_types.Cdata _ -> failwith "Session.make_children: this exception should never appear"
    | Element e -> begin
        match e.name with
          | "variable" -> Variable.make Session_variable depth sibl_num block
          | n -> Codegen_common.unknown_element n __LOCATION__
      end

let make depth idx (block : Common_types.parsed_block) =
  let element = Common_lib.unwrap_element block in
  let constructor parent_obj node =
    let this_obj = new session depth idx element parent_obj node in
      session_memo#add this_obj;
      (this_obj :> Elements.element_type) in
    Tree.Comp (constructor, make_children)

let find t = FIND_UNIQUE(Session)

