open Camlp4.PreCast
open Xnc_functional
open Common_types

class template filename parent_obj node =
object (self)
  inherit Elements.block 0 0 parent_obj node

  val module_name = "Template"
  val class_name = "template"

  method tag =  Codegen_common.Template

  method target_files =
    List.fold_left
      (fun acc (c : #Elements.element_type) -> acc @ (c#target_files))
      [] self#children

  method unique_id = "root" (* This is the root *)

  method get_page = Page.find self#node
end

let top_level_blocks filename depth sibl_num block =
  match block with
    | Proc_instr _
    | Print_proc_instr _
    | Cdata _ -> Text_blocks.make depth sibl_num block
    | Element e -> begin
        match e.name with
          | "page" -> Page.make filename depth sibl_num block
          | "session" -> Session.make depth sibl_num block
          | n -> Codegen_common.unknown_element n __LOCATION__
      end

let root_element filename depth sibl_num block =
  let constructor parent_obj node =
    let this_obj = new template filename parent_obj node in
      (this_obj :> Elements.element_type) in
    Tree.Comp (constructor, top_level_blocks filename)

let make filename (parsed_blocks : Common_types.document) =
  let block =
    {
      name = "__EX-NUNC_TEMPLATE__";
      attributes = [];
      sub_trees = parsed_blocks;
      el_pos = (-1,-1)
    } in
    Tree.make
      (root_element filename)
      Common_lib.get_sub_trees
      None
      None
      (Element block)

