INCLUDE "../camlp4_macros.ml"

open Xnc_functional
open Camlp4.PreCast
open Common_types
open Codegen_common

module El = Elements
module FC = Fragment_composer

let double_quote = Str.regexp_string "\""
let backslach = Str.regexp_string "\\"

class virtual text_block_base depth idx text line pos parent_obj node =
object (self)
  inherit El.block depth idx parent_obj node
    
  val module_name = "Text_blocks"
  val class_name = "text_block_base"

  val text = text
  val line = line
  val pos = pos

  method text : string = text
  method line : int = line
  method pos : int = pos

  method as_text_block = (self :> text_block_base)

  method target_files = [ Codegen_common.Page_target ]
end

class proc_instruction depth idx text line pos parent_obj node =
object (self)
  inherit text_block_base depth idx text line pos parent_obj node

  val class_name = "proc_instruction"

  method tag = Codegen_common.Proc_instruction

  method private render_page_structure comp =
    comp |>
      FC.append_gen ("(* Expanding processing instruction " ^
                     self#unique_id ^ " *)\n") |>
      FC.append_source text line pos |>
      FC.append_gen "\n"
end

class print_proc_instruction depth idx text line pos parent_obj node =
object (self)
  inherit text_block_base depth idx text line pos parent_obj node

  val class_name = "print_proc_instruction"

  method tag = Codegen_common.Print_proc_instruction

  method private render_page_structure comp =
    comp |>
      FC.append_gen ("(* Expanding print processing instruction " ^
                     self#unique_id ^ " *)\n" ^
                     "let " ^ self#unique_id ^
                     " = new Srv_page_element.literal_element (") |>
      FC.append_source text line pos |>
      FC.append_gen (") in\nSrv_page_structure.add_element " ^
                     self#parent_id ^ " " ^ self#unique_id ^ ";\n\n")
end

class cdata depth idx text line pos parent_obj node =
object (self)
  inherit text_block_base depth idx text line pos parent_obj node

  val class_name = "cdata"

  method tag = Codegen_common.Cdata

  method private render_page_structure comp =
    let q_text = text |> Str.global_replace backslach "\\\\\\\\" |>
      Str.global_replace double_quote "\\\"" in

      comp |> FC.append_gen
              ("(* Expanding CDATA block " ^ self#unique_id ^ " *)\n" ^
               "let " ^ self#unique_id ^ " = " ^
               "new Srv_page_element.literal_element \"" ^ q_text ^ "\" in\n" ^
               "Srv_page_structure.add_element " ^ self#parent_id ^ " " ^
               self#unique_id ^ ";\n")
end

let cdata_memo : cdata Elements.memo = new Elements.memo
let cast_cdata (o : Elements.element_type) =
  CAST_OR_ERROR (cdata_memo, o, __LOCATION__)
let find_cdata t = FIND_UNIQUE(Cdata)

let make (depth : int) (sibl_num : int) (block : parsed_block) =
  match block with
    | Proc_instr (text, (line, pos)) ->
        let constructor parent_obj node =
          let this_obj = new proc_instruction depth sibl_num text line pos parent_obj node in
            (this_obj :> El.element_type) in
          Tree.Comp (constructor, Tree.null_computation)
    | Print_proc_instr (text, (line, pos)) ->
        let constructor parent_obj node =
          let this_obj = new print_proc_instruction depth sibl_num text line pos parent_obj node in
            (this_obj :> El.element_type) in
          Tree.Comp(constructor, Tree.null_computation)
    | Common_types.Cdata (text, (line, pos)) ->
        let constructor parent_obj node =
          let this_obj = new cdata depth sibl_num text line pos parent_obj node in
            cdata_memo#add this_obj;
            (this_obj :> El.element_type) in
          Tree.Comp(constructor, Tree.null_computation)
    | Element _ -> failwith "Text_blocks.make: this exception should never appear"

