open Xnc_functional
open Camlp4.PreCast

type 'a tree =
    Node of ('a tree Lazy.t) option *   (* parent *)
             'a tree list *             (* children *)
             'a                         (* data *)

let fake_node data = Node (None, [], data)

let get_parent = function
    Node (parent, _, _) ->
      match parent with
        | None -> None
        | Some p -> Some (Lazy.force p)

let get_data = function Node (_, _, d) -> d

let get_children = function Node (_, ch, _) -> ch

let rec get_root t =
  match get_parent t with
    | None -> t
    | Some p -> get_root p

type ('a, 'b) computation = Comp of 'b * (int -> int -> 'a -> ('a, 'b) computation)
  
let null_computation (_ : int) (_ : int) (_ : 'a) : ('a, 'b) computation =
  failwith "module Tree: null computation called, I can't go further"

let rec make
    ?(depth = 0)
    ?(sibl_num = 0)
    meta_constructor
    get_children
    parent_obj
    parent_node
    parsed_block =
  let constructor, next_constr =
    match meta_constructor depth sibl_num parsed_block with
      | Comp (constructor, next_constr) -> constructor, next_constr in
    
  let rec this_node =
    lazy (
      let this_obj = constructor parent_obj this_node in
      let ch =
        List.fold_left
          (
            fun (sibl, acc) el ->
              let new_el =
                make ~depth:(depth + 1) ~sibl_num:sibl
                  next_constr get_children (Some this_obj) (Some this_node) el in
                sibl + 1, new_el::acc
          )
          (0, [])
          (get_children parsed_block) |> snd |> List.rev in
        Node (parent_node, ch, this_obj)
    ) in
    Lazy.force this_node

let rec fold ?(depth = 0) ?(sibl_num = 0) before_f after_f acc t =
  let acc' = before_f acc depth sibl_num t in
  let acc'' =
    List.fold_left
      (
        fun (sibl, acc) el ->
          (sibl + 1,
           fold ~depth:(depth + 1) ~sibl_num:sibl before_f after_f acc el)
      )
      (0, acc')
      (get_children t) |> snd in
    after_f acc'' depth sibl_num t

let iter ?(depth = 0) ?(sibl_num = 0) before_f after_f t =
  let aux_before _ depth sibl_num t = before_f depth sibl_num t in
  let aux_after _ depth sibl_num t = after_f depth sibl_num t in
    fold aux_before aux_after () t

let depth_first_fold f acc t =
  let before_aux acc _ _ t = f acc t in
  let after_aux acc _ _ _ = acc in
    fold before_aux after_aux acc t

let depth_first_iter f t =
  let f' = fun () t -> f t in
    depth_first_fold f' () t


