INCLUDE "../camlp4_macros.ml"

open Camlp4.PreCast
open Xnc_functional
open Common_types
open Codegen_common

module FC = Fragment_composer

class validator depth idx parsed_element parent_obj node =
object (self)
  inherit Elements.element depth idx parsed_element parent_obj node
  
  val module_name = "Validator"
  val class_name = "validator"

  method tag = Codegen_common.Validator

  method target_files =
    let children_target =
      List.fold_left
        (fun acc (c : #Elements.element_type) -> acc @ (c#target_files))
        [] self#children in
      [ Page_target ] @ children_target

  method private render_validators comp =
    let cl_v, cl_l, cl_p = attrs#value_and_pos "class" in
    let fid_v, fid_l, fid_p = attrs#value_and_pos "field-id" in
    comp |> FC.append_gen "self#add_validator\n" |>
      FC.append_source ("\"" ^ fid_v ^ "\"") fid_l (fid_p-1) |>
      FC.append_gen "\n(new " |>
      FC.append_source cl_v cl_l cl_p |>
      FC.append_gen "\n(self#get_cgi_context#argument_value " |>
      FC.append_source ("\"" ^ fid_v ^ "\"") fid_l (fid_p-1) |>
      FC.append_gen ")" |>
      fold_left
        (fun val_par c -> c |> FC.append_gen "\n         " |>
                          val_par#render_main_page Build_validators)
        self#children |>
      FC.append_gen ");\n"

  method private render_page_structure comp =
    comp |> FC.append_gen
      ("(* Rendering validator element \"" ^ self#unique_id ^ "\" *)
if (self#is_self_submitted)
then
  begin
    let " ^ self#unique_id ^ " =
      Srv_validator_collection.find_validator
        (self#get_validator_collection)
        \"" ^ (self#attrs#value "field-id") ^ "\" in
      Srv_page_structure.add_element
        " ^ !!(self#parent)#unique_id ^ "
        (new Srv_page_element.validator_element " ^ self#unique_id ^ ")
  end
else
  ();\n\n")
end

let make_children depth sibl_num block =
  match block with
    | Proc_instr _
    | Print_proc_instr _
    | Common_types.Cdata _ -> failwith "Validator.make_children: this exception should never appear"
    | Element e -> begin
        match e.name with
          | "validator-param" -> Validator_param.make depth sibl_num block
          | n -> unknown_element n __LOCATION__
      end

let make depth idx (block : Common_types.parsed_block) =
  let element = Common_lib.unwrap_element block in
  let constructor parent_obj node =
    let this_obj = new validator depth idx element parent_obj node in
      (this_obj :> Elements.element_type) in
    Tree.Comp (constructor, make_children)

let find t = FIND(Validator)

