open Camlp4.PreCast
open Xnc_functional
open Common_types
open Codegen_common

module FC = Fragment_composer

class validator_param depth idx parsed_element parent_obj node =
object (self)
  inherit Elements.element depth idx parsed_element parent_obj node

  val module_name = "Validator_param"
  val class_name = "validator_param"

  method target_files = [ Codegen_common.Page_target ]

  method tag = Codegen_common.Validator_param

  method private render_validators comp =
    let label = attrs#value_opt "label" in
    let comp = match label with
                | Some l ->
                    let line, pos = attrs#value_pos "label" in
                      FC.append_source ("~" ^ l ^ ":") line pos comp
                | None -> comp in
      match self#children with
        | [] -> comp
        | l ->  let cdata = l |> List.hd |> Text_blocks.cast_cdata in
                let text, line, pos = cdata#text, cdata#line, cdata#pos in
                  FC.append_source (" " ^ text ^ " ") line pos comp
end

let make_children depth sibl_num block =
  match block with
    | Proc_instr _
    | Print_proc_instr _ ->
        failwith "Validator_param.make_children: this exception should never appear"
    | Common_types.Cdata _ -> Text_blocks.make depth sibl_num block
    | Element e -> begin
        match e.name with
          | n -> Codegen_common.unknown_element n __LOCATION__
      end

let make depth idx (block : Common_types.parsed_block) =
  let element = Common_lib.unwrap_element block in
  let constructor parent_obj node =
    let this_obj = new validator_param depth idx element parent_obj node in
      (this_obj :> Elements.element_type) in
    Tree.Comp (constructor, make_children)

