INCLUDE "../camlp4_macros.ml"

open Camlp4.PreCast
open Xnc_functional
open Common_types
open Codegen_common

module FC = Fragment_composer

class virtual variable_base depth idx parsed_element parent_obj node =
object (self)
  inherit Elements.element depth idx parsed_element parent_obj node

  val module_name = "Variable"
  val class_name = "variable_base"
end

class page_parameters_variable depth idx parsed_element parent_obj node =
object (self)
  inherit variable_base depth idx parsed_element parent_obj node

  val module_name = "Variable"
  val class_name = "page_parameters_variable"

  method tag = Codegen_common.Page_parameters_variable

  method target_files = [ Codegen_common.Parameters_target ]
end

let param_var_memo : page_parameters_variable Elements.memo = new Elements.memo
let cast_param_var (o : Elements.element_type) =
  CAST_OR_ERROR (param_var_memo, o, __LOCATION__)

class session_variable depth idx parsed_element parent_obj node =
object (self)
  inherit variable_base depth idx parsed_element parent_obj node

  val module_name = "Variable"
  val class_name = "session_variable"

  method tag = Codegen_common.Session_variable

  method target_files = [ Codegen_common.Session_target ]

  method render_type_definition c =
    c |> FC.append_gen "mutable" |> attrs#render "name" |>
    FC.append_gen " : " |> attrs#render "type" |> FC.append_gen ";\n"

  method render_create c =
    let tr = match attrs#value "type" with
      | "string" -> fun s -> "\"" ^ s ^ "\""
      | "char" -> fun s -> "'" ^ s ^ "'"
      | _ -> fun s -> s in
      c |> attrs#render "name" |> FC.append_gen " = " |>
        attrs#render ~tr "default" |> FC.append_gen ";\n"

  method render_serializer c =
    let v_type = attrs#value "type" in
      c |> FC.append_gen "Xnc_field_collection.register_" |>
      FC.append_gen v_type |> FC.append_gen " fc " |>
      attrs#render ~tr:add_quotes "name" |>
      attrs#render ~before:" s." "name" |> FC.append_gen ";\n"

  method render_deserializer c =
    let v_name, v_type = attrs#value "name", attrs#value "type" in
      c |> attrs#render ~before:"let " "name" |>
      FC.append_gen (" = Xnc_field_collection.get_" ^ v_type ^ " fc ") |>
      attrs#render ~tr:add_quotes ~after:" in\n" "name" |>
      attrs#render ~before:"let session = { session with " "name" |>
      attrs#render ~before:" = " ~after:" } in\n\n" "name"
end

let session_var_memo : session_variable Elements.memo = new Elements.memo
let cast_session_var (o : Elements.element_type) =
  CAST_OR_ERROR (session_var_memo, o, __LOCATION__)

let make_children depth sibl_num old_data =
  match old_data with
    | Proc_instr _
    | Print_proc_instr _
    | Common_types.Cdata _ -> Text_blocks.make depth sibl_num old_data
    | Element e -> begin
        match e.name with
          | n -> unknown_element n __LOCATION__
      end

let make var_type depth idx (block : Common_types.parsed_block) =
  let element = Common_lib.unwrap_element block in
  let constructor parent_obj node =
    match var_type with
        | Session_variable ->
            let this_obj = new session_variable depth idx element parent_obj node in
              session_var_memo#add this_obj; (this_obj :> Elements.element_type)
        | Page_parameters_variable ->
            let this_obj = new page_parameters_variable depth idx element parent_obj node in
              param_var_memo#add this_obj; (this_obj :> Elements.element_type)
        | _ -> failwith "Variable.make: this exception should never appear" in
    Tree.Comp (constructor, make_children)

