open Xnc_stdlib
open Xnc_functional
open Common_types

exception Non_element_block of string
exception Attribute_not_found of string
exception No_value

let get_sub_trees = function
  | Proc_instr _ | Print_proc_instr _ | Cdata _ -> []
  | Element e -> e.sub_trees
;;

let unwrap_element block =
  let message = match block with
                  | Proc_instr _ -> "processing instruction"
                  | Print_proc_instr _ -> "print processing instruction"
                  | Cdata _ -> "cdata"
                  | _ -> "" in

    match block with
      | Element e -> e
      | _ -> failwith ("Can't unwrap this block: it's a " ^ message)
;;

(* TODO: COMMENT THIS FUNCION *)
let get_attribute_value
      (elem : parsed_element)
      (attribute_name : string)
      : string =
  let attr =
    List.find
      (
        fun at ->
          match at with
            | ((name, pos), (Literal(value), val_pos))
            | ((name, pos), (Parametric(value), val_pos)) ->
                if name = attribute_name
                then true
                else false
      )
      elem.attributes in

    match attr with
      | ((name, pos), (Literal(value), val_pos))
      | ((name, pos), (Parametric(value), val_pos)) -> value
;;

(* TODO: COMMENT THIS FUNCION *)
let find_all_child_element
      (elem : parsed_element)
      (child_name : string)
      : parsed_element list =
  let list_of_blks = List.filter
                       (
                         fun blk ->
                           match blk with
                             | Element(el) ->
                                 if el.name = child_name then true else false
                             | _ -> false
                       )
                       elem.sub_trees in
    List.map
      (
        fun blk ->
          match blk with
            | Element(el) -> el
            | _ -> failwith ("find_all_child_element: FATAL: a non element " ^
                   "block has been found, this should never happen!")
      )
      list_of_blks
;;

(* TODO: COMMENT THIS FUNCION *)
let get_optional_attribute
      (elem : parsed_element)
      (attribute_name : string)
      : Common_types.attribute option =
  try
    Some (
      List.find
        (
          fun at -> match at with
            | ((name, _), _) -> if name = attribute_name then true else false
        )
        elem.attributes
    )
  with
    | Not_found -> None
;;

(* TODO: COMMENT THIS FUNCION *)
let get_optional_attribute_value
      (attr : Common_types.attribute option)
      : string =

    match attr with
      | None -> raise No_value
      | Some (((name, pos), (Literal(value), val_pos)))
      | Some (((name, pos), (Parametric(value), val_pos))) -> value
;;

let extract_attribute_value_from_block
    ?(calling_function = "extract_attribute_value_from_block")
    (blk : Common_types.parsed_block)
    (attribute_name : string)
    : string =
  let elem = match blk with
    | Element el -> el
    | _ -> let error_message =
        Printf.sprintf
          ("%s: the" ^^
             " parsed_block class attribute is not an element," ^^
             " so no attributes available") calling_function in
        raise (Non_element_block error_message) in
  let attr =
    try
      List.find
        (
          fun at ->
            match at with
              | ((name, pos), (Literal(value), val_pos))
              | ((name, pos), (Parametric(value), val_pos)) ->
                  if name = attribute_name
                  then true
                  else false
        )
        elem.attributes
    with
      | Not_found -> let error_message =
          Printf.sprintf
            ("%s: the required attribute" ^^
               " \"%s\" does not exist")
            calling_function
            attribute_name in
          raise (Attribute_not_found error_message) in

    match attr with
      | ((name, pos), (Literal(value), val_pos))
      | ((name, pos), (Parametric(value), val_pos)) -> value

let optify_not_found f x =
  try Some (f x) with Not_found -> None

