open Xnc_stdlib
open Xnc_functional

type position = int * int

type 'a with_position = 'a * position

type attr_value =
  | Literal of string
  | Parametric of string

type attribute = string with_position * attr_value with_position

type parsed_block =
  | Proc_instr of string with_position
  | Print_proc_instr of string with_position
  | Cdata of string with_position
  | Element of parsed_element

and parsed_element =
  {
    name : string;
    attributes : attribute list;
    sub_trees : document;
    el_pos : position
  }

and
    document = parsed_block list

type card = int

type 'a with_range_constraints =
  | No_limits
  | Range of 'a * 'a

type attribute_type_contraint =
  | String
  | Bool
  | Int of int with_range_constraints
  | Float of float with_range_constraints

type attribute_constraint = {
  attr_type : attribute_type_contraint;
  mandatory : bool;
  param_allowed : bool;
  default : string option;
  valid_values: string list option;
}

type element_specific_validation = parsed_element -> unit

type element_constraint = {
  children : (card * card) String_map.t;
  attributes_map : attribute_constraint String_map.t;
  proc_instr_allowed : bool;
  cdata_allowed : bool;
  is_root_element : bool;
  validation : element_specific_validation;
}

type constraints_table =
    (string *
     (string *
      card * card)
                list *
     (string *
      attribute_type_contraint *
      bool *
      bool *
      string option *
      string list option) list *
     bool *
     bool *
     bool *
     element_specific_validation
    ) list

exception Unknown_element of string * position
exception Proc_instr_not_allowed of string * position
exception Cdata_not_allowed of string * position
exception Illegal_child of string * position * string
exception Too_few_children of string * position * int * string
exception Too_many_children of string * position * int * string
exception Missing_attribute of string * position * string
exception Unknown_attribute of string * position
exception Parametric_param_not_allowed of string * position
exception Invalid_attribute_value of string * position
exception Attribute_out_of_range of string * position * string * string
exception Invalid_attribute_type of string * position * string
exception Too_many_root_elements of string * position
exception Non_root_element of string * position

