open Xnc_functional
open Utility
open Common_types
open Codegen_common

module Fn = Filename

let render_line_annotations = true

let file_header output_file_name =
  Printf.sprintf
"(*******************************************************************************
 * This file is \"%s\"
 *
 * Automatically generated on %s
 *
 * PLEASE DO NOT EDIT
 ******************************************************************************)\n\n"
  output_file_name (timestamp_to_string ())
;;

let skip_file_header content =
    let str = file_header "TEST" in
    let header_line_no = ref 0 in
    let length = String.length str in
      for i = 0 to (length - 1) do
        if str.[i] = '\n' then incr header_line_no
      done;

      let length = String.length content in
      let i = ref 0 and skipped_nl = ref 0 in
        while !skipped_nl < !header_line_no do
          if content.[!i] = '\n' then incr skipped_nl;
          incr i;
        done;

        String.sub content !i (length - !i)
;;

class virtual file (file_name : string) =
object (self)
  val name = file_name
  method name = name
end

class virtual source_file file_name =
object (self)
  inherit file file_name
  val raw_content = string_of_file_name file_name

  method raw_content = raw_content
  method virtual template : Elements.element_type
  method virtual target_files : target_file_type list
  method virtual force_debug : unit
  method virtual get_target_pages : string list
end

let tree_of_filename file_name =
  let lexbuf = Lexing.from_string (string_of_file_name file_name) in
  let document = Xnunc_lexer.parse_xnunc_lexbuf lexbuf in
  let document = Tree_validation.remove_phantom_nodes document in
  let () = Tree_validation.validate_document document in
  let document = Tree_validation.insert_document_defaults document in
    Template.make file_name document |> Tree.get_data
;;

class xnunc_file global_validator file_name =
object (self)
  inherit source_file file_name
  val template = tree_of_filename file_name
  val global_validator = global_validator

  initializer global_validator#collect_info self

  method template = template
  method target_files = template#target_files

  method force_debug =
    try (Page.find template#node)#force_debug true
    with Not_found -> ()

  method get_target_pages =
    Jump_element.find template#node |> List.map (fun j -> j#target_page_name)
end

class ml_file file_name =
object (self)
  inherit source_file file_name

  method template = failwith "Files_common.ml_file#get_document: this is not an XML file!"

  method target_files = [ Codegen_common.ML_source_target ]
  method force_debug = ()
  method get_target_pages = []
end

class virtual generated_file
  (output_file_name : string) =
object (self)
  inherit file output_file_name

  val input_file_list = ref []

  method add_input_file (in_file : source_file) =
    input_file_list := in_file :: !input_file_list

  method private fold_on_input
      (f:Page.page_base -> Fragment_composer.t -> Fragment_composer.t)
      (comp:Fragment_composer.t) : Fragment_composer.t =
    fold_left (
      fun in_file comp ->
        let page = Page.find in_file#template#node in
          comp |> Fragment_composer.set_source_file in_file#name |> f page
      ) !input_file_list comp

  method virtual private really_render : string

  method render_file =
    let new_content =
      try self#really_render
      with exn -> Printf.eprintf "failed rendering file %s\n%!" name;
                  raise exn in
    let save_the_new_file () =
      Printf.printf "Rendering file \"%s\"...%!" name;
      string_to_file_name name new_content;
      Printf.printf " done!\n%!" in

      try
        let old_content = string_of_file_name name in
        let old_content' = skip_file_header old_content in
        let new_content' = skip_file_header new_content in
          if String.compare old_content' new_content' <> 0
            then save_the_new_file ()
      with
        | Sys_error _ -> save_the_new_file ()
end

let msg = Printf.sprintf

let compute_output_filename
    (input_file : source_file)
    (output_type : target_file_type) =
  match output_type with
    | Session_target -> "session_data.ml"
    | Parameters_target -> "page_parameters.ml"
    | Page_factory_builder_target -> "page_factory_builder.ml"
    | Page_target ->
        (input_file#name |> Fn.chop_extension |> norm_path) ^ ".ml"

    | Resource_list_builder_target -> "resource_list_builder.ml"
    | ML_source_target -> input_file#name |> norm_path
;;

