open Xnc_stdlib
open Xnc_functional
open Common_types
open Files_common

let (//) = Utility.(//)

let xnunc_file_regexp = Str.regexp "^.+\\.\\(xnunc\\|xnc\\)$"
let ml_file_regexp = Str.regexp "^.+\\.mli?$"
let session_file_regexp = Str.regexp "^session\\.app$"


let create_input_file global_validator filename =
  if Str.string_match xnunc_file_regexp filename 0
  then (new Files_common.xnunc_file global_validator filename :> source_file)
  else if Str.string_match ml_file_regexp filename 0
  then (new Files_common.ml_file filename :> source_file)
  else if Str.string_match session_file_regexp (Filename.basename filename) 0
  then (new Files_common.xnunc_file global_validator filename :> source_file)
  else failwith ("Files_factory.create_input_file: " ^
                 filename ^ ": unknowkn file type")


let create_output_file build_dir file map target =
  let output_file_name = build_dir//(compute_output_filename file target) in
    if (String_map.mem output_file_name map) = true
    then (* the output file already exixts *)
      let old_file = String_map.find output_file_name map in
        old_file#add_input_file file;
        String_map.add output_file_name old_file map
    else (* the output file is new *)
      let new_out_file = (
        match target with
          | Codegen_common.Session_target ->
              new Session_data_file.session_data_file output_file_name
          | Codegen_common.Parameters_target ->
              new Page_parameters_file.page_parameters_file output_file_name
          | Codegen_common.Page_factory_builder_target ->
              new Page_factory_file.page_factory_builder_file output_file_name
          | Codegen_common.Page_target ->
              new Main_page_file.main_page_file output_file_name
          | Codegen_common.Resource_list_builder_target ->
              new Resource_list_builder_file.resource_list_builder_file output_file_name
          | Codegen_common.ML_source_target ->
              new Ml_file.ml_file output_file_name
                :> generated_file) in
        new_out_file#add_input_file file;
        String_map.add output_file_name new_out_file map

