open Xnc_functional
open Files_common
open Codegen_common

module FC = Fragment_composer

class page_parameters_file (output_file_name : string) =
object (self)
  inherit generated_file output_file_name

  method private really_render =
    FC.empty |>
    FC.set_dest_file output_file_name |>

    (* Header *)
    FC.append_gen (file_header output_file_name) |>

    (* Type "t" definition *)
    FC.append_gen "type t =\n" |>
    FC.append_gen "  | No_parameters\n" |>
    FC.append_gen "  | Error_page_parameters of string * string\n" |>
    FC.append_gen "  | Not_found_page_parameters of string\n" |>
    self#fold_on_input (
      fun page comp ->
        page#render_page_parameters_file Param_type_def comp |>
          FC.append_gen "\n"
    ) |>
    FC.append_gen ";;\n\n" |>

    (* A standard function that builds the parameters of the error page *)
    FC.append_gen "let create_error_page_parameters error_description backtrace =
  Error_page_parameters (error_description, backtrace)
;;

let default_error_page_parameters =
  create_error_page_parameters \"\" \"\"
;;\n\n" |>

    (* The start page building function *)
    FC.append_gen "let default_start_page_parameters =\n" |>
    self#fold_on_input (fun page -> page#render_page_parameters_file Start_page_param) |>
    FC.append_gen "\n\n" |>

    (* The default_* functions *)
    self#fold_on_input (
      fun page comp ->
        page#render_page_parameters_file Default_generators comp |>
        FC.append_gen "\n"
    ) |>
    FC.append_gen "let get_error_description parameters =
  match parameters with
      Error_page_parameters (error, _) -> error
    | _ -> raise Srv_types.Invalid_page_parameters
;;

let get_backtrace parameters =
  match parameters with
      Error_page_parameters (_, bt) -> bt
    | _ -> raise Srv_types.Invalid_page_parameters
;;

let create_not_found_page_parameters missing_resource =
  Not_found_page_parameters missing_resource
;;

let default_not_found_page_parameters =
  create_not_found_page_parameters \"\"
;;

let get_missing_resource parameters =
  match parameters with
      Not_found_page_parameters mr -> mr
    | _ -> raise Srv_types.Invalid_page_parameters
;;\n\n" |>

    (* The to_field_collection function *)
    FC.append_gen "let to_field_collection parameters =
  let fc = Xnc_field_collection.create \"Parameters\" in
  let _ = match parameters with
      No_parameters ->
        ()
    | Error_page_parameters (error, backtrace) ->
        Xnc_field_collection.register_string fc \"error\" error;
        Xnc_field_collection.register_string fc \"backtrace\" backtrace
    | Not_found_page_parameters resource ->
        Xnc_field_collection.register_string fc \"resource\" resource;\n" |>
    self#fold_on_input (fun page -> page#render_page_parameters_file Serialize_param) |>
    FC.append_gen "  in\n    fc\n;;\n\n" |>

    (* The from_field_collection function *)
    FC.append_gen "let from_field_collection parameters fc =
  match parameters with
      No_parameters as p ->
        p
    | Error_page_parameters _ ->
        let error = Xnc_field_collection.get_string fc \"error\" in
        let backtrace = Xnc_field_collection.get_string fc \"backtrace\" in
          (Error_page_parameters (error, backtrace))
    | Not_found_page_parameters _ ->
        let resource = Xnc_field_collection.get_string fc \"resource\" in
          (Not_found_page_parameters resource)\n" |>
    self#fold_on_input (fun page -> page#render_page_parameters_file Deserialize_param) |>
    FC.append_gen ";;\n\n" |>

    (* The parameter builders *)
    self#fold_on_input (
      fun page comp ->
        page#render_page_parameters_file Params_builder  comp |>
        FC.append_gen "\n"
    ) |>

    (* Flush the content *)
    FC.flush ~annotations:render_line_annotations
end

