(** Ex Nunc syntactic tree validation module
  *
  * This module exports functions that implement the validation of a
  * {!type:Common_types.document}, normally obtained by parsing a channel with
  * the {!Xnunc_lexer.parse_xnunc_lexbuf} function. Validating functions
  * return [unit] and raise a number of exceptions in order to signal a
  * not conformity in the syntactic tree.
  *
  * @author Paolo Donadeo
  * @see <http://ex-nunc.sourceforge.net> http://ex-nunc.sourceforge.net
  * @version 0.2
  *)

val remove_phantom_nodes : Common_types.document -> Common_types.document
(** This function removes from a {!type:Common_types.document} all CDATA blocks
  * containing only white spaces i.e. all CDATA blocks matching this regular
  * expression: ["^[ \n\r\t]*$"].
  *)

val custom_validate_document : Common_types.constraints_table -> Common_types.document -> unit
(** Validates a document, given a table of syntactic contraints. The format of
  * the syntactic table is explained here:
  * {!type:Common_types.constraints_table}.
  *)

val validate_document : Common_types.document -> unit
(** Like {!Tree_validation.custom_validate_document} but syntactic
  * contraints are the default for the Ex Nunc template language.
  *)

val insert_document_defaults : Common_types.document -> Common_types.document
(** This function inspects the whole document tree searching for missing, non
  * mandatory attributes with a default value.
  * 
  * The output tree is equal to the input tree, but missing attribute default
  * values are added.
  *
  * {b Note}: the position of such added attributes is set to (-1, -1).
  *)

(** {!Tree_validation.validate_document} and
  * {!Tree_validation.custom_validate_document} raise the following exceptions:
  * {ul
  * {- {!exception:Common_types.Unknown_element}}
  * {- {!exception:Common_types.Proc_instr_not_allowed}}
  * {- {!exception:Common_types.Cdata_not_allowed}}
  * {- {!exception:Common_types.Illegal_child}}
  * {- {!exception:Common_types.Too_few_children}}
  * {- {!exception:Common_types.Too_many_children}}
  * {- {!exception:Common_types.Missing_attribute}}
  * {- {!exception:Common_types.Unknown_attribute}}
  * {- {!exception:Common_types.Parametric_param_not_allowed}}
  * {- {!exception:Common_types.Invalid_attribute_value}}
  * {- {!exception:Common_types.Attribute_out_of_range}}
  * {- {!exception:Common_types.Invalid_attribute_type}}
  * {- {!exception:Common_types.Too_many_root_elements}}
  * {- {!exception:Common_types.Non_root_element}}
  * }
  *)

