open Common_types


let constraints_table : Common_types.constraints_table =
  [
    (**************************************************************************)
    (*****                        PAGE ELEMENT                            *****)
    (**************************************************************************)
    "page", (* element name *)
    [
      "page-parameters", 0, 1; (* element child with min and max card *)
      "head", 1, 1;
      "body", 1, 1;
      "script", 0, max_int;
    ],
    [ (* Attributes *)
      (* name, type, mandatory, param allowed, default, valid values *)
      (* Ex Nunc attributes *)
      "inherit", String, false, false, Some "Page_modules.Page_behavior.page_behavior_base", None;
      "xnc-version", Float(No_limits), true, false, None , None;
      "type", String, false, false, Some "standard", Some ["standard"; "start"; "error"; "not-found"];
      "debug", Bool, false, false, Some "false", Some ["true"; "false"];

      (* XML related attributes *)
      "xml-version", String, false, false, Some "1.0", None;
      (* encoding validity is checked by the "page_validation" function*)
      "encoding", String, false, false, Some "ISO-8859-1", None;
      "dtd", String, false, false, Some "xhtml1-transitional",
      Some [ "xhtml1-strict";
             "xhtml1-transitional";
             "xhtml1-frameset";
             "xhtml11" ];

      (* XHTML 1.0 attributes *)
      "lang", String, false, true, None, None; (* XHTML 1.0 Spec: %i18n *)
      "dir", String, false, true, None, Some["ltr"; "rtl"];
    ],
    false, (* processing instructions allowed? *)
    false, (* cdata block allowed? *)
    true, (* is a root_element? *)
    Element_validations.page_validation;
    (**************************************************************************)

    (**************************************************************************)
    (*****                        HEAD ELEMENT                            *****)
    (**************************************************************************)
    "head", (* element name *)
    [
    ],
    [ (* Attributes *)
      (* name, type, mandatory, param allowed, default, valid values *)
      "id", String, false, false, None, None;
      "lang", String, false, true, None, None; (* XHTML 1.0 Spec: %i18n *)
      "dir", String, false, true, None, Some["ltr"; "rtl"];
      "profile", String, false, true, None, None; (* XHTML 1.0 HEAD specific attributes *)
    ],
    true, (* processing instructions allowed? *)
    true, (* cdata block allowed? *)
    false, (* is a root_element? *)
    Element_validations.no_validation;
    (**************************************************************************)

    (**************************************************************************)
    (*****                        BODY ELEMENT                            *****)
    (**************************************************************************)
    "body", (* element name *)
    [
      "form", 0, 1;
      "script", 0, max_int
    ],
    [ (* Attributes *)
      (* name, type, mandatory, param allowed, default, valid values *)
      "id", String, false, false, None, None; (* XHTML 1.0 Spec: %coreattrs *)
      "class", String, false, true, None, None;
      "style", String, false, true, None, None;
      "title", String, false, true, None, None;
      "lang", String, false, true, None, None; (* XHTML 1.0 Spec: %i18n *)
      "dir", String, false, true, None, Some["ltr"; "rtl"];
      "onclick", String, false, true, None, None; (* XHTML 1.0 Spec: %events *)
      "ondblclick", String, false, true, None, None;
      "onmousedown", String, false, true, None, None;
      "onmouseup", String, false, true, None, None;
      "onmouseover", String, false, true, None, None;
      "onmousemove", String, false, true, None, None;
      "onmouseout", String, false, true, None, None;
      "onkeypress", String, false, true, None, None;
      "onkeydown", String, false, true, None, None;
      "onkeyup", String, false, true, None, None;
      "onload", String, false, true, None, None; (* XHTML 1.0 BODY specific attributes *)
      "onunload", String, false, true, None, None;
    ],
    true, (* processing instructions allowed? *)
    true, (* cdata block allowed? *)
    false, (* is a root_element? *)
    Element_validations.no_validation;
    (**************************************************************************)

    (**************************************************************************)
    (*****                   PAGE-PARAMETER ELEMENT                       *****)
    (**************************************************************************)
    "page-parameters",
    [
      "variable", 1, max_int
    ],
    [ (* Attributes *)
    ],
    false,
    false,
    false,
    Element_validations.no_validation;
    (**************************************************************************)

    (**************************************************************************)
    (*****                       SCRIPT ELEMENT                           *****)
    (**************************************************************************)
    "script",
    [
    ],
    [ (* Attributes *)
      "scope",
      String, false, false, Some "module", Some ["module"; "class"]
    ],
    false,
    true,
    false,
    Element_validations.no_validation;
    (**************************************************************************)

    (**************************************************************************)
    (*****                      VARIABLE ELEMENT                          *****)
    (**************************************************************************)
    "variable",
    [
    ],
    [ (* Attributes *)
      "name",
      String, true, false, None, None;
      "type",
      String, true, false, None, Some["int"; "float"; "char"; "string"; "bool"];
      "default",
      String, true, false, None, None;
    ],
    false,
    false,
    false,
    Element_validations.no_validation;
    (**************************************************************************)

    (**************************************************************************)
    (*****                        FORM ELEMENT                            *****)
    (**************************************************************************)
    "form", (* element name *)
    [
      "form-data", 0, 1; (* element child with min and max card *)
      "button", 0, max_int;
      "validation-summary", 0, 1;
      "input", 0, max_int;
      "validator", 0, max_int;
    ],
    [ (* Attributes *)
      (* name, type, mandatory, param allowed, default, valid values *)
      "id", String, false, false, None, None; (* XHTML 1.0 Spec: %coreattrs *)
      "class", String, false, true, None, None;
      "style", String, false, true, None, None;
      "title", String, false, true, None, None;
      "lang", String, false, true, None, None; (* XHTML 1.0 Spec: %i18n *)
      "dir", String, false, true, None, Some["ltr"; "rtl"];
      "onclick", String, false, true, None, None; (* XHTML 1.0 Spec: %events *)
      "ondblclick", String, false, true, None, None;
      "onmousedown", String, false, true, None, None;
      "onmouseup", String, false, true, None, None;
      "onmouseover", String, false, true, None, None;
      "onmousemove", String, false, true, None, None;
      "onmouseout", String, false, true, None, None;
      "onkeypress", String, false, true, None, None;
      "onkeydown", String, false, true, None, None;
      "onkeyup", String, false, true, None, None;

      (* XHTML 1.0 FORM specific attributes *)
      (* The "action" and "method" attribute cannot be specified in
       * Ex-nunc, because they are generated by the framework *)
      "enctype", String, false, false, Some "application/x-www-form-urlencoded",
          Some["application/x-www-form-urlencoded"; "multipart/form-data"];
      "onsubmit", String, false, true, None, None;
      "onreset", String, false, true, None, None;
      "accept", String, false, true, None, None;
      "accept-charset", String, false, true, None, None;
    ],
    true, (* processing instructions allowed? *)
    true, (* cdata block allowed? *)
    false, (* is a root_element? *)
    Element_validations.no_validation;
    (**************************************************************************)
    
    (**************************************************************************)
    (*****                       INPUT ELEMENT                            *****)
    (**************************************************************************)
    "input", (* element name *)
    [
    ],
    [ (* Attributes *)
      (* name, type, mandatory, param allowed, default, valid values *)
      "id", String, true, false, None, None; (* XHTML 1.0 Spec: %coreattrs *)
      "class", String, false, true, None, None;
      "style", String, false, true, None, None;
      "title", String, false, true, None, None;
      "lang", String, false, true, None, None; (* XHTML 1.0 Spec: %i18n *)
      "dir", String, false, true, None, Some["ltr"; "rtl"];
      "onclick", String, false, true, None, None; (* XHTML 1.0 Spec: %events *)
      "ondblclick", String, false, true, None, None;
      "onmousedown", String, false, true, None, None;
      "onmouseup", String, false, true, None, None;
      "onmouseover", String, false, true, None, None;
      "onmousemove", String, false, true, None, None;
      "onmouseout", String, false, true, None, None;
      "onkeypress", String, false, true, None, None;
      "onkeydown", String, false, true, None, None;
      "onkeyup", String, false, true, None, None;

      (* XHTML 1.0 Spec: input specific attributes *)
      "type", String, true, true, None, Some["text"; "password"; "checkbox";
                                             "radio"; "submit"; "reset";
                                             "file"; "hidden"; "image";
                                             "button" ];
      "name", String, true, true, None, None;
      (* The standard "value" attribute is missing: the user cannot specify a
       * value, he must use the xnc:form-data element
       *)
      "checked", String, false, true, None, None;
      "disabled", String, false, true, None, None;
      "readonly", String, false, true, None, None;
      "size", String, false, true, None, None;
      "maxlength", String, false, true, None, None;
      "src", String, false, true, None, None;
      "alt", String, false, true, None, None;
      "usemap", String, false, true, None, None;
      "ismap", String, false, true, None, None;
      "tabindex", Int(No_limits), false, true, None, None;
      "accesskey", String, false, true, None, None; (* XHTML 1.0 Spec: %focus *)
      "onfocus", String, false, true, None, None;
      "onblur", String, false, true, None, None;
      "onselect", String, false, true, None, None;
      "onchange", String, false, true, None, None;
      "accept", String, false, true, None, None;
    ],
    false, (* processing instructions allowed? *)
    false, (* cdata block allowed? *)
    false, (* is a root_element? *)
    Element_validations.no_validation;
    (**************************************************************************)

    (**************************************************************************)
    (*****                      FORM-DATA ELEMENT                         *****)
    (**************************************************************************)
    "form-data", (* element name *)
    [
      "field", 1, max_int; (* element child with min and max card *)
    ],
    [ (* Attributes *)
    ],
    false, (* processing instructions allowed? *)
    false, (* cdata block allowed? *)
    false, (* is a root_element? *)
    Element_validations.no_validation;
    (**************************************************************************)

    (**************************************************************************)
    (*****                       FIELD ELEMENT                            *****)
    (**************************************************************************)
    "field", (* element name *)
    [
    ],
    [ (* Attributes *)
      "name",
      String, true, false, None, None;
      "type",
      String, true, false, None, Some["int"; "float"; "char"; "string"; "bool"];
      "default",
      String, false, true, None, None;
    ],
    false, (* processing instructions allowed? *)
    false, (* cdata block allowed? *)
    false, (* is a root_element? *)
    Element_validations.no_validation;
    (**************************************************************************)

    (**************************************************************************)
    (*****                      SESSION ELEMENT                           *****)
    (**************************************************************************)
    "session", (* element name *)
    [
      "variable", 1, 16; (* element child with min and max card *)
    ],
    [ (* Attributes *)
    ],
    false, (* processing instructions allowed? *)
    false, (* cdata block allowed? *)
    true, (* is a root_element? *)
    Element_validations.no_validation;
    (**************************************************************************)

    (**************************************************************************)
    (*****                       BUTTON ELEMENT                           *****)
    (**************************************************************************)
    "button", (* element name *)
    [
      "onclick", 0, 1;
    ],
    [ (* Attributes *)
      (* name, type, mandatory, param allowed, default, valid values *)
      "id", String, true, false, None, None; (* XHTML 1.0 Spec: %coreattrs *)
      "class", String, false, true, None, None;
      "style", String, false, true, None, None;
      "title", String, false, true, None, None;
      "lang", String, false, true, None, None; (* XHTML 1.0 Spec: %i18n *)
      "dir", String, false, true, None, Some["ltr"; "rtl"];
      "onclick", String, false, false, None, None; (* XHTML 1.0 Spec: %events *)
      "ondblclick", String, false, true, None, None;
      "onmousedown", String, false, true, None, None;
      "onmouseup", String, false, true, None, None;
      "onmouseover", String, false, true, None, None;
      "onmousemove", String, false, true, None, None;
      "onmouseout", String, false, true, None, None;
      "onkeypress", String, false, true, None, None;
      "onkeydown", String, false, true, None, None;
      "onkeyup", String, false, true, None, None;
      "accesskey", String, false, true, None, None; (* XHTML 1.0 Spec: %focus *)
      "tabindex", Int(No_limits), false, true, None, None;
      "onfocus", String, false, true, None, None;
      "onblur", String, false, true, None, None;
      "name", String, true, true, None, None; (* XHTML 1.0 Button specific Attributes*)
      "value", String, false, true, None, None;
      "type", String, false, true, Some "submit", Some["button"; "submit"; "reset"];
      "disabled", String, false, true, None, Some["disabled"];
    ],
    false, (* processing instructions allowed? *)
    false, (* cdata block allowed? *)
    false, (* is a root_element? *)
    Element_validations.no_validation;
    (**************************************************************************)

    (**************************************************************************)
    (*****                   VALIDATION-SUMMARY ELEMENT                   *****)
    (**************************************************************************)
    "validation-summary", (* element name *)
    [
    ],
    [ (* Attributes *)
      "id", String, true, false, None, None;
      "header", String, true, false, None, None;
    ],
    false, (* processing instructions allowed? *)
    false, (* cdata block allowed? *)
    false, (* is a root_element? *)
    Element_validations.no_validation;
    (**************************************************************************)

    (**************************************************************************)
    (*****                         ONCLICK ELEMENT                        *****)
    (**************************************************************************)
    "onclick", (* element name *)
    [
      "call", 0, max_int; (* element child with min and max card *)
    ],
    [ (* Attributes *)
    ],
    false, (* processing instructions allowed? *)
    true, (* cdata block allowed? *)
    false, (* is a root_element? *)
    Element_validations.no_validation;
    (**************************************************************************)

    (**************************************************************************)
    (*****                          CALL ELEMENT                          *****)
    (**************************************************************************)
    "call", (* element name *)
    [
      "if-invalid", 0, 1; (* element child with min and max card *)
    ],
    [ (* Attributes *)
      (* name, type, mandatory, param allowed, default, valid values *)
      "page", String, true, false, None, None;
      "mode", String, false, false, Some "post", Some["post"; "cgi"; "get"];
    ],
    false, (* processing instructions allowed? *)
    true, (* cdata block allowed? *)
    false, (* is a root_element? *)
    Element_validations.no_validation;
    (**************************************************************************)

    (**************************************************************************)
    (*****                       IF-INVALID ELEMENT                       *****)
    (**************************************************************************)
    "if-invalid", (* element name *)
    [
    ],
    [ (* Attributes *)
      (* name, type, mandatory, param allowed, default, valid values *)
      "page", String, true, false, None, None;
    ],
    false, (* processing instructions allowed? *)
    true, (* cdata block allowed? *)
    false, (* is a root_element? *)
    Element_validations.no_validation;
    (**************************************************************************)

    (**************************************************************************)
    (*****                      VALIDATOR ELEMENT                         *****)
    (**************************************************************************)
    "validator", (* element name *)
    [
      "validator-param", 0, max_int; (* element child with min and max card *)
    ],
    [ (* Attributes *)
      (* name, type, mandatory, param allowed, default, valid values *)
      "class", String, true, false, None, None;
      "field-id", String, true, false, None, None;
    ],
    false, (* processing instructions allowed? *)
    false, (* cdata block allowed? *)
    false, (* is a root_element? *)
    Element_validations.no_validation;
    (**************************************************************************)

    (**************************************************************************)
    (*****                   VALIDATOR-PARAM ELEMENT                      *****)
    (**************************************************************************)
    "validator-param", (* element name *)
    [
    ],
    [ (* Attributes *)
      (* name, type, mandatory, param allowed, default, valid values *)
      "label", String, false, false, None, None;
    ],
    false, (* processing instructions allowed? *)
    true,  (* cdata block allowed? *)
    false, (* is a root_element? *)
    Element_validations.no_validation;
    (**************************************************************************)
  ]

