(** Ex Nunc lexer module
  *
  * This module implements the low level parser for the .xnunc template file.
  * Functions provided by this module are used by every module that want to
  * scan a xnunc template file.
  *
  * @author Paolo Donadeo
  * @see <http://ex-nunc.sourceforge.net> http://ex-nunc.sourceforge.net
  * @version 0.2
  *)

exception Parse_error of string
(** The [Parse_error] exception is raised when the parser discover a syntax
  * error in the source file of channel. The [string] is a message reporting the
  * reason of the error and the position inside the stream.
  *)

val parse_xnunc_lexbuf : Lexing.lexbuf -> Common_types.document
(** This is the lexing function: provided a standard {!type:Lexing.lexbuf}
  * value, the function parses it and returns a [document]; if the lexer has
  * complains about what you wrote, [parse_xnunc_lexbuf] raises a
  * {!exception:Xnunc_lexer.Parse_error} exception.
  *)

val parsed_block_compare : Common_types.parsed_block -> Common_types.parsed_block -> bool
(** Compare two parsed blocks and return [true] if they are perfectly equal. The
  * function considers equivalent blocks that have same names, attributes,
  * children {b position}. So two Ex Nunc files syntactically equivalent but
  * with some different {i white spaces} are considered different.
  *)

val document_compare : Common_types.document -> Common_types.document -> bool
(** Like [parsed_block_compare] but compares two document.
  *
  *)

val print_parsed_block : Common_types.parsed_block -> string
(** Prints a block in a string. {b Note}: this is not to be considered a {i
  * pretty print} but only an utility for debug purposes.
  *)

val print_document : Common_types.document -> string
(** Like [print_parsed_block] but prints a [document].
  *)

